@@ -74,10 +74,6 @@
 
 #include "clutter/clutter-actor-private.h"
 
-#ifdef CLUTTER_WINDOWING_X11
-#include <clutter/x11/clutter-x11.h>
-#endif
-
 #include <math.h>
 
 #include "cally-actor.h"
@@ -1090,7 +1086,7 @@ cally_actor_add_action_full (CallyActor          *cally_actor,
 
   priv = cally_actor->priv;
 
-  info = g_slice_new (CallyActorActionInfo);
+  info = g_new0 (CallyActorActionInfo, 1);
   info->name = g_strdup (action_name);
   info->description = g_strdup (action_description);
   info->keybinding = g_strdup (action_keybinding);
@@ -1195,5 +1191,5 @@ _cally_actor_destroy_action_info (gpointer action_info,
   if (info->notify)
     info->notify (info->user_data);
 
-  g_slice_free (CallyActorActionInfo, info);
+  g_free (info);
 }
