@@ -387,44 +387,6 @@ typedef enum
   CLUTTER_MODIFIER_MASK = 0x5c001fff
 } ClutterModifierType;
 
-/**
- * ClutterKeyboardA11yFlags:
- * @CLUTTER_A11Y_KEYBOARD_ENABLED:
- * @CLUTTER_A11Y_TIMEOUT_ENABLED:
- * @CLUTTER_A11Y_MOUSE_KEYS_ENABLED:
- * @CLUTTER_A11Y_SLOW_KEYS_ENABLED:
- * @CLUTTER_A11Y_SLOW_KEYS_BEEP_PRESS:
- * @CLUTTER_A11Y_SLOW_KEYS_BEEP_ACCEPT:
- * @CLUTTER_A11Y_SLOW_KEYS_BEEP_REJECT:
- * @CLUTTER_A11Y_BOUNCE_KEYS_ENABLED:
- * @CLUTTER_A11Y_BOUNCE_KEYS_BEEP_REJECT:
- * @CLUTTER_A11Y_TOGGLE_KEYS_ENABLED:
- * @CLUTTER_A11Y_STICKY_KEYS_ENABLED:
- * @CLUTTER_A11Y_STICKY_KEYS_TWO_KEY_OFF:
- * @CLUTTER_A11Y_STICKY_KEYS_BEEP:
- * @CLUTTER_A11Y_FEATURE_STATE_CHANGE_BEEP:
- *
- * Keyboard accessibility features applied to a ClutterInputDevice keyboard.
- *
- */
-typedef enum
-{
-  CLUTTER_A11Y_KEYBOARD_ENABLED          = 1 << 0,
-  CLUTTER_A11Y_TIMEOUT_ENABLED           = 1 << 1,
-  CLUTTER_A11Y_MOUSE_KEYS_ENABLED        = 1 << 2,
-  CLUTTER_A11Y_SLOW_KEYS_ENABLED         = 1 << 3,
-  CLUTTER_A11Y_SLOW_KEYS_BEEP_PRESS      = 1 << 4,
-  CLUTTER_A11Y_SLOW_KEYS_BEEP_ACCEPT     = 1 << 5,
-  CLUTTER_A11Y_SLOW_KEYS_BEEP_REJECT     = 1 << 6,
-  CLUTTER_A11Y_BOUNCE_KEYS_ENABLED       = 1 << 7,
-  CLUTTER_A11Y_BOUNCE_KEYS_BEEP_REJECT   = 1 << 8,
-  CLUTTER_A11Y_TOGGLE_KEYS_ENABLED       = 1 << 9,
-  CLUTTER_A11Y_STICKY_KEYS_ENABLED       = 1 << 10,
-  CLUTTER_A11Y_STICKY_KEYS_TWO_KEY_OFF   = 1 << 11,
-  CLUTTER_A11Y_STICKY_KEYS_BEEP          = 1 << 12,
-  CLUTTER_A11Y_FEATURE_STATE_CHANGE_BEEP = 1 << 13,
-} ClutterKeyboardA11yFlags;
-
 /**
  * ClutterPointerA11yFlags:
  * @CLUTTER_A11Y_POINTER_ENABLED:
@@ -840,7 +802,9 @@ typedef enum /*< flags prefix=CLUTTER_EVENT >*/
   CLUTTER_EVENT_NONE              = 0,
   CLUTTER_EVENT_FLAG_SYNTHETIC    = 1 << 0,
   CLUTTER_EVENT_FLAG_INPUT_METHOD = 1 << 1,
-  CLUTTER_EVENT_FLAG_REPEATED     = 1 << 2
+  CLUTTER_EVENT_FLAG_REPEATED     = 1 << 2,
+  CLUTTER_EVENT_FLAG_RELATIVE_MOTION = 1 << 3,
+  CLUTTER_EVENT_FLAG_GRAB_NOTIFY  = 1 << 4,
 } ClutterEventFlags;
 
 /**
@@ -854,9 +818,6 @@ typedef enum /*< flags prefix=CLUTTER_EVENT >*/
  * @CLUTTER_BUTTON_PRESS: Pointer button press event
  * @CLUTTER_BUTTON_RELEASE: Pointer button release event
  * @CLUTTER_SCROLL: Pointer scroll event
- * @CLUTTER_STAGE_STATE: Stage state change event
- * @CLUTTER_DESTROY_NOTIFY: Destroy notification event
- * @CLUTTER_CLIENT_MESSAGE: Client message event
  * @CLUTTER_TOUCH_BEGIN: A new touch event sequence has started;
  *   event added in 1.10
  * @CLUTTER_TOUCH_UPDATE: A touch event sequence has been updated;
@@ -869,6 +830,11 @@ typedef enum /*< flags prefix=CLUTTER_EVENT >*/
  *   determined by its phase field; event added in 1.24
  * @CLUTTER_TOUCHPAD_SWIPE: A swipe gesture event, the current state is
  *   determined by its phase field; event added in 1.24
+ * @CLUTTER_TOUCHPAD_HOLD: A hold gesture event, the current state is
+ *   determined by its phase field. A hold gesture starts when the user places a
+ *   finger on the touchpad and ends when all fingers are lifted. It is
+ *   cancelled when the finger(s) move past a certain threshold.
+ *   Event added in 40.4
  * @CLUTTER_PROXIMITY_IN: A tool entered in proximity to a tablet;
  *   event added in 1.28
  * @CLUTTER_PROXIMITY_OUT: A tool left from the proximity area of a tablet;
@@ -891,15 +857,13 @@ typedef enum /*< prefix=CLUTTER >*/
   CLUTTER_BUTTON_PRESS,
   CLUTTER_BUTTON_RELEASE,
   CLUTTER_SCROLL,
-  CLUTTER_STAGE_STATE,
-  CLUTTER_DESTROY_NOTIFY,
-  CLUTTER_CLIENT_MESSAGE,
   CLUTTER_TOUCH_BEGIN,
   CLUTTER_TOUCH_UPDATE,
   CLUTTER_TOUCH_END,
   CLUTTER_TOUCH_CANCEL,
   CLUTTER_TOUCHPAD_PINCH,
   CLUTTER_TOUCHPAD_SWIPE,
+  CLUTTER_TOUCHPAD_HOLD,
   CLUTTER_PROXIMITY_IN,
   CLUTTER_PROXIMITY_OUT,
   CLUTTER_PAD_BUTTON_PRESS,
@@ -939,27 +903,9 @@ typedef enum /*< prefix=CLUTTER_SCROLL >*/
   CLUTTER_SCROLL_SMOOTH
 } ClutterScrollDirection;
 
-/**
- * ClutterStageState:
- * @CLUTTER_STAGE_STATE_ACTIVATED: Activated mask
- *
- * Stage state masks, used by the #ClutterEvent of type %CLUTTER_STAGE_STATE.
- *
- * Since: 0.4
- */
-typedef enum
-{
-  CLUTTER_STAGE_STATE_ACTIVATED        = (1 << 3)
-} ClutterStageState;
-
 /**
  * ClutterFeatureFlags:
- * @CLUTTER_FEATURE_STAGE_STATIC: Set if stage size if fixed (i.e framebuffer)
- * @CLUTTER_FEATURE_STAGE_CURSOR: Set if stage has a graphical cursor.
  * @CLUTTER_FEATURE_SHADERS_GLSL: Set if the backend supports GLSL shaders.
- * @CLUTTER_FEATURE_OFFSCREEN: Set if the backend supports offscreen rendering.
- * @CLUTTER_FEATURE_STAGE_MULTIPLE: Set if multiple stages are supported.
- * @CLUTTER_FEATURE_SWAP_EVENTS: Set if the GLX_INTEL_swap_event is supported.
  *
  * Runtime flags indicating specific features available via Clutter window
  * system and graphics backend.
@@ -968,12 +914,7 @@ typedef enum
  */
 typedef enum
 {
-  CLUTTER_FEATURE_STAGE_STATIC           = (1 << 6),
-  CLUTTER_FEATURE_STAGE_CURSOR           = (1 << 8),
   CLUTTER_FEATURE_SHADERS_GLSL           = (1 << 9),
-  CLUTTER_FEATURE_OFFSCREEN              = (1 << 10),
-  CLUTTER_FEATURE_STAGE_MULTIPLE         = (1 << 11),
-  CLUTTER_FEATURE_SWAP_EVENTS            = (1 << 12)
 } ClutterFeatureFlags;
 
 /**
@@ -1088,6 +1029,20 @@ typedef enum
   CLUTTER_INPUT_AXIS_LAST
 } ClutterInputAxis;
 
+typedef enum
+{
+  CLUTTER_INPUT_AXIS_FLAG_NONE = 0,
+  CLUTTER_INPUT_AXIS_FLAG_X = 1 << CLUTTER_INPUT_AXIS_X,
+  CLUTTER_INPUT_AXIS_FLAG_Y = 1 << CLUTTER_INPUT_AXIS_Y,
+  CLUTTER_INPUT_AXIS_FLAG_PRESSURE = 1 << CLUTTER_INPUT_AXIS_PRESSURE,
+  CLUTTER_INPUT_AXIS_FLAG_XTILT = 1 << CLUTTER_INPUT_AXIS_XTILT,
+  CLUTTER_INPUT_AXIS_FLAG_YTILT = 1 << CLUTTER_INPUT_AXIS_YTILT,
+  CLUTTER_INPUT_AXIS_FLAG_WHEEL = 1 << CLUTTER_INPUT_AXIS_WHEEL,
+  CLUTTER_INPUT_AXIS_FLAG_DISTANCE = 1 << CLUTTER_INPUT_AXIS_DISTANCE,
+  CLUTTER_INPUT_AXIS_FLAG_ROTATION = 1 << CLUTTER_INPUT_AXIS_ROTATION,
+  CLUTTER_INPUT_AXIS_FLAG_SLIDER = 1 << CLUTTER_INPUT_AXIS_SLIDER,
+} ClutterInputAxisFlags;
+
 /**
  * ClutterSnapEdge:
  * @CLUTTER_SNAP_EDGE_TOP: the top edge
@@ -1481,24 +1436,6 @@ typedef enum
   CLUTTER_STEP_MODE_END
 } ClutterStepMode;
 
-/**
- * ClutterZoomAxis:
- * @CLUTTER_ZOOM_X_AXIS: Scale only on the X axis
- * @CLUTTER_ZOOM_Y_AXIS: Scale only on the Y axis
- * @CLUTTER_ZOOM_BOTH: Scale on both axis
- *
- * The axis of the constraint that should be applied by the
- * zooming action.
- *
- * Since: 1.12
- */
-typedef enum /*< prefix=CLUTTER_ZOOM >*/
-{
-  CLUTTER_ZOOM_X_AXIS,
-  CLUTTER_ZOOM_Y_AXIS,
-  CLUTTER_ZOOM_BOTH
-} ClutterZoomAxis;
-
 /**
  * ClutterGestureTriggerEdge:
  * @CLUTTER_GESTURE_TRIGGER_EDGE_NONE: Tell #ClutterGestureAction that
@@ -1637,9 +1574,10 @@ typedef enum
 
 typedef enum
 {
-  CLUTTER_INPUT_DEVICE_MAPPING_ABSOLUTE,
-  CLUTTER_INPUT_DEVICE_MAPPING_RELATIVE,
-} ClutterInputDeviceMapping;
+  CLUTTER_PAD_FEATURE_BUTTON,
+  CLUTTER_PAD_FEATURE_RING,
+  CLUTTER_PAD_FEATURE_STRIP,
+} ClutterInputDevicePadFeature;
 
 typedef enum
 {
@@ -1679,6 +1617,27 @@ typedef enum
   CLUTTER_INPUT_PANEL_STATE_TOGGLE,
 } ClutterInputPanelState;
 
+typedef enum
+{
+  CLUTTER_PREEDIT_RESET_CLEAR,
+  CLUTTER_PREEDIT_RESET_COMMIT,
+} ClutterPreeditResetMode;
+
+typedef enum
+{
+  CLUTTER_PHASE_CAPTURE,
+  CLUTTER_PHASE_BUBBLE,
+} ClutterEventPhase;
+
+typedef enum
+{
+  CLUTTER_GRAB_STATE_NONE = 0,
+  CLUTTER_GRAB_STATE_POINTER = 1 << 0,
+  CLUTTER_GRAB_STATE_KEYBOARD = 1 << 1,
+  CLUTTER_GRAB_STATE_ALL = (CLUTTER_GRAB_STATE_POINTER |
+                            CLUTTER_GRAB_STATE_KEYBOARD),
+} ClutterGrabState;
+
 G_END_DECLS
 
 #endif /* __CLUTTER_ENUMS_H__ */
