@@ -433,6 +433,11 @@ clutter_event_get_position (const ClutterEvent *event,
       graphene_point_init (position, event->touchpad_swipe.x,
                            event->touchpad_swipe.y);
       break;
+
+    case CLUTTER_TOUCHPAD_HOLD:
+      graphene_point_init (position, event->touchpad_hold.x,
+                           event->touchpad_hold.y);
+      break;
     }
 
 }
@@ -512,6 +517,11 @@ clutter_event_set_coords (ClutterEvent *event,
       event->touchpad_swipe.x = x;
       event->touchpad_swipe.y = y;
       break;
+
+    case CLUTTER_TOUCHPAD_HOLD:
+      event->touchpad_hold.x = x;
+      event->touchpad_hold.y = y;
+      break;
     }
 }
 
@@ -531,7 +541,8 @@ clutter_event_get_source (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, NULL);
 
-  return event->any.source;
+  return clutter_stage_get_event_actor (clutter_event_get_stage (event),
+                                        event);
 }
 
 /**
@@ -547,10 +558,6 @@ void
 clutter_event_set_source (ClutterEvent *event,
                           ClutterActor *actor)
 {
-  g_return_if_fail (event != NULL);
-  g_return_if_fail (actor == NULL || CLUTTER_IS_ACTOR (actor));
-
-  event->any.source = actor;
 }
 
 /**
@@ -823,27 +830,6 @@ clutter_event_set_button (ClutterEvent *event,
   event->button.button = button;
 }
 
-/**
- * clutter_event_get_click_count:
- * @event: a #ClutterEvent of type %CLUTTER_BUTTON_PRESS or
- *   of type %CLUTTER_BUTTON_RELEASE
- *
- * Retrieves the number of clicks of @event
- *
- * Return value: the click count
- *
- * Since: 1.0
- */
-guint32
-clutter_event_get_click_count (const ClutterEvent *event)
-{
-  g_return_val_if_fail (event != NULL, 0);
-  g_return_val_if_fail (event->type == CLUTTER_BUTTON_PRESS ||
-                        event->type == CLUTTER_BUTTON_RELEASE, 0);
-
-  return event->button.click_count;
-}
-
 /* keys */
 
 /**
@@ -1088,6 +1074,7 @@ clutter_event_set_device (ClutterEvent       *event,
 
     case CLUTTER_TOUCHPAD_PINCH:
     case CLUTTER_TOUCHPAD_SWIPE:
+    case CLUTTER_TOUCHPAD_HOLD:
       /* Rely on priv data for these */
       break;
 
@@ -1185,6 +1172,7 @@ clutter_event_get_device (const ClutterEvent *event)
 
     case CLUTTER_TOUCHPAD_PINCH:
     case CLUTTER_TOUCHPAD_SWIPE:
+    case CLUTTER_TOUCHPAD_HOLD:
       /* Rely on priv data for these */
       break;
 
@@ -1649,6 +1637,7 @@ clutter_event_get_axes (const ClutterEvent *event,
 
     case CLUTTER_TOUCHPAD_PINCH:
     case CLUTTER_TOUCHPAD_SWIPE:
+    case CLUTTER_TOUCHPAD_HOLD:
     case CLUTTER_PAD_BUTTON_PRESS:
     case CLUTTER_PAD_BUTTON_RELEASE:
     case CLUTTER_PAD_STRIP:
@@ -1894,12 +1883,15 @@ clutter_event_get_touchpad_gesture_finger_count (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_TOUCHPAD_SWIPE ||
-                        event->type == CLUTTER_TOUCHPAD_PINCH, 0);
+                        event->type == CLUTTER_TOUCHPAD_PINCH ||
+                        event->type == CLUTTER_TOUCHPAD_HOLD, 0);
 
   if (event->type == CLUTTER_TOUCHPAD_SWIPE)
     return event->touchpad_swipe.n_fingers;
   else if (event->type == CLUTTER_TOUCHPAD_PINCH)
     return event->touchpad_pinch.n_fingers;
+  else if (event->type == CLUTTER_TOUCHPAD_HOLD)
+    return event->touchpad_hold.n_fingers;
 
   return 0;
 }
@@ -1958,12 +1950,15 @@ clutter_event_get_gesture_phase (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_TOUCHPAD_PINCH ||
-                        event->type == CLUTTER_TOUCHPAD_SWIPE, 0);
+                        event->type == CLUTTER_TOUCHPAD_SWIPE ||
+                        event->type == CLUTTER_TOUCHPAD_HOLD, 0);
 
   if (event->type == CLUTTER_TOUCHPAD_PINCH)
     return event->touchpad_pinch.phase;
   else if (event->type == CLUTTER_TOUCHPAD_SWIPE)
     return event->touchpad_swipe.phase;
+  else if (event->type == CLUTTER_TOUCHPAD_HOLD)
+    return event->touchpad_hold.phase;
 
   /* Shouldn't ever happen */
   return CLUTTER_TOUCHPAD_GESTURE_PHASE_BEGIN;
@@ -1989,7 +1984,8 @@ clutter_event_get_gesture_motion_delta (const ClutterEvent *event,
 {
   g_return_if_fail (event != NULL);
   g_return_if_fail (event->type == CLUTTER_TOUCHPAD_PINCH ||
-                    event->type == CLUTTER_TOUCHPAD_SWIPE);
+                    event->type == CLUTTER_TOUCHPAD_SWIPE ||
+                    event->type == CLUTTER_TOUCHPAD_HOLD);
 
   if (event->type == CLUTTER_TOUCHPAD_PINCH)
     {
@@ -2005,6 +2001,13 @@ clutter_event_get_gesture_motion_delta (const ClutterEvent *event,
       if (dy)
         *dy = event->touchpad_swipe.dy;
     }
+  else if (event->type == CLUTTER_TOUCHPAD_HOLD)
+    {
+      if (dx)
+        *dx = 0;
+      if (dy)
+        *dy = 0;
+    }
 }
 
 /**
@@ -2026,7 +2029,8 @@ clutter_event_get_gesture_motion_delta_unaccelerated (const ClutterEvent *event,
 {
   g_return_if_fail (event != NULL);
   g_return_if_fail (event->type == CLUTTER_TOUCHPAD_PINCH ||
-                    event->type == CLUTTER_TOUCHPAD_SWIPE);
+                    event->type == CLUTTER_TOUCHPAD_SWIPE ||
+                    event->type == CLUTTER_TOUCHPAD_HOLD);
 
   if (event->type == CLUTTER_TOUCHPAD_PINCH)
     {
@@ -2042,6 +2046,13 @@ clutter_event_get_gesture_motion_delta_unaccelerated (const ClutterEvent *event,
       if (dy)
         *dy = event->touchpad_swipe.dy_unaccel;
     }
+  else if (event->type == CLUTTER_TOUCHPAD_HOLD)
+    {
+      if (dx)
+        *dx = 0;
+      if (dy)
+        *dy = 0;
+    }
 }
 /**
  * clutter_event_get_scroll_source:
