@@ -116,6 +116,7 @@ typedef struct _ClutterCrossingEvent    ClutterCrossingEvent;
 typedef struct _ClutterTouchEvent       ClutterTouchEvent;
 typedef struct _ClutterTouchpadPinchEvent ClutterTouchpadPinchEvent;
 typedef struct _ClutterTouchpadSwipeEvent ClutterTouchpadSwipeEvent;
+typedef struct _ClutterTouchpadHoldEvent ClutterTouchpadHoldEvent;
 typedef struct _ClutterProximityEvent   ClutterProximityEvent;
 typedef struct _ClutterPadButtonEvent   ClutterPadButtonEvent;
 typedef struct _ClutterPadStripEvent    ClutterPadStripEvent;
@@ -140,7 +141,6 @@ struct _ClutterAnyEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 };
 
 /**
@@ -167,7 +167,6 @@ struct _ClutterKeyEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   ClutterModifierType modifier_state;
   guint keyval;
@@ -188,8 +187,6 @@ struct _ClutterKeyEvent
  * @y: event Y coordinate, relative to the stage
  * @modifier_state: button modifiers
  * @button: event button
- * @click_count: number of button presses within the default time
- *   and radius
  * @axes: reserved for future use
  * @device: the device that originated the event. If you want the physical
  * device the event originated from, use clutter_event_get_source_device()
@@ -208,13 +205,11 @@ struct _ClutterButtonEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   gfloat x;
   gfloat y;
   ClutterModifierType modifier_state;
   guint32 button;
-  guint click_count;
   gdouble *axes; /* Future use */
   ClutterInputDevice *device;
   uint32_t evdev_code;
@@ -240,7 +235,6 @@ struct _ClutterProximityEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
   ClutterInputDevice *device;
 };
 
@@ -267,7 +261,6 @@ struct _ClutterCrossingEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   gfloat x;
   gfloat y;
@@ -300,7 +293,6 @@ struct _ClutterMotionEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   gfloat x;
   gfloat y;
@@ -342,7 +334,6 @@ struct _ClutterScrollEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   gfloat x;
   gfloat y;
@@ -392,7 +383,6 @@ struct _ClutterTouchEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   gfloat x;
   gfloat y;
@@ -438,7 +428,6 @@ struct _ClutterTouchpadPinchEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   ClutterTouchpadGesturePhase phase;
   gfloat x;
@@ -481,7 +470,6 @@ struct _ClutterTouchpadSwipeEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   ClutterTouchpadGesturePhase phase;
   guint n_fingers;
@@ -493,13 +481,47 @@ struct _ClutterTouchpadSwipeEvent
   gfloat dy_unaccel;
 };
 
+/**
+ * ClutterTouchpadHoldEvent
+ * @type: event type
+ * @time: event time
+ * @flags: event flags
+ * @stage: event source stage
+ * @source: event source actor (unused)
+ * @phase: the current phase of the gesture
+ * @n_fingers: the number of fingers triggering the swipe
+ * @x: the X coordinate of the pointer, relative to the stage
+ * @y: the Y coordinate of the pointer, relative to the stage
+ *
+ * Used for touchpad hold gesture events. The current state of the
+ * gesture will be determined by the @phase field.
+ *
+ * A hold gesture starts when the user places one or many fingers on the
+ * touchpad and ends when all fingers are lifted. It is cancelled when the
+ * finger(s) move past a certain threshold.
+ * Unlike swipe and pinch, @phase can only be
+ * CLUTTER_TOUCHPAD_GESTURE_PHASE_BEGIN, CLUTTER_TOUCHPAD_GESTURE_PHASE_END and
+ * CLUTTER_TOUCHPAD_GESTURE_PHASE_CANCEL.
+ */
+struct _ClutterTouchpadHoldEvent
+{
+  ClutterEventType type;
+  guint32 time;
+  ClutterEventFlags flags;
+  ClutterStage *stage;
+
+  ClutterTouchpadGesturePhase phase;
+  uint32_t n_fingers;
+  float x;
+  float y;
+};
+
 struct _ClutterPadButtonEvent
 {
   ClutterEventType type;
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   guint32 button;
   guint32 group;
@@ -513,7 +535,6 @@ struct _ClutterPadStripEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   ClutterInputDevice *device;
   ClutterInputDevicePadSource strip_source;
@@ -529,7 +550,6 @@ struct _ClutterPadRingEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   ClutterInputDevice *device;
   ClutterInputDevicePadSource ring_source;
@@ -545,7 +565,6 @@ struct _ClutterDeviceEvent
   guint32 time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   ClutterInputDevice *device;
 };
@@ -556,7 +575,6 @@ struct _ClutterIMEvent
   uint32_t time;
   ClutterEventFlags flags;
   ClutterStage *stage;
-  ClutterActor *source;
 
   char *text;
   int32_t offset;
@@ -585,6 +603,7 @@ union _ClutterEvent
   ClutterTouchEvent touch;
   ClutterTouchpadPinchEvent touchpad_pinch;
   ClutterTouchpadSwipeEvent touchpad_swipe;
+  ClutterTouchpadHoldEvent touchpad_hold;
   ClutterProximityEvent proximity;
   ClutterPadButtonEvent pad_button;
   ClutterPadStripEvent pad_strip;
@@ -681,10 +700,10 @@ void                    clutter_event_set_device_tool           (ClutterEvent
 CLUTTER_EXPORT
 ClutterInputDeviceTool *clutter_event_get_device_tool           (const ClutterEvent     *event);
 
-CLUTTER_EXPORT
+CLUTTER_DEPRECATED_FOR(clutter_stage_get_event_actor)
 void                    clutter_event_set_source                (ClutterEvent           *event,
                                                                  ClutterActor           *actor);
-CLUTTER_EXPORT
+CLUTTER_DEPRECATED
 ClutterActor *          clutter_event_get_source                (const ClutterEvent     *event);
 CLUTTER_EXPORT
 void                    clutter_event_set_stage                 (ClutterEvent           *event,
@@ -740,8 +759,6 @@ void                    clutter_event_set_button                (ClutterEvent
 CLUTTER_EXPORT
 guint32                 clutter_event_get_button                (const ClutterEvent     *event);
 CLUTTER_EXPORT
-guint                   clutter_event_get_click_count           (const ClutterEvent     *event);
-CLUTTER_EXPORT
 void                    clutter_event_set_related               (ClutterEvent           *event,
                                                                  ClutterActor           *actor);
 CLUTTER_EXPORT
