@@ -33,8 +33,6 @@
  *
  * It is possible to ask whether Clutter has support for specific features at
  * run-time.
- *
- * See also cogl_get_features() and #CoglFeatureFlags
  */
 
 #include "clutter-build-config.h"
@@ -60,29 +58,19 @@ typedef struct ClutterFeatures
 static ClutterFeatures* __features = NULL;
 
 static ClutterFeatureFlags
-clutter_features_from_cogl (guint cogl_flags)
+clutter_features_from_cogl (void)
 {
   ClutterFeatureFlags clutter_flags = 0;
   
-  if (cogl_flags & COGL_FEATURE_TEXTURE_YUV)
-    clutter_flags |= CLUTTER_FEATURE_TEXTURE_YUV;
-  
-  if (cogl_flags & COGL_FEATURE_TEXTURE_READ_PIXELS)
-    clutter_flags |= CLUTTER_FEATURE_TEXTURE_READ_PIXELS;
-  
   clutter_flags |= CLUTTER_FEATURE_SHADERS_GLSL;
   
-  if (cogl_flags & COGL_FEATURE_OFFSCREEN)
-    clutter_flags |= CLUTTER_FEATURE_OFFSCREEN;
-  
   return clutter_flags;
 }
 
 gboolean
-_clutter_feature_init (GError **error)
+clutter_feature_init (ClutterMainContext  *context,
+                      GError             **error)
 {
-  ClutterMainContext *context;
-
   CLUTTER_NOTE (MISC, "checking features");
 
   if (!__features)
@@ -95,14 +83,11 @@ _clutter_feature_init (GError **error)
   if (__features->features_set)
     return TRUE;
 
-  context = _clutter_context_get_default ();
-
   /* makes sure we have a GL context; if we have, this is a no-op */
   if (!_clutter_backend_create_context (context->backend, error))
     return FALSE;
 
-  __features->flags = (clutter_features_from_cogl (cogl_get_features ())
-                    | _clutter_backend_get_features (context->backend));
+  __features->flags = clutter_features_from_cogl ();
 
   __features->features_set = TRUE;
 
