@@ -51,30 +51,30 @@ static CoglPipeline *default_color_pipeline   = NULL;
 static CoglPipeline *default_texture_pipeline = NULL;
 
 /*< private >
- * _clutter_paint_node_init_types:
+ * clutter_paint_node_init_types:
  *
  * Initializes the required types for ClutterPaintNode subclasses
  */
 void
-_clutter_paint_node_init_types (void)
+clutter_paint_node_init_types (ClutterBackend *clutter_backend)
 {
-  CoglContext *ctx;
+  CoglContext *cogl_context;
   CoglColor cogl_color;
   GType node_type G_GNUC_UNUSED;
 
   if (G_LIKELY (default_color_pipeline != NULL))
     return;
 
-  ctx = clutter_backend_get_cogl_context (clutter_get_default_backend ());
+  cogl_context = clutter_backend_get_cogl_context (clutter_backend);
 
   node_type = clutter_paint_node_get_type ();
 
   cogl_color_init_from_4f (&cogl_color, 1.0, 1.0, 1.0, 1.0);
 
-  default_color_pipeline = cogl_pipeline_new (ctx);
+  default_color_pipeline = cogl_pipeline_new (cogl_context);
   cogl_pipeline_set_color (default_color_pipeline, &cogl_color);
 
-  default_texture_pipeline = cogl_pipeline_new (ctx);
+  default_texture_pipeline = cogl_pipeline_new (cogl_context);
   cogl_pipeline_set_layer_null_texture (default_texture_pipeline, 0);
   cogl_pipeline_set_color (default_texture_pipeline, &cogl_color);
   cogl_pipeline_set_layer_wrap_mode (default_texture_pipeline, 0,
@@ -484,9 +484,9 @@ clutter_pipeline_node_draw (ClutterPaintNode    *node,
           break;
 
         case PAINT_OP_PRIMITIVE:
-          cogl_framebuffer_draw_primitive (fb,
-                                           pnode->pipeline,
-                                           op->op.primitive);
+          cogl_primitive_draw (op->op.primitive,
+                               fb,
+                               pnode->pipeline);
           break;
         }
     }
@@ -1467,9 +1467,9 @@ clutter_layer_node_post_draw (ClutterPaintNode    *node,
           break;
 
         case PAINT_OP_PRIMITIVE:
-          cogl_framebuffer_draw_primitive (fb,
-                                           lnode->pipeline,
-                                           op->op.primitive);
+          cogl_primitive_draw (op->op.primitive,
+                               fb,
+                               lnode->pipeline);
           break;
         }
     }
@@ -1551,29 +1551,29 @@ clutter_layer_node_new (const graphene_matrix_t *projection,
                         float                    height,
                         guint8                   opacity)
 {
-  ClutterLayerNode *res;
+  ClutterLayerNode *lnode;
   CoglContext *context;
   CoglTexture2D *tex_2d;
   CoglTexture *texture;
   CoglColor color;
-  CoglOffscreen *offscreen;
+  g_autoptr (CoglOffscreen) offscreen = NULL;
   g_autoptr (GError) error = NULL;
 
-  res = _clutter_paint_node_create (CLUTTER_TYPE_LAYER_NODE);
+  lnode = _clutter_paint_node_create (CLUTTER_TYPE_LAYER_NODE);
 
-  res->needs_fbo_setup = TRUE;
-  res->projection = *projection;
-  res->viewport = *viewport;
-  res->fbo_width = width;
-  res->fbo_height = height;
-  res->opacity = opacity;
+  lnode->needs_fbo_setup = TRUE;
+  lnode->projection = *projection;
+  lnode->viewport = *viewport;
+  lnode->fbo_width = width;
+  lnode->fbo_height = height;
+  lnode->opacity = opacity;
 
   /* the texture backing the FBO */
   context = clutter_backend_get_cogl_context (clutter_get_default_backend ());
 
   tex_2d = cogl_texture_2d_new_with_size (context,
-                                          MAX (res->fbo_width, 1),
-                                          MAX (res->fbo_height, 1));
+                                          MAX (lnode->fbo_width, 1),
+                                          MAX (lnode->fbo_height, 1));
   texture = COGL_TEXTURE (tex_2d);
   cogl_texture_set_premultiplied (texture, TRUE);
 
@@ -1582,11 +1582,11 @@ clutter_layer_node_new (const graphene_matrix_t *projection,
     {
       g_warning ("Unable to create an allocate paint node offscreen: %s",
                  error->message);
-      g_object_unref (offscreen);
-      goto out;
+      cogl_object_unref (texture);
+      return NULL;
     }
 
-  res->offscreen = COGL_FRAMEBUFFER (offscreen);
+  lnode->offscreen = COGL_FRAMEBUFFER (g_steal_pointer (&offscreen));
 
   cogl_color_init_from_4ub (&color, opacity, opacity, opacity, opacity);
 
@@ -1594,17 +1594,16 @@ clutter_layer_node_new (const graphene_matrix_t *projection,
    * interpolation filters because the texture is always
    * going to be painted at a 1:1 texel:pixel ratio
    */
-  res->pipeline = cogl_pipeline_copy (default_texture_pipeline);
-  cogl_pipeline_set_layer_filters (res->pipeline, 0,
+  lnode->pipeline = cogl_pipeline_copy (default_texture_pipeline);
+  cogl_pipeline_set_layer_filters (lnode->pipeline, 0,
                                    COGL_PIPELINE_FILTER_NEAREST,
                                    COGL_PIPELINE_FILTER_NEAREST);
-  cogl_pipeline_set_layer_texture (res->pipeline, 0, texture);
-  cogl_pipeline_set_color (res->pipeline, &color);
+  cogl_pipeline_set_layer_texture (lnode->pipeline, 0, texture);
+  cogl_pipeline_set_color (lnode->pipeline, &color);
 
-out:
   cogl_object_unref (texture);
 
-  return (ClutterPaintNode *) res;
+  return (ClutterPaintNode *) lnode;
 }
 
 /**
