@@ -22,6 +22,7 @@
 #ifndef __CLUTTER_STAGE_PRIVATE_H__
 #define __CLUTTER_STAGE_PRIVATE_H__
 
+#include <clutter/clutter-grab.h>
 #include <clutter/clutter-stage-window.h>
 #include <clutter/clutter-stage.h>
 #include <clutter/clutter-input-device.h>
@@ -31,6 +32,13 @@
 
 G_BEGIN_DECLS
 
+typedef enum
+{
+  CLUTTER_DEVICE_UPDATE_NONE = 0,
+  CLUTTER_DEVICE_UPDATE_EMIT_CROSSING = 1 << 0,
+  CLUTTER_DEVICE_UPDATE_IGNORE_CACHE = 1 << 1,
+} ClutterDeviceUpdateFlags;
+
 /* stage */
 ClutterStageWindow *_clutter_stage_get_default_window    (void);
 
@@ -67,7 +75,8 @@ void                _clutter_stage_maybe_setup_viewport  (ClutterStage
                                                           ClutterStageView      *view);
 void                clutter_stage_maybe_relayout         (ClutterActor          *stage);
 void                clutter_stage_maybe_finish_queue_redraws (ClutterStage      *stage);
-GSList *            clutter_stage_find_updated_devices   (ClutterStage          *stage);
+GSList *            clutter_stage_find_updated_devices   (ClutterStage          *stage,
+                                                          ClutterStageView      *view);
 void                clutter_stage_update_devices         (ClutterStage          *stage,
                                                           GSList                *devices);
 void                clutter_stage_finish_layout          (ClutterStage          *stage);
@@ -81,11 +90,6 @@ void     _clutter_stage_process_queued_events             (ClutterStage *stage);
 void     _clutter_stage_update_input_devices              (ClutterStage *stage);
 gboolean _clutter_stage_has_full_redraw_queued            (ClutterStage *stage);
 
-ClutterActor *_clutter_stage_do_pick (ClutterStage    *stage,
-                                      float            x,
-                                      float            y,
-                                      ClutterPickMode  mode);
-
 ClutterPaintVolume *_clutter_stage_paint_volume_stack_allocate (ClutterStage *stage);
 void                _clutter_stage_paint_volume_stack_free_all (ClutterStage *stage);
 
@@ -134,11 +138,21 @@ void clutter_stage_update_device_entry (ClutterStage         *self,
                                         ClutterInputDevice   *device,
                                         ClutterEventSequence *sequence,
                                         graphene_point_t      coords,
-                                        ClutterActor         *actor);
+                                        ClutterActor         *actor,
+                                        cairo_region_t       *clear_area);
 
 void clutter_stage_remove_device_entry (ClutterStage         *self,
                                         ClutterInputDevice   *device,
                                         ClutterEventSequence *sequence);
+ClutterActor * clutter_stage_pick_and_update_device (ClutterStage             *stage,
+                                                     ClutterInputDevice       *device,
+                                                     ClutterEventSequence     *sequence,
+                                                     ClutterDeviceUpdateFlags  flags,
+                                                     graphene_point_t          point,
+                                                     uint32_t                  time_ms);
+
+void clutter_stage_unlink_grab (ClutterStage *self,
+                                ClutterGrab  *grab);
 
 G_END_DECLS
 
