@@ -28,7 +28,6 @@
 
 #define __CLUTTER_H_INSIDE__
 
-#include "clutter-config.h"
 #include "clutter-types.h"
 
 #include "clutter-action.h"
@@ -63,9 +62,11 @@
 #include "clutter-feature.h"
 #include "clutter-fixed-layout.h"
 #include "clutter-flow-layout.h"
+#include "clutter-frame-clock.h"
+#include "clutter-frame.h"
 #include "clutter-gesture-action.h"
+#include "clutter-grab.h"
 #include "clutter-grid-layout.h"
-#include "clutter-group.h"
 #include "clutter-image.h"
 #include "clutter-input-device.h"
 #include "clutter-input-device-tool.h"
@@ -101,6 +102,7 @@
 #include "clutter-stage-view.h"
 #include "clutter-tap-action.h"
 #include "clutter-text.h"
+#include "clutter-texture-content.h"
 #include "clutter-timeline.h"
 #include "clutter-transition-group.h"
 #include "clutter-transition.h"
