@@ -146,6 +146,7 @@ cogl_context_new (CoglDisplay *display,
       CoglRenderer *renderer = cogl_renderer_new ();
       if (!cogl_renderer_connect (renderer, error))
         {
+          cogl_object_unref (renderer);
           g_free (context);
           return NULL;
         }
@@ -223,8 +224,6 @@ cogl_context_new (CoglDisplay *display,
   context->current_clip_stack_valid = FALSE;
   context->current_clip_stack = NULL;
 
-  context->legacy_backface_culling_enabled = FALSE;
-
   graphene_matrix_init_identity (&context->identity_matrix);
   graphene_matrix_init_identity (&context->y_flip_matrix);
   graphene_matrix_scale (&context->y_flip_matrix, 1, -1, 1);
@@ -276,8 +275,6 @@ cogl_context_new (CoglDisplay *display,
   context->depth_range_near_cache = 0;
   context->depth_range_far_cache = 1;
 
-  context->legacy_depth_test_enabled = FALSE;
-
   context->pipeline_cache = _cogl_pipeline_cache_new ();
 
   for (i = 0; i < COGL_BUFFER_BIND_TARGET_COUNT; i++)
@@ -522,7 +519,7 @@ int64_t
 cogl_context_get_gpu_time_ns (CoglContext *context)
 {
   g_return_val_if_fail (cogl_has_feature (context,
-                                          COGL_FEATURE_ID_GET_GPU_TIME),
+                                          COGL_FEATURE_ID_TIMESTAMP_QUERY),
                         0);
 
   return context->driver_vtable->get_gpu_time_ns (context);
