@@ -107,7 +107,7 @@ G_BEGIN_DECLS
  * The snippets can be added to a pipeline with
  * cogl_pipeline_add_snippet() or
  * cogl_pipeline_add_layer_snippet(). Which function to use depends on
- * which hook the snippet is targetting. The snippets are all
+ * which hook the snippet is targeting. The snippets are all
  * generated in the order they are added to the pipeline. That is, the
  * post strings are executed in the order they are added to the
  * pipeline and the pre strings are executed in reverse order. If any
@@ -345,6 +345,7 @@ typedef struct _CoglSnippet CoglSnippet;
  *
  * Returns: a #GType that can be used with the GLib type system.
  */
+COGL_EXPORT
 GType cogl_snippet_get_gtype (void);
 
 /* Enumeration of all the hook points that a snippet can be attached
@@ -644,7 +645,7 @@ GType cogl_snippet_get_gtype (void);
  * </para>
  * <para>
  * The ‘post’ string in @snippet will be inserted after texture lookup
- * has been preformed. Here the snippet can modify the cogl_texel
+ * has been performed. Here the snippet can modify the cogl_texel
  * variable to alter the returned texel.
  * </para>
  *   </glossdef>
@@ -690,7 +691,7 @@ typedef enum
  * Since: 1.10
  * Stability: Unstable
  */
-CoglSnippet *
+COGL_EXPORT CoglSnippet *
 cogl_snippet_new (CoglSnippetHook hook,
                   const char *declarations,
                   const char *post);
@@ -704,7 +705,7 @@ cogl_snippet_new (CoglSnippetHook hook,
  * Since: 1.10
  * Stability: Unstable
  */
-CoglSnippetHook
+COGL_EXPORT CoglSnippetHook
 cogl_snippet_get_hook (CoglSnippet *snippet);
 
 /**
@@ -719,7 +720,7 @@ cogl_snippet_get_hook (CoglSnippet *snippet);
  * Since: 1.10
  * Stability: Unstable
  */
-gboolean
+COGL_EXPORT gboolean
 cogl_is_snippet (void *object);
 
 /**
@@ -740,7 +741,7 @@ cogl_is_snippet (void *object);
  * Since: 1.10
  * Stability: Unstable
  */
-void
+COGL_EXPORT void
 cogl_snippet_set_declarations (CoglSnippet *snippet,
                                const char *declarations);
 
@@ -754,7 +755,7 @@ cogl_snippet_set_declarations (CoglSnippet *snippet,
  * Since: 1.10
  * Stability: Unstable
  */
-const char *
+COGL_EXPORT const char *
 cogl_snippet_get_declarations (CoglSnippet *snippet);
 
 /**
@@ -774,7 +775,7 @@ cogl_snippet_get_declarations (CoglSnippet *snippet);
  * Since: 1.10
  * Stability: Unstable
  */
-void
+COGL_EXPORT void
 cogl_snippet_set_pre (CoglSnippet *snippet,
                       const char *pre);
 
@@ -788,7 +789,7 @@ cogl_snippet_set_pre (CoglSnippet *snippet,
  * Since: 1.10
  * Stability: Unstable
  */
-const char *
+COGL_EXPORT const char *
 cogl_snippet_get_pre (CoglSnippet *snippet);
 
 /**
@@ -808,7 +809,7 @@ cogl_snippet_get_pre (CoglSnippet *snippet);
  * Since: 1.10
  * Stability: Unstable
  */
-void
+COGL_EXPORT void
 cogl_snippet_set_replace (CoglSnippet *snippet,
                           const char *replace);
 
@@ -822,7 +823,7 @@ cogl_snippet_set_replace (CoglSnippet *snippet,
  * Since: 1.10
  * Stability: Unstable
  */
-const char *
+COGL_EXPORT const char *
 cogl_snippet_get_replace (CoglSnippet *snippet);
 
 /**
@@ -842,7 +843,7 @@ cogl_snippet_get_replace (CoglSnippet *snippet);
  * Since: 1.10
  * Stability: Unstable
  */
-void
+COGL_EXPORT void
 cogl_snippet_set_post (CoglSnippet *snippet,
                        const char *post);
 
@@ -856,7 +857,7 @@ cogl_snippet_set_post (CoglSnippet *snippet,
  * Since: 1.10
  * Stability: Unstable
  */
-const char *
+COGL_EXPORT const char *
 cogl_snippet_get_post (CoglSnippet *snippet);
 
 G_END_DECLS
