@@ -48,7 +48,6 @@
 #include "cogl-bitmap-private.h"
 #include "cogl-atlas-texture-private.h"
 #include "cogl-sub-texture.h"
-#include "cogl-texture-2d-gl.h"
 
 #include "deprecated/cogl-auto-texture.h"
 
@@ -69,66 +68,6 @@ set_auto_mipmap_cb (CoglTexture *sub_texture,
                                           FALSE);
 }
 
-CoglTexture *
-cogl_texture_new_with_size (unsigned int width,
-			    unsigned int height,
-                            CoglTextureFlags flags,
-			    CoglPixelFormat internal_format)
-{
-  CoglTexture *tex;
-  GError *skip_error = NULL;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
-
-  /* First try creating a fast-path non-sliced texture */
-  tex = COGL_TEXTURE (cogl_texture_2d_new_with_size (ctx, width, height));
-
-  _cogl_texture_set_internal_format (tex, internal_format);
-
-  if (!cogl_texture_allocate (tex, &skip_error))
-    {
-      g_error_free (skip_error);
-      skip_error = NULL;
-      cogl_object_unref (tex);
-      tex = NULL;
-    }
-
-  if (!tex)
-    {
-      /* If it fails resort to sliced textures */
-      int max_waste = flags & COGL_TEXTURE_NO_SLICING ? -1 : COGL_TEXTURE_MAX_WASTE;
-      tex = COGL_TEXTURE (cogl_texture_2d_sliced_new_with_size (ctx,
-                                                                width,
-                                                                height,
-                                                                max_waste));
-
-      _cogl_texture_set_internal_format (tex, internal_format);
-    }
-
-  /* NB: This api existed before Cogl introduced lazy allocation of
-   * textures and so we maintain its original synchronous allocation
-   * semantics and return NULL if allocation fails... */
-  if (!cogl_texture_allocate (tex, &skip_error))
-    {
-      g_error_free (skip_error);
-      cogl_object_unref (tex);
-      return NULL;
-    }
-
-  if (tex &&
-      flags & COGL_TEXTURE_NO_AUTO_MIPMAP)
-    {
-      cogl_meta_texture_foreach_in_region (COGL_META_TEXTURE (tex),
-                                           0, 0, 1, 1,
-                                           COGL_PIPELINE_WRAP_MODE_CLAMP_TO_EDGE,
-                                           COGL_PIPELINE_WRAP_MODE_CLAMP_TO_EDGE,
-                                           set_auto_mipmap_cb,
-                                           NULL);
-    }
-
-  return tex;
-}
-
 static CoglTexture *
 _cogl_texture_new_from_data (CoglContext *ctx,
                              int width,
@@ -144,11 +83,12 @@ _cogl_texture_new_from_data (CoglContext *ctx,
   CoglTexture *tex;
 
   g_return_val_if_fail (format != COGL_PIXEL_FORMAT_ANY, NULL);
+  g_return_val_if_fail (cogl_pixel_format_get_n_planes (format) == 1, NULL);
   g_return_val_if_fail (data != NULL, NULL);
 
   /* Rowstride from width if not given */
   if (rowstride == 0)
-    rowstride = width * _cogl_pixel_format_get_bytes_per_pixel (format);
+    rowstride = width * cogl_pixel_format_get_bytes_per_pixel (format, 0);
 
   /* Wrap the data into a bitmap */
   bmp = cogl_bitmap_new_for_data (ctx,
@@ -177,7 +117,7 @@ cogl_texture_new_from_data (int width,
                             int rowstride,
                             const uint8_t *data)
 {
-  GError *ignore_error = NULL;
+  g_autoptr (GError) error = NULL;
   CoglTexture *tex;
 
   _COGL_GET_CONTEXT (ctx, NULL);
@@ -188,9 +128,17 @@ cogl_texture_new_from_data (int width,
                                      format, internal_format,
                                      rowstride,
                                      data,
-                                     &ignore_error);
+                                     &error);
   if (!tex)
-    g_error_free (ignore_error);
+    {
+      COGL_NOTE (TEXTURES, "Failed to create texture with size %dx%d and "
+                 "format %s (internal: %s) from data: %s",
+                 width, height,
+                 cogl_pixel_format_to_string (format),
+                 cogl_pixel_format_to_string (internal_format),
+                 error->message);
+      return NULL;
+    }
   return tex;
 }
 
@@ -231,6 +179,15 @@ _cogl_texture_new_from_bitmap (CoglBitmap *bitmap,
 
   if (!cogl_texture_allocate (tex, &internal_error))
     {
+      COGL_NOTE (TEXTURES,
+                 "Failed to allocate texture from bitmap with size "
+                 "%dx%d and format %s (internal: %s), "
+                 "falling back on slicing: %s",
+                 cogl_bitmap_get_width (bitmap),
+                 cogl_bitmap_get_height (bitmap),
+                 cogl_pixel_format_to_string (cogl_bitmap_get_format (bitmap)),
+                 cogl_pixel_format_to_string (internal_format),
+                 internal_error->message);
       g_error_free (internal_error);
       internal_error = NULL;
       cogl_object_unref (tex);
@@ -273,15 +230,20 @@ cogl_texture_new_from_bitmap (CoglBitmap *bitmap,
                               CoglTextureFlags flags,
                               CoglPixelFormat internal_format)
 {
-  GError *ignore_error = NULL;
+  g_autoptr (GError) error = NULL;
+
   CoglTexture *tex =
     _cogl_texture_new_from_bitmap (bitmap,
                                    flags,
                                    internal_format,
                                    FALSE, /* can't convert in-place */
-                                   &ignore_error);
+                                   &error);
   if (!tex)
-    g_error_free (ignore_error);
+    {
+      COGL_NOTE (TEXTURES, "Failed to create texture from bitmap: %s",
+                 error->message);
+      return NULL;
+    }
   return tex;
 }
 
@@ -311,16 +273,3 @@ cogl_texture_new_from_file (const char        *filename,
 
   return texture;
 }
-
-CoglTexture *
-cogl_texture_new_from_sub_texture (CoglTexture *full_texture,
-                                   int sub_x,
-                                   int sub_y,
-                                   int sub_width,
-                                   int sub_height)
-{
-  _COGL_GET_CONTEXT (ctx, NULL);
-  return COGL_TEXTURE (cogl_sub_texture_new (ctx,
-                                             full_texture, sub_x, sub_y,
-                                             sub_width, sub_height));
-}
