@@ -28,6 +28,7 @@
 #include "winsys/cogl-onscreen-egl.h"
 
 #include "cogl-context-private.h"
+#include "cogl-frame-info-private.h"
 #include "cogl-renderer-private.h"
 #include "cogl-trace.h"
 #include "winsys/cogl-winsys-egl-private.h"
@@ -241,6 +242,31 @@ cogl_onscreen_egl_swap_region (CoglOnscreen  *onscreen,
     g_warning ("Error reported by eglSwapBuffersRegion");
 }
 
+static void
+cogl_onscreen_egl_queue_damage_region (CoglOnscreen *onscreen,
+                                       const int    *rectangles,
+                                       int           n_rectangles)
+{
+  CoglOnscreenEgl *onscreen_egl = COGL_ONSCREEN_EGL (onscreen);
+  CoglOnscreenEglPrivate *priv =
+    cogl_onscreen_egl_get_instance_private (onscreen_egl);
+  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
+  CoglContext *context = cogl_framebuffer_get_context (framebuffer);
+  CoglRenderer *renderer = context->display->renderer;
+  CoglRendererEGL *egl_renderer = renderer->winsys;
+
+  g_return_if_fail (n_rectangles > 0);
+
+  if (!egl_renderer->pf_eglSetDamageRegion)
+    return;
+
+  if (egl_renderer->pf_eglSetDamageRegion (egl_renderer->edpy,
+                                           priv->egl_surface,
+                                           rectangles,
+                                           n_rectangles) == EGL_FALSE)
+    g_warning ("Error reported by eglSetDamageRegion");
+}
+
 static void
 cogl_onscreen_egl_swap_buffers_with_damage (CoglOnscreen  *onscreen,
                                             const int     *rectangles,
@@ -269,6 +295,21 @@ cogl_onscreen_egl_swap_buffers_with_damage (CoglOnscreen  *onscreen,
                                         COGL_FRAMEBUFFER (onscreen),
                                         COGL_FRAMEBUFFER_STATE_BIND);
 
+  if (cogl_has_feature (context, COGL_FEATURE_ID_TIMESTAMP_QUERY))
+    {
+      info->gpu_time_before_buffer_swap_ns =
+        cogl_context_get_gpu_time_ns (context);
+    }
+
+  info->cpu_time_before_buffer_swap_us = g_get_monotonic_time ();
+
+  /* Set up a timestamp query for when all rendering will be finished. */
+  if (cogl_has_feature (context, COGL_FEATURE_ID_TIMESTAMP_QUERY))
+    {
+      info->timestamp_query =
+        cogl_framebuffer_create_timestamp_query (COGL_FRAMEBUFFER (onscreen));
+    }
+
   if (n_rectangles && egl_renderer->pf_eglSwapBuffersWithDamage)
     {
       CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
@@ -329,6 +370,8 @@ cogl_onscreen_egl_class_init (CoglOnscreenEglClass *klass)
   object_class->dispose = cogl_onscreen_egl_dispose;
 
   onscreen_class->bind = cogl_onscreen_egl_bind;
+  onscreen_class->queue_damage_region =
+    cogl_onscreen_egl_queue_damage_region;
   onscreen_class->swap_buffers_with_damage =
     cogl_onscreen_egl_swap_buffers_with_damage;
   onscreen_class->swap_region = cogl_onscreen_egl_swap_region;
