@@ -43,13 +43,14 @@
 #include "cogl-renderer-private.h"
 #include "cogl-texture-pixmap-x11-private.h"
 #include "cogl-texture-2d-private.h"
+#include "driver/gl/cogl-texture-2d-gl-private.h"
 #include "cogl-texture-2d.h"
 #include "cogl-poll-private.h"
+#include "winsys/cogl-onscreen-egl.h"
+#include "winsys/cogl-onscreen-xlib.h"
 #include "winsys/cogl-winsys-egl-x11-private.h"
 #include "winsys/cogl-winsys-egl-private.h"
 
-#define COGL_ONSCREEN_X11_EVENT_MASK (StructureNotifyMask | ExposureMask)
-
 static const CoglWinsysEGLVtable _cogl_winsys_egl_vtable;
 
 typedef struct _CoglDisplayXlib
@@ -57,16 +58,12 @@ typedef struct _CoglDisplayXlib
   Window dummy_xwin;
 } CoglDisplayXlib;
 
-typedef struct _CoglOnscreenXlib
-{
-  Window xwin;
-} CoglOnscreenXlib;
-
 #ifdef EGL_KHR_image_pixmap
 typedef struct _CoglTexturePixmapEGL
 {
   EGLImageKHR image;
   CoglTexture *texture;
+  gboolean bind_tex_image_queued;
 } CoglTexturePixmapEGL;
 #endif
 
@@ -78,89 +75,32 @@ find_onscreen_for_xid (CoglContext *context, uint32_t xid)
   for (l = context->framebuffers; l; l = l->next)
     {
       CoglFramebuffer *framebuffer = l->data;
-      CoglOnscreenEGL *egl_onscreen;
-      CoglOnscreenXlib *xlib_onscreen;
+      CoglOnscreen *onscreen;
 
-      if (framebuffer->type != COGL_FRAMEBUFFER_TYPE_ONSCREEN)
+      if (!COGL_IS_ONSCREEN (framebuffer))
         continue;
 
-      egl_onscreen = COGL_ONSCREEN (framebuffer)->winsys;
-      xlib_onscreen = egl_onscreen->platform;
-      if (xlib_onscreen->xwin == (Window)xid)
-        return COGL_ONSCREEN (framebuffer);
+      onscreen = COGL_ONSCREEN (framebuffer);
+      if (cogl_onscreen_xlib_is_for_window (onscreen, (Window) xid))
+        return onscreen;
     }
 
   return NULL;
 }
 
-static void
-flush_pending_resize_notifications_cb (void *data,
-                                       void *user_data)
-{
-  CoglFramebuffer *framebuffer = data;
-
-  if (framebuffer->type == COGL_FRAMEBUFFER_TYPE_ONSCREEN)
-    {
-      CoglOnscreen *onscreen = COGL_ONSCREEN (framebuffer);
-      CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
-
-      if (egl_onscreen->pending_resize_notify)
-        {
-          _cogl_onscreen_notify_resize (onscreen);
-          egl_onscreen->pending_resize_notify = FALSE;
-        }
-    }
-}
-
-static void
-flush_pending_resize_notifications_idle (void *user_data)
-{
-  CoglContext *context = user_data;
-  CoglRenderer *renderer = context->display->renderer;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
-
-  /* This needs to be disconnected before invoking the callbacks in
-   * case the callbacks cause it to be queued again */
-  _cogl_closure_disconnect (egl_renderer->resize_notify_idle);
-  egl_renderer->resize_notify_idle = NULL;
-
-  g_list_foreach (context->framebuffers,
-                  flush_pending_resize_notifications_cb,
-                  NULL);
-}
-
 static void
 notify_resize (CoglContext *context,
                Window drawable,
                int width,
                int height)
 {
-  CoglRenderer *renderer = context->display->renderer;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
-  CoglOnscreen *onscreen = find_onscreen_for_xid (context, drawable);
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
-  CoglOnscreenEGL *egl_onscreen;
+  CoglOnscreen *onscreen;
 
+  onscreen = find_onscreen_for_xid (context, drawable);
   if (!onscreen)
     return;
 
-  egl_onscreen = onscreen->winsys;
-
-  _cogl_framebuffer_winsys_update_size (framebuffer, width, height);
-
-  /* We only want to notify that a resize happened when the
-   * application calls cogl_context_dispatch so instead of immediately
-   * notifying we queue an idle callback */
-  if (!egl_renderer->resize_notify_idle)
-    {
-      egl_renderer->resize_notify_idle =
-        _cogl_poll_renderer_add_idle (renderer,
-                                      flush_pending_resize_notifications_idle,
-                                      context,
-                                      NULL);
-    }
-
-  egl_onscreen->pending_resize_notify = TRUE;
+  cogl_onscreen_xlib_resize (onscreen, width, height);
 }
 
 static CoglFilterReturn
@@ -196,8 +136,9 @@ event_filter_cb (XEvent *xevent, void *data)
   return COGL_FILTER_CONTINUE;
 }
 
-static XVisualInfo *
-get_visual_info (CoglDisplay *display, EGLConfig egl_config)
+XVisualInfo *
+cogl_display_xlib_get_visual_info (CoglDisplay *display,
+                                   EGLConfig    egl_config)
 {
   CoglXlibRenderer *xlib_renderer =
     _cogl_xlib_renderer_get_data (display->renderer);
@@ -254,7 +195,7 @@ _cogl_winsys_renderer_disconnect (CoglRenderer *renderer)
 
   eglTerminate (egl_renderer->edpy);
 
-  g_slice_free (CoglRendererEGL, egl_renderer);
+  g_free (egl_renderer);
 }
 
 static EGLDisplay
@@ -297,7 +238,7 @@ _cogl_winsys_renderer_connect (CoglRenderer *renderer,
   CoglRendererEGL *egl_renderer;
   CoglXlibRenderer *xlib_renderer;
 
-  renderer->winsys = g_slice_new0 (CoglRendererEGL);
+  renderer->winsys = g_new0 (CoglRendererEGL, 1);
   egl_renderer = renderer->winsys;
   xlib_renderer = _cogl_xlib_renderer_get_data (renderer);
 
@@ -319,9 +260,9 @@ error:
 }
 
 static int
-_cogl_winsys_egl_add_config_attributes (CoglDisplay *display,
-                                        CoglFramebufferConfig *config,
-                                        EGLint *attributes)
+_cogl_winsys_egl_add_config_attributes (CoglDisplay                 *display,
+                                        const CoglFramebufferConfig *config,
+                                        EGLint                      *attributes)
 {
   int i = 0;
 
@@ -364,7 +305,7 @@ _cogl_winsys_egl_display_setup (CoglDisplay *display,
   CoglDisplayEGL *egl_display = display->winsys;
   CoglDisplayXlib *xlib_display;
 
-  xlib_display = g_slice_new0 (CoglDisplayXlib);
+  xlib_display = g_new0 (CoglDisplayXlib, 1);
   egl_display->platform = xlib_display;
 
   return TRUE;
@@ -375,7 +316,7 @@ _cogl_winsys_egl_display_destroy (CoglDisplay *display)
 {
   CoglDisplayEGL *egl_display = display->winsys;
 
-  g_slice_free (CoglDisplayXlib, egl_display->platform);
+  g_free (egl_display->platform);
 }
 
 static gboolean
@@ -386,10 +327,6 @@ _cogl_winsys_egl_context_init (CoglContext *context,
                                  event_filter_cb,
                                  context);
 
-  COGL_FLAGS_SET (context->winsys_features,
-                  COGL_WINSYS_FEATURE_MULTIPLE_ONSCREEN,
-                  TRUE);
-
   /* We'll manually handle queueing dirty events in response to
    * Expose events from X */
   COGL_FLAGS_SET (context->private_features,
@@ -407,204 +344,6 @@ _cogl_winsys_egl_context_deinit (CoglContext *context)
                                     context);
 }
 
-static gboolean
-_cogl_winsys_egl_onscreen_init (CoglOnscreen *onscreen,
-                                EGLConfig egl_config,
-                                GError **error)
-{
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
-  CoglContext *context = framebuffer->context;
-  CoglDisplay *display = context->display;
-  CoglRenderer *renderer = display->renderer;
-  CoglRendererEGL *egl_renderer = renderer->winsys;
-  CoglXlibRenderer *xlib_renderer =
-    _cogl_xlib_renderer_get_data (renderer);
-  CoglOnscreenXlib *xlib_onscreen;
-  CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
-  Window xwin;
-
-  /* FIXME: We need to explicitly Select for ConfigureNotify events.
-   * We need to document that for windows we create then toolkits
-   * must be careful not to clear event mask bits that we select.
-   */
-
-    {
-      int width;
-      int height;
-      CoglXlibTrapState state;
-      XVisualInfo *xvisinfo;
-      XSetWindowAttributes xattr;
-      unsigned long mask;
-      int xerror;
-
-      width = cogl_framebuffer_get_width (framebuffer);
-      height = cogl_framebuffer_get_height (framebuffer);
-
-      _cogl_xlib_renderer_trap_errors (display->renderer, &state);
-
-      xvisinfo = get_visual_info (display, egl_config);
-      if (xvisinfo == NULL)
-        {
-          g_set_error (error, COGL_WINSYS_ERROR,
-                       COGL_WINSYS_ERROR_CREATE_ONSCREEN,
-                       "Unable to retrieve the X11 visual of context's "
-                       "fbconfig");
-          return FALSE;
-        }
-
-      /* window attributes */
-      xattr.background_pixel =
-        WhitePixel (xlib_renderer->xdpy,
-                    DefaultScreen (xlib_renderer->xdpy));
-      xattr.border_pixel = 0;
-      /* XXX: is this an X resource that we are leaking‽... */
-      xattr.colormap =
-        XCreateColormap (xlib_renderer->xdpy,
-                         DefaultRootWindow (xlib_renderer->xdpy),
-                         xvisinfo->visual,
-                         AllocNone);
-      xattr.event_mask = COGL_ONSCREEN_X11_EVENT_MASK;
-
-      mask = CWBorderPixel | CWColormap | CWEventMask;
-
-      xwin = XCreateWindow (xlib_renderer->xdpy,
-                            DefaultRootWindow (xlib_renderer->xdpy),
-                            0, 0,
-                            width, height,
-                            0,
-                            xvisinfo->depth,
-                            InputOutput,
-                            xvisinfo->visual,
-                            mask, &xattr);
-
-      XFree (xvisinfo);
-
-      XSync (xlib_renderer->xdpy, False);
-      xerror =
-        _cogl_xlib_renderer_untrap_errors (display->renderer, &state);
-      if (xerror)
-        {
-          char message[1000];
-          XGetErrorText (xlib_renderer->xdpy, xerror,
-                         message, sizeof (message));
-          g_set_error (error, COGL_WINSYS_ERROR,
-                       COGL_WINSYS_ERROR_CREATE_ONSCREEN,
-                       "X error while creating Window for CoglOnscreen: %s",
-                       message);
-          return FALSE;
-        }
-    }
-
-  xlib_onscreen = g_slice_new (CoglOnscreenXlib);
-  egl_onscreen->platform = xlib_onscreen;
-
-  xlib_onscreen->xwin = xwin;
-
-  egl_onscreen->egl_surface =
-    eglCreateWindowSurface (egl_renderer->edpy,
-                            egl_config,
-                            (EGLNativeWindowType) xlib_onscreen->xwin,
-                            NULL);
-
-  return TRUE;
-}
-
-static void
-_cogl_winsys_egl_onscreen_deinit (CoglOnscreen *onscreen)
-{
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
-  CoglContext *context = framebuffer->context;
-  CoglRenderer *renderer = context->display->renderer;
-  CoglXlibRenderer *xlib_renderer =
-    _cogl_xlib_renderer_get_data (renderer);
-  CoglXlibTrapState old_state;
-  CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
-  CoglOnscreenXlib *xlib_onscreen = egl_onscreen->platform;
-
-  _cogl_xlib_renderer_trap_errors (renderer, &old_state);
-
-  if (xlib_onscreen->xwin != None)
-    {
-      XDestroyWindow (xlib_renderer->xdpy, xlib_onscreen->xwin);
-      xlib_onscreen->xwin = None;
-    }
-  else
-    xlib_onscreen->xwin = None;
-
-  XSync (xlib_renderer->xdpy, False);
-
-  if (_cogl_xlib_renderer_untrap_errors (renderer,
-                                         &old_state) != Success)
-    g_warning ("X Error while destroying X window");
-
-  g_slice_free (CoglOnscreenXlib, xlib_onscreen);
-}
-
-static void
-_cogl_winsys_onscreen_set_visibility (CoglOnscreen *onscreen,
-                                      gboolean visibility)
-{
-  CoglContext *context = COGL_FRAMEBUFFER (onscreen)->context;
-  CoglRenderer *renderer = context->display->renderer;
-  CoglXlibRenderer *xlib_renderer =
-    _cogl_xlib_renderer_get_data (renderer);
-  CoglOnscreenEGL *onscreen_egl = onscreen->winsys;
-  CoglOnscreenXlib *xlib_onscreen = onscreen_egl->platform;
-
-  if (visibility)
-    XMapWindow (xlib_renderer->xdpy, xlib_onscreen->xwin);
-  else
-    XUnmapWindow (xlib_renderer->xdpy, xlib_onscreen->xwin);
-}
-
-static void
-_cogl_winsys_onscreen_set_resizable (CoglOnscreen *onscreen,
-                                     gboolean resizable)
-{
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
-  CoglContext *context = framebuffer->context;
-  CoglXlibRenderer *xlib_renderer =
-    _cogl_xlib_renderer_get_data (context->display->renderer);
-  CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
-  CoglOnscreenXlib *xlib_onscreen = egl_onscreen->platform;
-
-  XSizeHints *size_hints = XAllocSizeHints ();
-
-  if (resizable)
-    {
-      /* TODO: Add cogl_onscreen_request_minimum_size () */
-      size_hints->min_width = 1;
-      size_hints->min_height = 1;
-
-      size_hints->max_width = INT_MAX;
-      size_hints->max_height = INT_MAX;
-    }
-  else
-    {
-      int width = cogl_framebuffer_get_width (framebuffer);
-      int height = cogl_framebuffer_get_height (framebuffer);
-
-      size_hints->min_width = width;
-      size_hints->min_height = height;
-
-      size_hints->max_width = width;
-      size_hints->max_height = height;
-    }
-
-  XSetWMNormalHints (xlib_renderer->xdpy, xlib_onscreen->xwin, size_hints);
-
-  XFree (size_hints);
-}
-
-static uint32_t
-_cogl_winsys_onscreen_x11_get_window_xid (CoglOnscreen *onscreen)
-{
-  CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
-  CoglOnscreenXlib *xlib_onscreen = egl_onscreen->platform;
-
-  return xlib_onscreen->xwin;
-}
-
 static gboolean
 _cogl_winsys_egl_context_created (CoglDisplay *display,
                                   GError **error)
@@ -619,7 +358,8 @@ _cogl_winsys_egl_context_created (CoglDisplay *display,
   XSetWindowAttributes attrs;
   const char *error_message;
 
-  xvisinfo = get_visual_info (display, egl_display->egl_config);
+  xvisinfo = cogl_display_xlib_get_visual_info (display,
+                                                egl_display->egl_config);
   if (xvisinfo == NULL)
     {
       error_message = "Unable to find suitable X visual";
@@ -758,6 +498,9 @@ _cogl_winsys_texture_pixmap_x11_create (CoglTexturePixmapX11 *tex_pixmap)
                                         COGL_EGL_IMAGE_FLAG_NONE,
                                         NULL));
 
+  /* The image is initially bound as part of the creation */
+  egl_tex_pixmap->bind_tex_image_queued = FALSE;
+
   tex_pixmap->winsys = egl_tex_pixmap;
 
   return TRUE;
@@ -792,15 +535,42 @@ _cogl_winsys_texture_pixmap_x11_update (CoglTexturePixmapX11 *tex_pixmap,
                                         CoglTexturePixmapStereoMode stereo_mode,
                                         gboolean needs_mipmap)
 {
+  CoglTexturePixmapEGL *egl_tex_pixmap = tex_pixmap->winsys;
+  CoglTexture2D *tex_2d;
+  GError *error = NULL;
+
   if (needs_mipmap)
     return FALSE;
 
+  if (egl_tex_pixmap->bind_tex_image_queued)
+    {
+      COGL_NOTE (TEXTURE_PIXMAP, "Rebinding GLXPixmap for %p", tex_pixmap);
+
+      tex_2d = COGL_TEXTURE_2D (egl_tex_pixmap->texture);
+
+      if (cogl_texture_2d_gl_bind_egl_image (tex_2d,
+                                             egl_tex_pixmap->image,
+                                             &error))
+        {
+          egl_tex_pixmap->bind_tex_image_queued = FALSE;
+        }
+      else
+        {
+          g_warning ("Failed to rebind EGLImage to CoglTexture2D: %s",
+                     error->message);
+          g_error_free (error);
+        }
+    }
+
   return TRUE;
 }
 
 static void
 _cogl_winsys_texture_pixmap_x11_damage_notify (CoglTexturePixmapX11 *tex_pixmap)
 {
+  CoglTexturePixmapEGL *egl_tex_pixmap = tex_pixmap->winsys;
+
+  egl_tex_pixmap->bind_tex_image_queued = TRUE;
 }
 
 static CoglTexture *
@@ -825,11 +595,9 @@ _cogl_winsys_egl_vtable =
     .cleanup_context = _cogl_winsys_egl_cleanup_context,
     .context_init = _cogl_winsys_egl_context_init,
     .context_deinit = _cogl_winsys_egl_context_deinit,
-    .onscreen_init = _cogl_winsys_egl_onscreen_init,
-    .onscreen_deinit = _cogl_winsys_egl_onscreen_deinit
   };
 
-const CoglWinsysVtable *
+COGL_EXPORT const CoglWinsysVtable *
 _cogl_winsys_egl_xlib_get_vtable (void)
 {
   static gboolean vtable_inited = FALSE;
@@ -850,14 +618,6 @@ _cogl_winsys_egl_xlib_get_vtable (void)
       vtable.renderer_connect = _cogl_winsys_renderer_connect;
       vtable.renderer_disconnect = _cogl_winsys_renderer_disconnect;
 
-      vtable.onscreen_set_visibility =
-        _cogl_winsys_onscreen_set_visibility;
-      vtable.onscreen_set_resizable =
-        _cogl_winsys_onscreen_set_resizable;
-
-      vtable.onscreen_x11_get_window_xid =
-        _cogl_winsys_onscreen_x11_get_window_xid;
-
 #ifdef EGL_KHR_image_pixmap
       /* X11 tfp support... */
       /* XXX: instead of having a rather monolithic winsys vtable we could
