@@ -28,6 +28,7 @@ void test_pipeline_uniforms (void);
 void test_snippets (void);
 void test_custom_attributes (void);
 void test_offscreen (void);
+void test_journal_unref_flush (void);
 void test_framebuffer_get_bits (void);
 void test_point_size (void);
 void test_point_size_attribute (void);
@@ -44,7 +45,6 @@ void test_pipeline_shader_state (void);
 void test_gles2_context (void);
 void test_gles2_context_fbo (void);
 void test_gles2_context_copy_tex_image (void);
-void test_euler (void);
 void test_color_hsl (void);
 void test_fence (void);
 void test_texture_no_allocate (void);
