@@ -26,6 +26,15 @@
 G_DEFINE_TYPE_WITH_PRIVATE (MetaBarrier, meta_barrier, G_TYPE_OBJECT)
 G_DEFINE_TYPE (MetaBarrierImpl, meta_barrier_impl, G_TYPE_OBJECT)
 
+G_STATIC_ASSERT ((int) META_BARRIER_DIRECTION_POSITIVE_X ==
+                 (int) META_BORDER_MOTION_DIRECTION_POSITIVE_X);
+G_STATIC_ASSERT ((int) META_BARRIER_DIRECTION_POSITIVE_Y ==
+                 (int) META_BORDER_MOTION_DIRECTION_POSITIVE_Y);
+G_STATIC_ASSERT ((int) META_BARRIER_DIRECTION_NEGATIVE_X ==
+                 (int) META_BORDER_MOTION_DIRECTION_NEGATIVE_X);
+G_STATIC_ASSERT ((int) META_BARRIER_DIRECTION_NEGATIVE_Y ==
+                 (int) META_BORDER_MOTION_DIRECTION_NEGATIVE_Y);
+
 enum
 {
   PROP_0,
