@@ -74,7 +74,6 @@ struct _MetaInputSettingsPrivate
   GSettings *trackball_settings;
   GSettings *keyboard_settings;
   GSettings *keyboard_a11y_settings;
-  GSettings *mouse_a11y_settings;
 
   GList *devices;
   GHashTable *mappable_devices;
@@ -110,15 +109,6 @@ enum
 
 static guint signals[N_SIGNALS] = { 0 };
 
-enum
-{
-  PROP_0,
-  PROP_SEAT,
-  N_PROPS,
-};
-
-static GParamSpec *props[N_PROPS] = { 0 };
-
 static GSList *
 meta_input_settings_get_devices (MetaInputSettings      *settings,
                                  ClutterInputDeviceType  type)
@@ -152,58 +142,14 @@ meta_input_settings_dispose (GObject *object)
   g_clear_object (&priv->trackball_settings);
   g_clear_object (&priv->keyboard_settings);
   g_clear_object (&priv->keyboard_a11y_settings);
-  g_clear_object (&priv->mouse_a11y_settings);
   g_clear_pointer (&priv->mappable_devices, g_hash_table_unref);
   g_clear_pointer (&priv->current_tools, g_hash_table_unref);
 
   g_clear_pointer (&priv->two_finger_devices, g_hash_table_destroy);
-  g_clear_object (&priv->seat);
 
   G_OBJECT_CLASS (meta_input_settings_parent_class)->dispose (object);
 }
 
-static void
-meta_input_settings_set_property (GObject      *object,
-                                  guint         prop_id,
-                                  const GValue *value,
-                                  GParamSpec   *pspec)
-{
-  MetaInputSettings *input_settings = META_INPUT_SETTINGS (object);
-  MetaInputSettingsPrivate *priv =
-    meta_input_settings_get_instance_private (input_settings);
-
-  switch (prop_id)
-    {
-    case PROP_SEAT:
-      priv->seat = g_value_dup_object (value);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-static void
-meta_input_settings_get_property (GObject    *object,
-                                  guint       prop_id,
-                                  GValue     *value,
-                                  GParamSpec *pspec)
-{
-  MetaInputSettings *input_settings = META_INPUT_SETTINGS (object);
-  MetaInputSettingsPrivate *priv =
-    meta_input_settings_get_instance_private (input_settings);
-
-  switch (prop_id)
-    {
-    case PROP_SEAT:
-      g_value_set_object (value, priv->seat);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
 static void
 settings_device_set_bool_setting (MetaInputSettings  *input_settings,
                                   ClutterInputDevice *device,
@@ -1273,97 +1219,6 @@ meta_input_keyboard_a11y_settings_changed (GSettings  *settings,
   load_keyboard_a11y_settings (input_settings);
 }
 
-struct _pointer_a11y_settings_flags_pair {
-  const char *name;
-  ClutterPointerA11yFlags flag;
-} pointer_a11y_settings_flags_pair[] = {
-  { "secondary-click-enabled", CLUTTER_A11Y_SECONDARY_CLICK_ENABLED },
-  { "dwell-click-enabled",     CLUTTER_A11Y_DWELL_ENABLED },
-};
-
-static ClutterPointerA11yDwellDirection
-pointer_a11y_dwell_direction_from_setting (MetaInputSettings *input_settings,
-                                           const char        *key)
-{
-  MetaInputSettingsPrivate *priv = meta_input_settings_get_instance_private (input_settings);
-  GDesktopMouseDwellDirection dwell_gesture_direction;
-
-  dwell_gesture_direction = g_settings_get_enum (priv->mouse_a11y_settings, key);
-  switch (dwell_gesture_direction)
-    {
-    case G_DESKTOP_MOUSE_DWELL_DIRECTION_LEFT:
-      return CLUTTER_A11Y_DWELL_DIRECTION_LEFT;
-      break;
-    case G_DESKTOP_MOUSE_DWELL_DIRECTION_RIGHT:
-      return CLUTTER_A11Y_DWELL_DIRECTION_RIGHT;
-      break;
-    case G_DESKTOP_MOUSE_DWELL_DIRECTION_UP:
-      return CLUTTER_A11Y_DWELL_DIRECTION_UP;
-      break;
-    case G_DESKTOP_MOUSE_DWELL_DIRECTION_DOWN:
-      return CLUTTER_A11Y_DWELL_DIRECTION_DOWN;
-      break;
-    default:
-      break;
-    }
-  return CLUTTER_A11Y_DWELL_DIRECTION_NONE;
-}
-
-static void
-load_pointer_a11y_settings (MetaInputSettings  *input_settings)
-{
-  MetaInputSettingsPrivate *priv = meta_input_settings_get_instance_private (input_settings);
-  ClutterPointerA11ySettings pointer_a11y_settings;
-  GDesktopMouseDwellMode dwell_mode;
-  guint i;
-
-  clutter_seat_get_pointer_a11y_settings (CLUTTER_SEAT (priv->seat),
-                                          &pointer_a11y_settings);
-  pointer_a11y_settings.controls = 0;
-  for (i = 0; i < G_N_ELEMENTS (pointer_a11y_settings_flags_pair); i++)
-    {
-      if (g_settings_get_boolean (priv->mouse_a11y_settings, pointer_a11y_settings_flags_pair[i].name))
-        pointer_a11y_settings.controls |= pointer_a11y_settings_flags_pair[i].flag;
-    }
-
-  /* "secondary-click-time" is expressed in seconds */
-  pointer_a11y_settings.secondary_click_delay =
-    (1000 * g_settings_get_double (priv->mouse_a11y_settings, "secondary-click-time"));
-  /* "dwell-time" is expressed in seconds */
-  pointer_a11y_settings.dwell_delay =
-    (1000 * g_settings_get_double (priv->mouse_a11y_settings, "dwell-time"));
-  pointer_a11y_settings.dwell_threshold = g_settings_get_int (priv->mouse_a11y_settings,
-                                                              "dwell-threshold");
-
-  dwell_mode = g_settings_get_enum (priv->mouse_a11y_settings, "dwell-mode");
-  if (dwell_mode == G_DESKTOP_MOUSE_DWELL_MODE_WINDOW)
-    pointer_a11y_settings.dwell_mode = CLUTTER_A11Y_DWELL_MODE_WINDOW;
-  else
-    pointer_a11y_settings.dwell_mode = CLUTTER_A11Y_DWELL_MODE_GESTURE;
-
-  pointer_a11y_settings.dwell_gesture_single =
-    pointer_a11y_dwell_direction_from_setting (input_settings, "dwell-gesture-single");
-  pointer_a11y_settings.dwell_gesture_double =
-    pointer_a11y_dwell_direction_from_setting (input_settings, "dwell-gesture-double");
-  pointer_a11y_settings.dwell_gesture_drag =
-    pointer_a11y_dwell_direction_from_setting (input_settings, "dwell-gesture-drag");
-  pointer_a11y_settings.dwell_gesture_secondary =
-    pointer_a11y_dwell_direction_from_setting (input_settings, "dwell-gesture-secondary");
-
-  clutter_seat_set_pointer_a11y_settings (CLUTTER_SEAT (priv->seat),
-                                          &pointer_a11y_settings);
-}
-
-static void
-meta_input_mouse_a11y_settings_changed (GSettings  *settings,
-                                        const char *key,
-                                        gpointer    user_data)
-{
-  MetaInputSettings *input_settings = META_INPUT_SETTINGS (user_data);
-
-  load_pointer_a11y_settings (input_settings);
-}
-
 static GSettings *
 lookup_device_settings (ClutterInputDevice *device)
 {
@@ -1750,7 +1605,6 @@ meta_input_settings_constructed (GObject *object)
   check_mappable_devices (input_settings);
 
   load_keyboard_a11y_settings (input_settings);
-  load_pointer_a11y_settings (input_settings);
 }
 
 static void
@@ -1760,8 +1614,6 @@ meta_input_settings_class_init (MetaInputSettingsClass *klass)
 
   object_class->dispose = meta_input_settings_dispose;
   object_class->constructed = meta_input_settings_constructed;
-  object_class->set_property = meta_input_settings_set_property;
-  object_class->get_property = meta_input_settings_get_property;
 
   quark_tool_settings =
     g_quark_from_static_string ("meta-input-settings-tool-settings");
@@ -1774,15 +1626,6 @@ meta_input_settings_class_init (MetaInputSettingsClass *klass)
                   G_TYPE_NONE, 1,
                   G_TYPE_POINTER);
 
-  props[PROP_SEAT] =
-    g_param_spec_object ("seat",
-                         "Seat",
-                         "Seat",
-                         CLUTTER_TYPE_SEAT,
-                         CLUTTER_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY);
-
-  g_object_class_install_properties (object_class, N_PROPS, props);
 }
 
 static void
@@ -1812,10 +1655,6 @@ meta_input_settings_init (MetaInputSettings *settings)
   g_signal_connect (priv->keyboard_a11y_settings, "changed",
                     G_CALLBACK (meta_input_keyboard_a11y_settings_changed), settings);
 
-  priv->mouse_a11y_settings = g_settings_new ("org.gnome.desktop.a11y.mouse");
-  g_signal_connect (priv->mouse_a11y_settings, "changed",
-                    G_CALLBACK (meta_input_mouse_a11y_settings_changed), settings);
-
   priv->mappable_devices =
     g_hash_table_new_full (NULL, NULL, NULL, (GDestroyNotify) device_mapping_info_free);
 
