@@ -34,6 +34,17 @@ enum
 
 static guint signals[N_SIGNALS];
 
+enum
+{
+  PROP_0,
+
+  PROP_HAS_ACCELEROMETER,
+
+  PROP_LAST
+};
+
+static GParamSpec *props[PROP_LAST];
+
 struct _MetaOrientationManager
 {
   GObject parent_instance;
@@ -41,9 +52,11 @@ struct _MetaOrientationManager
   GCancellable *cancellable;
 
   guint iio_watch_id;
+  guint sync_idle_id;
   GDBusProxy *iio_proxy;
   MetaOrientation prev_orientation;
   MetaOrientation curr_orientation;
+  guint has_accel : 1;
 
   GSettings *settings;
 };
@@ -71,22 +84,24 @@ orientation_from_string (const char *orientation)
 static void
 read_iio_proxy (MetaOrientationManager *self)
 {
-  gboolean has_accel = FALSE;
   GVariant *v;
 
   self->curr_orientation = META_ORIENTATION_UNDEFINED;
 
   if (!self->iio_proxy)
-    return;
+    {
+      self->has_accel = FALSE;
+      return;
+    }
 
   v = g_dbus_proxy_get_cached_property (self->iio_proxy, "HasAccelerometer");
   if (v)
     {
-      has_accel = g_variant_get_boolean (v);
+      self->has_accel = !!g_variant_get_boolean (v);
       g_variant_unref (v);
     }
 
-  if (has_accel)
+  if (self->has_accel)
     {
       v = g_dbus_proxy_get_cached_property (self->iio_proxy, "AccelerometerOrientation");
       if (v)
@@ -100,11 +115,16 @@ read_iio_proxy (MetaOrientationManager *self)
 static void
 sync_state (MetaOrientationManager *self)
 {
-  if (g_settings_get_boolean (self->settings, ORIENTATION_LOCK_KEY))
-    return;
+  gboolean had_accel = self->has_accel;
 
   read_iio_proxy (self);
 
+  if (had_accel != self->has_accel)
+    g_object_notify_by_pspec (G_OBJECT (self), props[PROP_HAS_ACCELEROMETER]);
+
+  if (g_settings_get_boolean (self->settings, ORIENTATION_LOCK_KEY))
+    return;
+
   if (self->prev_orientation == self->curr_orientation)
     return;
 
@@ -116,13 +136,39 @@ sync_state (MetaOrientationManager *self)
   g_signal_emit (self, signals[ORIENTATION_CHANGED], 0);
 }
 
+static gboolean
+changed_idle (gpointer user_data)
+{
+  MetaOrientationManager *self = user_data;
+
+  self->sync_idle_id = 0;
+  sync_state (self);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+queue_sync_state (MetaOrientationManager *self)
+{
+  /* We need this idle to avoid triggering events happening while the session
+   * is not active (under X11), ideally this should be handled by stopping
+   * events if the session is not active, but we'll need a MetaLogind available
+   * in all the backends for having this working.
+   */
+
+  if (self->sync_idle_id)
+    return;
+
+  self->sync_idle_id = g_idle_add (changed_idle, self);
+}
+
 static void
 orientation_lock_changed (GSettings *settings,
                           gchar     *key,
                           gpointer   user_data)
 {
   MetaOrientationManager *self = user_data;
-  sync_state (self);
+  queue_sync_state (self);
 }
 
 static void
@@ -132,7 +178,7 @@ iio_properties_changed (GDBusProxy *proxy,
                         gpointer    user_data)
 {
   MetaOrientationManager *self = user_data;
-  sync_state (self);
+  queue_sync_state (self);
 }
 
 static void
@@ -241,6 +287,25 @@ meta_orientation_manager_init (MetaOrientationManager *self)
   sync_state (self);
 }
 
+static void
+meta_orientation_manager_get_property (GObject    *object,
+                                       guint       prop_id,
+                                       GValue     *value,
+                                       GParamSpec *pspec)
+{
+  MetaOrientationManager *self = META_ORIENTATION_MANAGER (object);
+
+  switch (prop_id)
+    {
+    case PROP_HAS_ACCELEROMETER:
+      g_value_set_boolean (value, self->has_accel);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
 static void
 meta_orientation_manager_finalize (GObject *object)
 {
@@ -250,6 +315,7 @@ meta_orientation_manager_finalize (GObject *object)
   g_clear_object (&self->cancellable);
 
   g_bus_unwatch_name (self->iio_watch_id);
+  g_clear_handle_id (&self->sync_idle_id, g_source_remove);
   g_clear_object (&self->iio_proxy);
 
   g_clear_object (&self->settings);
@@ -263,6 +329,7 @@ meta_orientation_manager_class_init (MetaOrientationManagerClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize = meta_orientation_manager_finalize;
+  gobject_class->get_property = meta_orientation_manager_get_property;
 
   signals[ORIENTATION_CHANGED] =
     g_signal_new ("orientation-changed",
@@ -271,6 +338,16 @@ meta_orientation_manager_class_init (MetaOrientationManagerClass *klass)
                   0,
                   NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
+
+  props[PROP_HAS_ACCELEROMETER] =
+    g_param_spec_boolean ("has-accelerometer",
+                          "Has accelerometer",
+                          "Has accelerometer",
+                          FALSE,
+                          G_PARAM_READABLE |
+                          G_PARAM_EXPLICIT_NOTIFY |
+                          G_PARAM_STATIC_STRINGS);
+  g_object_class_install_properties (gobject_class, PROP_LAST, props);
 }
 
 MetaOrientation
@@ -278,3 +355,9 @@ meta_orientation_manager_get_orientation (MetaOrientationManager *self)
 {
   return self->curr_orientation;
 }
+
+gboolean
+meta_orientation_manager_has_accelerometer (MetaOrientationManager *self)
+{
+  return self->has_accel;
+}
