@@ -231,6 +231,25 @@ maybe_record_frame_on_idle (gpointer user_data)
   return G_SOURCE_REMOVE;
 }
 
+static void
+before_stage_painted (MetaStage           *stage,
+                      ClutterStageView    *view,
+                      ClutterPaintContext *paint_context,
+                      gpointer             user_data)
+{
+  MetaScreenCastAreaStreamSrc *area_src =
+    META_SCREEN_CAST_AREA_STREAM_SRC (user_data);
+  MetaScreenCastStreamSrc *src = META_SCREEN_CAST_STREAM_SRC (area_src);
+
+  if (area_src->maybe_record_idle_id)
+    return;
+
+  if (!clutter_stage_view_peek_scanout (view))
+    return;
+
+  area_src->maybe_record_idle_id = g_idle_add (maybe_record_frame_on_idle, src);
+}
+
 static void
 stage_painted (MetaStage           *stage,
                ClutterStageView    *view,
@@ -267,8 +286,7 @@ stage_painted (MetaStage           *stage,
 }
 
 static void
-add_view_painted_watches (MetaScreenCastAreaStreamSrc *area_src,
-                          MetaStageWatchPhase          watch_phase)
+add_view_painted_watches (MetaScreenCastAreaStreamSrc *area_src)
 {
   MetaScreenCastStreamSrc *src = META_SCREEN_CAST_STREAM_SRC (area_src);
   MetaScreenCastStream *stream = meta_screen_cast_stream_src_get_stream (src);
@@ -296,7 +314,15 @@ add_view_painted_watches (MetaScreenCastAreaStreamSrc *area_src,
 
           watch = meta_stage_watch_view (meta_stage,
                                          CLUTTER_STAGE_VIEW (view),
-                                         watch_phase,
+                                         META_STAGE_WATCH_BEFORE_PAINT,
+                                         before_stage_painted,
+                                         area_src);
+
+          area_src->watches = g_list_prepend (area_src->watches, watch);
+
+          watch = meta_stage_watch_view (meta_stage,
+                                         CLUTTER_STAGE_VIEW (view),
+                                         META_STAGE_WATCH_AFTER_ACTOR_PAINT,
                                          stage_painted,
                                          area_src);
 
@@ -316,8 +342,7 @@ on_monitors_changed (MetaMonitorManager          *monitor_manager,
     meta_stage_remove_watch (stage, l->data);
   g_clear_pointer (&area_src->watches, g_list_free);
 
-  add_view_painted_watches (area_src,
-                            META_STAGE_WATCH_AFTER_ACTOR_PAINT);
+  add_view_painted_watches (area_src);
 }
 
 static void
@@ -349,14 +374,12 @@ meta_screen_cast_area_stream_src_enable (MetaScreenCastStreamSrc *src)
       meta_cursor_tracker_track_position (cursor_tracker);
       G_GNUC_FALLTHROUGH;
     case META_SCREEN_CAST_CURSOR_MODE_HIDDEN:
-      add_view_painted_watches (area_src,
-                                META_STAGE_WATCH_AFTER_ACTOR_PAINT);
+      add_view_painted_watches (area_src);
       break;
     case META_SCREEN_CAST_CURSOR_MODE_EMBEDDED:
       inhibit_hw_cursor (area_src);
       meta_cursor_tracker_track_position (cursor_tracker);
-      add_view_painted_watches (area_src,
-                                META_STAGE_WATCH_AFTER_ACTOR_PAINT);
+      add_view_painted_watches (area_src);
       break;
     }
 
