@@ -30,6 +30,7 @@
 #include "backends/meta-screen-cast-area-stream.h"
 #include "backends/meta-screen-cast-monitor-stream.h"
 #include "backends/meta-screen-cast-stream.h"
+#include "backends/meta-screen-cast-virtual-stream.h"
 #include "backends/meta-screen-cast-window-stream.h"
 #include "core/display-private.h"
 
@@ -50,6 +51,7 @@ struct _MetaScreenCastSession
 
   MetaScreenCastSessionHandle *handle;
 
+  gboolean is_active;
   gboolean disable_animations;
 };
 
@@ -116,14 +118,24 @@ meta_screen_cast_session_start (MetaScreenCastSession  *session,
 
   init_remote_access_handle (session);
 
+  session->is_active = TRUE;
+
   return TRUE;
 }
 
+gboolean
+meta_screen_cast_session_is_active (MetaScreenCastSession *session)
+{
+  return session->is_active;
+}
+
 void
 meta_screen_cast_session_close (MetaScreenCastSession *session)
 {
   MetaDBusScreenCastSession *skeleton = META_DBUS_SCREEN_CAST_SESSION (session);
 
+  session->is_active = FALSE;
+
   g_list_free_full (session->streams, g_object_unref);
 
   meta_dbus_session_notify_closed (META_DBUS_SESSION (session));
@@ -193,6 +205,12 @@ meta_screen_cast_session_get_peer_name (MetaScreenCastSession *session)
   return session->peer_name;
 }
 
+MetaScreenCastSessionType
+meta_screen_cast_session_get_session_type (MetaScreenCastSession *session)
+{
+  return session->session_type;
+}
+
 static gboolean
 check_permission (MetaScreenCastSession *session,
                   GDBusMethodInvocation *invocation)
@@ -600,6 +618,83 @@ handle_record_area (MetaDBusScreenCastSession *skeleton,
   return TRUE;
 }
 
+static gboolean
+handle_record_virtual (MetaDBusScreenCastSession *skeleton,
+                       GDBusMethodInvocation     *invocation,
+                       GVariant                  *properties_variant)
+{
+  MetaScreenCastSession *session = META_SCREEN_CAST_SESSION (skeleton);
+  GDBusInterfaceSkeleton *interface_skeleton;
+  GDBusConnection *connection;
+  MetaScreenCastCursorMode cursor_mode;
+  gboolean is_platform;
+  MetaScreenCastFlag flags;
+  g_autoptr (GError) error = NULL;
+  MetaScreenCastVirtualStream *virtual_stream;
+  MetaScreenCastStream *stream;
+  char *stream_path;
+
+  if (!check_permission (session, invocation))
+    {
+      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
+                                             G_DBUS_ERROR_ACCESS_DENIED,
+                                             "Permission denied");
+      return TRUE;
+    }
+
+  if (!g_variant_lookup (properties_variant, "cursor-mode", "u", &cursor_mode))
+    {
+      cursor_mode = META_SCREEN_CAST_CURSOR_MODE_HIDDEN;
+    }
+  else
+    {
+      if (!is_valid_cursor_mode (cursor_mode))
+        {
+          g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
+                                                 G_DBUS_ERROR_FAILED,
+                                                 "Unknown cursor mode");
+          return TRUE;
+        }
+    }
+
+  if (!g_variant_lookup (properties_variant, "is-platform", "b", &is_platform))
+    is_platform = FALSE;
+
+  interface_skeleton = G_DBUS_INTERFACE_SKELETON (skeleton);
+  connection = g_dbus_interface_skeleton_get_connection (interface_skeleton);
+
+  flags = META_SCREEN_CAST_FLAG_NONE;
+  if (is_platform)
+    flags |= META_SCREEN_CAST_FLAG_IS_PLATFORM;
+
+  virtual_stream = meta_screen_cast_virtual_stream_new (session,
+                                                        connection,
+                                                        cursor_mode,
+                                                        flags,
+                                                        &error);
+  if (!virtual_stream)
+    {
+      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
+                                             G_DBUS_ERROR_FAILED,
+                                             "Failed to record virtual: %s",
+                                             error->message);
+      return TRUE;
+    }
+
+  stream = META_SCREEN_CAST_STREAM (virtual_stream);
+  stream_path = meta_screen_cast_stream_get_object_path (stream);
+
+  session->streams = g_list_append (session->streams, stream);
+
+  g_signal_connect (stream, "closed", G_CALLBACK (on_stream_closed), session);
+
+  meta_dbus_screen_cast_session_complete_record_virtual (skeleton,
+                                                         invocation,
+                                                         stream_path);
+
+  return TRUE;
+}
+
 static void
 meta_screen_cast_session_init_iface (MetaDBusScreenCastSessionIface *iface)
 {
@@ -608,6 +703,7 @@ meta_screen_cast_session_init_iface (MetaDBusScreenCastSessionIface *iface)
   iface->handle_record_monitor = handle_record_monitor;
   iface->handle_record_window = handle_record_window;
   iface->handle_record_area = handle_record_area;
+  iface->handle_record_virtual = handle_record_virtual;
 }
 
 static void
