@@ -41,6 +41,7 @@ typedef enum _MetaScreenCastRecordFlag
 {
   META_SCREEN_CAST_RECORD_FLAG_NONE = 0,
   META_SCREEN_CAST_RECORD_FLAG_CURSOR_ONLY = 1 << 0,
+  META_SCREEN_CAST_RECORD_FLAG_DMABUF_ONLY = 1 << 1,
 } MetaScreenCastRecordFlag;
 
 #define META_TYPE_SCREEN_CAST_STREAM_SRC (meta_screen_cast_stream_src_get_type ())
@@ -53,13 +54,16 @@ struct _MetaScreenCastStreamSrcClass
 {
   GObjectClass parent_class;
 
-  void (* get_specs) (MetaScreenCastStreamSrc *src,
-                      int                     *width,
-                      int                     *height,
-                      float                   *frame_rate);
+  gboolean (* get_specs) (MetaScreenCastStreamSrc *src,
+                          int                     *width,
+                          int                     *height,
+                          float                   *frame_rate);
   void (* enable) (MetaScreenCastStreamSrc *src);
   void (* disable) (MetaScreenCastStreamSrc *src);
   gboolean (* record_to_buffer) (MetaScreenCastStreamSrc  *src,
+                                 int                       width,
+                                 int                       height,
+                                 int                       stride,
                                  uint8_t                  *data,
                                  GError                  **error);
   gboolean (* record_to_framebuffer) (MetaScreenCastStreamSrc  *src,
@@ -71,19 +75,20 @@ struct _MetaScreenCastStreamSrcClass
                               MetaRectangle           *crop_rect);
   void (* set_cursor_metadata) (MetaScreenCastStreamSrc *src,
                                 struct spa_meta_cursor  *spa_meta_cursor);
+
+  void (* notify_params_updated) (MetaScreenCastStreamSrc   *src,
+                                  struct spa_video_info_raw *video_format);
 };
 
+void meta_screen_cast_stream_src_close (MetaScreenCastStreamSrc *src);
+
+gboolean meta_screen_cast_stream_src_is_enabled (MetaScreenCastStreamSrc *src);
+
 void meta_screen_cast_stream_src_maybe_record_frame (MetaScreenCastStreamSrc  *src,
                                                      MetaScreenCastRecordFlag  flags);
 
 gboolean meta_screen_cast_stream_src_pending_follow_up_frame (MetaScreenCastStreamSrc *src);
 
-int meta_screen_cast_stream_src_get_stride (MetaScreenCastStreamSrc *src);
-
-int meta_screen_cast_stream_src_get_width (MetaScreenCastStreamSrc *src);
-
-int meta_screen_cast_stream_src_get_height (MetaScreenCastStreamSrc *src);
-
 MetaScreenCastStream * meta_screen_cast_stream_src_get_stream (MetaScreenCastStreamSrc *src);
 
 gboolean meta_screen_cast_stream_src_draw_cursor_into (MetaScreenCastStreamSrc  *src,
