@@ -27,13 +27,27 @@
 #include <stdint.h>
 
 #include "cogl/cogl.h"
+#include "core/util-private.h"
+
+typedef enum _MetaDrmBufferFlags
+{
+  META_DRM_BUFFER_FLAG_NONE = 0,
+  META_DRM_BUFFER_FLAG_DISABLE_MODIFIERS = 1 << 0,
+} MetaDrmBufferFlags;
 
 #define META_TYPE_DRM_BUFFER (meta_drm_buffer_get_type ())
+META_EXPORT_TEST
 G_DECLARE_DERIVABLE_TYPE (MetaDrmBuffer,
                           meta_drm_buffer,
                           META, DRM_BUFFER,
                           GObject)
 
+int meta_drm_buffer_export_fd (MetaDrmBuffer  *buffer,
+                               GError        **error);
+
+gboolean meta_drm_buffer_ensure_fb_id (MetaDrmBuffer  *buffer,
+                                       GError        **error);
+
 uint32_t meta_drm_buffer_get_fb_id (MetaDrmBuffer *buffer);
 
 int meta_drm_buffer_get_width (MetaDrmBuffer *buffer);
@@ -42,8 +56,15 @@ int meta_drm_buffer_get_height (MetaDrmBuffer *buffer);
 
 int meta_drm_buffer_get_stride (MetaDrmBuffer *buffer);
 
+int meta_drm_buffer_get_bpp (MetaDrmBuffer *buffer);
+
 uint32_t meta_drm_buffer_get_format (MetaDrmBuffer *buffer);
 
+int meta_drm_buffer_get_offset (MetaDrmBuffer *buffer,
+                                int            plane);
+
+uint64_t meta_drm_buffer_get_modifier (MetaDrmBuffer *buffer);
+
 gboolean meta_drm_buffer_supports_fill_timings (MetaDrmBuffer *buffer);
 
 gboolean meta_drm_buffer_fill_timings (MetaDrmBuffer  *buffer,
