@@ -92,27 +92,26 @@ meta_kms_crtc_is_active (MetaKmsCrtc *crtc)
   return crtc->current_state.is_active;
 }
 
-
-gboolean
-meta_kms_crtc_has_gamma (MetaKmsCrtc *crtc)
-{
-  return crtc->current_state.gamma.size > 0;
-}
-
 static void
 read_gamma_state (MetaKmsCrtc       *crtc,
                   MetaKmsCrtcState  *crtc_state,
                   MetaKmsImplDevice *impl_device,
                   drmModeCrtc       *drm_crtc)
 {
-  g_assert (!crtc_state->gamma.red &&
-            !crtc_state->gamma.green &&
-            !crtc_state->gamma.blue);
+  if (crtc_state->gamma.size != drm_crtc->gamma_size)
+    {
+      crtc_state->gamma.size = drm_crtc->gamma_size;
 
-  crtc_state->gamma.size = drm_crtc->gamma_size;
-  crtc_state->gamma.red = g_new0 (uint16_t, drm_crtc->gamma_size);
-  crtc_state->gamma.green = g_new0 (uint16_t, drm_crtc->gamma_size);
-  crtc_state->gamma.blue = g_new0 (uint16_t, drm_crtc->gamma_size);
+      crtc_state->gamma.red = g_realloc_n (crtc_state->gamma.red,
+                                           drm_crtc->gamma_size,
+                                           sizeof (uint16_t));
+      crtc_state->gamma.green = g_realloc_n (crtc_state->gamma.green,
+                                             drm_crtc->gamma_size,
+                                             sizeof (uint16_t));
+      crtc_state->gamma.blue = g_realloc_n (crtc_state->gamma.blue,
+                                            drm_crtc->gamma_size,
+                                            sizeof (uint16_t));
+    }
 
   drmModeCrtcGetGamma (meta_kms_impl_device_get_fd (impl_device),
                        crtc->id,
@@ -203,6 +202,7 @@ meta_kms_crtc_read_state (MetaKmsCrtc             *crtc,
 
   crtc_state.is_drm_mode_valid = drm_crtc->mode_valid;
   crtc_state.drm_mode = drm_crtc->mode;
+  crtc_state.gamma.size = 0;
 
   active_prop = &crtc->prop_table.props[META_KMS_CRTC_PROP_ACTIVE];
   if (active_prop->prop_id)
@@ -217,8 +217,6 @@ meta_kms_crtc_read_state (MetaKmsCrtc             *crtc,
       crtc_state.is_active = drm_crtc->mode_valid;
     }
 
-  read_gamma_state (crtc, &crtc_state, impl_device, drm_crtc);
-
   if (!crtc_state.is_active)
     {
       if (crtc->current_state.is_active)
@@ -226,6 +224,7 @@ meta_kms_crtc_read_state (MetaKmsCrtc             *crtc,
     }
   else
     {
+      read_gamma_state (crtc, &crtc_state, impl_device, drm_crtc);
       changes = meta_kms_crtc_state_changes (&crtc->current_state, &crtc_state);
     }
 
@@ -278,6 +277,15 @@ out:
   return changes;
 }
 
+void
+meta_kms_crtc_disable (MetaKmsCrtc *crtc)
+{
+  crtc->current_state.is_active = FALSE;
+  crtc->current_state.rect = (MetaRectangle) { 0 };
+  crtc->current_state.is_drm_mode_valid = FALSE;
+  crtc->current_state.drm_mode = (drmModeModeInfo) { 0 };
+}
+
 void
 meta_kms_crtc_predict_state (MetaKmsCrtc   *crtc,
                              MetaKmsUpdate *update)
