@@ -25,6 +25,7 @@
 #include <xf86drmMode.h>
 
 #include "backends/native/meta-kms-types.h"
+#include "core/util-private.h"
 #include "meta/boxes.h"
 
 typedef struct _MetaKmsCrtcState
@@ -54,20 +55,23 @@ typedef struct _MetaKmsCrtcGamma
 } MetaKmsCrtcGamma;
 
 #define META_TYPE_KMS_CRTC (meta_kms_crtc_get_type ())
+META_EXPORT_TEST
 G_DECLARE_FINAL_TYPE (MetaKmsCrtc, meta_kms_crtc,
                       META, KMS_CRTC,
                       GObject)
 
+META_EXPORT_TEST
 MetaKmsDevice * meta_kms_crtc_get_device (MetaKmsCrtc *crtc);
 
+META_EXPORT_TEST
 const MetaKmsCrtcState * meta_kms_crtc_get_current_state (MetaKmsCrtc *crtc);
 
+META_EXPORT_TEST
 uint32_t meta_kms_crtc_get_id (MetaKmsCrtc *crtc);
 
 int meta_kms_crtc_get_idx (MetaKmsCrtc *crtc);
 
-gboolean meta_kms_crtc_has_gamma (MetaKmsCrtc *crtc);
-
+META_EXPORT_TEST
 gboolean meta_kms_crtc_is_active (MetaKmsCrtc *crtc);
 
 void meta_kms_crtc_gamma_free (MetaKmsCrtcGamma *gamma);
