@@ -396,7 +396,7 @@ update_connectors (MetaKmsImplDevice *impl_device,
   MetaKmsImplDevicePrivate *priv =
     meta_kms_impl_device_get_instance_private (impl_device);
   g_autolist (MetaKmsConnector) connectors = NULL;
-  gboolean needs_full_change = FALSE;
+  gboolean added_connector = FALSE;
   unsigned int i;
   int fd;
 
@@ -420,7 +420,7 @@ update_connectors (MetaKmsImplDevice *impl_device,
         {
           connector = meta_kms_connector_new (impl_device, drm_connector,
                                               drm_resources);
-          needs_full_change = TRUE;
+          added_connector = TRUE;
         }
 
       drmModeFreeConnector (drm_connector);
@@ -428,7 +428,8 @@ update_connectors (MetaKmsImplDevice *impl_device,
       connectors = g_list_prepend (connectors, connector);
     }
 
-  if (!needs_full_change)
+  if (!added_connector &&
+      g_list_length (connectors) == g_list_length (priv->connectors))
     return META_KMS_UPDATE_CHANGE_NONE;
 
   g_list_free_full (priv->connectors, g_object_unref);
@@ -771,7 +772,7 @@ err:
   return META_KMS_UPDATE_CHANGE_FULL;
 }
 
-void
+static void
 meta_kms_impl_device_predict_states (MetaKmsImplDevice *impl_device,
                                      MetaKmsUpdate     *update)
 {
@@ -815,11 +816,30 @@ meta_kms_impl_device_process_update (MetaKmsImplDevice *impl_device,
 
   meta_kms_impl_device_hold_fd (impl_device);
   feedback = klass->process_update (impl_device, update, flags);
+  if (!(flags & META_KMS_UPDATE_FLAG_TEST_ONLY))
+    meta_kms_impl_device_predict_states (impl_device, update);
   meta_kms_impl_device_unhold_fd (impl_device);
 
   return feedback;
 }
 
+void
+meta_kms_impl_device_disable (MetaKmsImplDevice *impl_device)
+{
+  MetaKmsImplDevicePrivate *priv =
+    meta_kms_impl_device_get_instance_private (impl_device);
+  MetaKmsImplDeviceClass *klass = META_KMS_IMPL_DEVICE_GET_CLASS (impl_device);
+
+  if (!priv->device_file)
+    return;
+
+  meta_kms_impl_device_hold_fd (impl_device);
+  klass->disable (impl_device);
+  g_list_foreach (priv->crtcs, (GFunc) meta_kms_crtc_disable, NULL);
+  g_list_foreach (priv->connectors, (GFunc) meta_kms_connector_disable, NULL);
+  meta_kms_impl_device_unhold_fd (impl_device);
+}
+
 void
 meta_kms_impl_device_handle_page_flip_callback (MetaKmsImplDevice   *impl_device,
                                                 MetaKmsPageFlipData *page_flip_data)
@@ -851,6 +871,19 @@ meta_kms_impl_device_hold_fd (MetaKmsImplDevice *impl_device)
   priv->fd_hold_count++;
 }
 
+static void
+clear_fd_source (MetaKmsImplDevice *impl_device)
+{
+  MetaKmsImplDevicePrivate *priv =
+    meta_kms_impl_device_get_instance_private (impl_device);
+
+  if (!priv->fd_source)
+    return;
+
+  g_source_destroy (priv->fd_source);
+  g_clear_pointer (&priv->fd_source, g_source_unref);
+}
+
 void
 meta_kms_impl_device_unhold_fd (MetaKmsImplDevice *impl_device)
 {
@@ -866,12 +899,7 @@ meta_kms_impl_device_unhold_fd (MetaKmsImplDevice *impl_device)
   if (priv->fd_hold_count == 0)
     {
       g_clear_pointer (&priv->device_file, meta_device_file_release);
-
-      if (priv->fd_source)
-        {
-          g_source_destroy (priv->fd_source);
-          g_clear_pointer (&priv->fd_source, g_source_unref);
-        }
+      clear_fd_source (impl_device);
     }
 }
 
@@ -998,6 +1026,8 @@ meta_kms_impl_device_prepare_shutdown (MetaKmsImplDevice *impl_device)
 
   if (klass->prepare_shutdown)
     klass->prepare_shutdown (impl_device);
+
+  clear_fd_source (impl_device);
 }
 
 static gboolean
