@@ -75,6 +75,7 @@ struct _MetaKmsImplDeviceClass
   MetaKmsFeedback * (* process_update) (MetaKmsImplDevice *impl_device,
                                         MetaKmsUpdate     *update,
                                         MetaKmsUpdateFlag  flags);
+  void (* disable) (MetaKmsImplDevice *impl_device);
   void (* handle_page_flip_callback) (MetaKmsImplDevice   *impl_device,
                                       MetaKmsPageFlipData *page_flip_data);
   void (* discard_pending_page_flips) (MetaKmsImplDevice *impl_device);
@@ -124,6 +125,8 @@ const char * meta_kms_impl_device_get_path (MetaKmsImplDevice *impl_device);
 gboolean meta_kms_impl_device_dispatch (MetaKmsImplDevice  *impl_device,
                                         GError            **error);
 
+void meta_kms_impl_device_disable (MetaKmsImplDevice *impl_device);
+
 drmModePropertyPtr meta_kms_impl_device_find_property (MetaKmsImplDevice       *impl_device,
                                                        drmModeObjectProperties *props,
                                                        const char              *prop_name,
@@ -139,9 +142,6 @@ MetaKmsUpdateChanges meta_kms_impl_device_update_states (MetaKmsImplDevice *impl
                                                          uint32_t           crtc_id,
                                                          uint32_t           connector_id);
 
-void meta_kms_impl_device_predict_states (MetaKmsImplDevice *impl_device,
-                                          MetaKmsUpdate     *update);
-
 void meta_kms_impl_device_notify_modes_set (MetaKmsImplDevice *impl_device);
 
 MetaKmsPlane * meta_kms_impl_device_add_fake_plane (MetaKmsImplDevice *impl_device,
