@@ -64,6 +64,22 @@ typedef enum _MetaKmsDeviceFlag
   META_KMS_DEVICE_FLAG_NO_MODE_SETTING = 1 << 4,
 } MetaKmsDeviceFlag;
 
+typedef enum _MetaKmsUpdateChanges
+{
+  META_KMS_UPDATE_CHANGE_NONE = 0,
+  META_KMS_UPDATE_CHANGE_GAMMA = 1 << 0,
+  META_KMS_UPDATE_CHANGE_NO_DEVICES = 1 << 1,
+  META_KMS_UPDATE_CHANGE_PRIVACY_SCREEN = 1 << 2,
+  META_KMS_UPDATE_CHANGE_FULL = -1,
+} MetaKmsUpdateChanges;
+
+typedef enum _MetaKmsUpdateFlag
+{
+  META_KMS_UPDATE_FLAG_NONE = 0,
+  META_KMS_UPDATE_FLAG_PRESERVE_ON_ERROR = 1 << 0,
+  META_KMS_UPDATE_FLAG_TEST_ONLY = 1 << 1,
+} MetaKmsUpdateFlag;
+
 typedef enum _MetaKmsPlaneType MetaKmsPlaneType;
 
 typedef enum _MetaKmsPropType
