@@ -43,6 +43,8 @@
 #include "backends/native/meta-kms-utils.h"
 #include "backends/native/meta-kms.h"
 #include "backends/native/meta-output-kms.h"
+#include "backends/native/meta-render-device-gbm.h"
+#include "backends/native/meta-render-device.h"
 #include "backends/native/meta-renderer-native-gles3.h"
 #include "backends/native/meta-renderer-native-private.h"
 
@@ -226,6 +228,8 @@ notify_view_crtc_presented (MetaRendererView *view,
 
   frame_info = cogl_onscreen_peek_head_frame_info (onscreen);
 
+  g_return_if_fail (frame_info != NULL);
+
   crtc = META_CRTC (meta_crtc_kms_from_kms_crtc (kms_crtc));
   maybe_update_frame_info (crtc, frame_info, time_us, flags, sequence);
 
@@ -369,6 +373,7 @@ custom_egl_stream_page_flip (gpointer custom_page_flip_data,
   MetaRendererView *view = user_data;
   MetaEgl *egl = meta_onscreen_native_get_egl (onscreen_native);
   MetaRendererNativeGpuData *renderer_gpu_data;
+  MetaRenderDevice *render_device;
   EGLDisplay *egl_display;
   EGLAttrib *acquire_attribs;
   g_autoptr (GError) error = NULL;
@@ -382,8 +387,9 @@ custom_egl_stream_page_flip (gpointer custom_page_flip_data,
   renderer_gpu_data =
     meta_renderer_native_get_gpu_data (onscreen_native->renderer_native,
                                        onscreen_native->render_gpu);
+  render_device = renderer_gpu_data->render_device;
 
-  egl_display = renderer_gpu_data->egl_display;
+  egl_display = meta_render_device_get_egl_display (render_device);
   if (!meta_egl_stream_consumer_acquire_attrib (egl,
                                                 egl_display,
                                                 onscreen_native->egl.stream,
@@ -548,10 +554,14 @@ secondary_gpu_state_free (MetaOnscreenNativeSecondaryGpuState *secondary_gpu_sta
   if (secondary_gpu_state->egl_surface != EGL_NO_SURFACE)
     {
       MetaRendererNativeGpuData *renderer_gpu_data;
+      MetaRenderDevice *render_device;
+      EGLDisplay egl_display;
 
       renderer_gpu_data = secondary_gpu_state->renderer_gpu_data;
+      render_device = renderer_gpu_data->render_device;
+      egl_display = meta_render_device_get_egl_display (render_device);
       meta_egl_destroy_surface (egl,
-                                renderer_gpu_data->egl_display,
+                                egl_display,
                                 secondary_gpu_state->egl_surface,
                                 NULL);
     }
@@ -570,46 +580,25 @@ import_shared_framebuffer (CoglOnscreen                        *onscreen,
                            MetaOnscreenNativeSecondaryGpuState *secondary_gpu_state)
 {
   MetaOnscreenNative *onscreen_native = META_ONSCREEN_NATIVE (onscreen);
-  MetaGpuKms *gpu_kms;
-  MetaDeviceFile *device_file;
-  struct gbm_device *gbm_device;
-  MetaDrmBufferGbm *buffer_gbm;
-  MetaDrmBufferImport *buffer_import;
+  MetaRenderDevice *render_device;
   g_autoptr (GError) error = NULL;
+  MetaDrmBuffer *imported_buffer;
 
-  buffer_gbm = META_DRM_BUFFER_GBM (onscreen_native->gbm.next_fb);
-
-  gpu_kms = secondary_gpu_state->gpu_kms;
-  device_file = secondary_gpu_state->renderer_gpu_data->device_file;
-  gbm_device = meta_gbm_device_from_gpu (gpu_kms);
-  buffer_import = meta_drm_buffer_import_new (device_file,
-                                              gbm_device,
-                                              buffer_gbm,
-                                              &error);
-  if (!buffer_import)
+  render_device = secondary_gpu_state->renderer_gpu_data->render_device;
+  imported_buffer =
+    meta_render_device_import_dma_buf (render_device,
+                                       onscreen_native->gbm.next_fb,
+                                       &error);
+  if (!imported_buffer)
     {
       meta_topic (META_DEBUG_KMS,
                   "Zero-copy disabled for %s, "
                   "meta_drm_buffer_import_new failed: %s",
-                  meta_device_file_get_path (device_file),
+                  meta_render_device_get_name (render_device),
                   error->message);
 
       g_warn_if_fail (secondary_gpu_state->import_status ==
                       META_SHARED_FRAMEBUFFER_IMPORT_STATUS_NONE);
-
-      /*
-       * Fall back. If META_SHARED_FRAMEBUFFER_IMPORT_STATUS_NONE is
-       * in effect, we have COPY_MODE_PRIMARY prepared already, so we
-       * simply retry with that path. Import status cannot be FAILED,
-       * because we should not retry if failed once.
-       *
-       * If import status is OK, that is unexpected and we do not
-       * have the fallback path prepared which means this output cannot
-       * work anymore.
-       */
-      secondary_gpu_state->renderer_gpu_data->secondary.copy_mode =
-        META_SHARED_FRAMEBUFFER_COPY_MODE_PRIMARY;
-
       secondary_gpu_state->import_status =
         META_SHARED_FRAMEBUFFER_IMPORT_STATUS_FAILED;
       return FALSE;
@@ -620,7 +609,7 @@ import_shared_framebuffer (CoglOnscreen                        *onscreen,
    * when we are sure to succeed.
    */
   g_clear_object (&secondary_gpu_state->gbm.next_fb);
-  secondary_gpu_state->gbm.next_fb = META_DRM_BUFFER (buffer_import);
+  secondary_gpu_state->gbm.next_fb = imported_buffer;
 
   if (secondary_gpu_state->import_status ==
       META_SHARED_FRAMEBUFFER_IMPORT_STATUS_NONE)
@@ -633,7 +622,7 @@ import_shared_framebuffer (CoglOnscreen                        *onscreen,
 
       meta_topic (META_DEBUG_KMS,
                   "Using zero-copy for %s succeeded once.",
-                  meta_device_file_get_path (device_file));
+                  meta_render_device_get_name (render_device));
     }
 
   secondary_gpu_state->import_status =
@@ -651,9 +640,12 @@ copy_shared_framebuffer_gpu (CoglOnscreen                        *onscreen,
   MetaRendererNative *renderer_native = renderer_gpu_data->renderer_native;
   MetaEgl *egl = meta_renderer_native_get_egl (renderer_native);
   MetaGles3 *gles3 = meta_renderer_native_get_gles3 (renderer_native);
+  MetaRenderDevice *render_device;
+  EGLDisplay egl_display;
   GError *error = NULL;
   gboolean use_modifiers;
   MetaDeviceFile *device_file;
+  MetaDrmBufferFlags flags;
   MetaDrmBufferGbm *buffer_gbm;
   struct gbm_bo *bo;
 
@@ -663,8 +655,11 @@ copy_shared_framebuffer_gpu (CoglOnscreen                        *onscreen,
   g_warn_if_fail (secondary_gpu_state->gbm.next_fb == NULL);
   g_clear_object (&secondary_gpu_state->gbm.next_fb);
 
+  render_device = renderer_gpu_data->render_device;
+  egl_display = meta_render_device_get_egl_display (render_device);
+
   if (!meta_egl_make_current (egl,
-                              renderer_gpu_data->egl_display,
+                              egl_display,
                               secondary_gpu_state->egl_surface,
                               secondary_gpu_state->egl_surface,
                               renderer_gpu_data->secondary.egl_context,
@@ -682,7 +677,7 @@ copy_shared_framebuffer_gpu (CoglOnscreen                        *onscreen,
   bo = meta_drm_buffer_gbm_get_bo (buffer_gbm);
   if (!meta_renderer_native_gles3_blit_shared_bo (egl,
                                                   gles3,
-                                                  renderer_gpu_data->egl_display,
+                                                  egl_display,
                                                   renderer_gpu_data->secondary.egl_context,
                                                   secondary_gpu_state->egl_surface,
                                                   bo,
@@ -694,7 +689,7 @@ copy_shared_framebuffer_gpu (CoglOnscreen                        *onscreen,
     }
 
   if (!meta_egl_swap_buffers (egl,
-                              renderer_gpu_data->egl_display,
+                              egl_display,
                               secondary_gpu_state->egl_surface,
                               &error))
     {
@@ -704,11 +699,16 @@ copy_shared_framebuffer_gpu (CoglOnscreen                        *onscreen,
     }
 
   use_modifiers = meta_renderer_native_use_modifiers (renderer_native);
-  device_file = secondary_gpu_state->renderer_gpu_data->device_file;
+  device_file = meta_render_device_get_device_file (render_device);
+
+  flags = META_DRM_BUFFER_FLAG_NONE;
+  if (!use_modifiers)
+    flags |= META_DRM_BUFFER_FLAG_DISABLE_MODIFIERS;
+
   buffer_gbm =
     meta_drm_buffer_gbm_new_lock_front (device_file,
                                         secondary_gpu_state->gbm.surface,
-                                        use_modifiers,
+                                        flags,
                                         &error);
   if (!buffer_gbm)
     {
@@ -916,10 +916,10 @@ update_secondary_gpu_state_pre_swap_buffers (CoglOnscreen *onscreen,
   if (secondary_gpu_state)
     {
       MetaRendererNativeGpuData *renderer_gpu_data;
-      MetaDeviceFile *device_file;
+      MetaRenderDevice *render_device;
 
       renderer_gpu_data = secondary_gpu_state->renderer_gpu_data;
-      device_file = renderer_gpu_data->device_file;
+      render_device = renderer_gpu_data->render_device;
       switch (renderer_gpu_data->secondary.copy_mode)
         {
         case META_SHARED_FRAMEBUFFER_COPY_MODE_SECONDARY_GPU:
@@ -942,7 +942,7 @@ update_secondary_gpu_state_pre_swap_buffers (CoglOnscreen *onscreen,
                 {
                   meta_topic (META_DEBUG_KMS,
                               "Using primary GPU to copy for %s failed once.",
-                              meta_device_file_get_path (device_file));
+                              meta_render_device_get_name (render_device));
                   secondary_gpu_state->noted_primary_gpu_copy_failed = TRUE;
                 }
 
@@ -954,7 +954,7 @@ update_secondary_gpu_state_pre_swap_buffers (CoglOnscreen *onscreen,
             {
               meta_topic (META_DEBUG_KMS,
                           "Using primary GPU to copy for %s succeeded once.",
-                          meta_device_file_get_path (device_file));
+                          meta_render_device_get_name (render_device));
               secondary_gpu_state->noted_primary_gpu_copy_ok = TRUE;
             }
           break;
@@ -981,13 +981,18 @@ update_secondary_gpu_state_post_swap_buffers (CoglOnscreen *onscreen,
       renderer_gpu_data =
         meta_renderer_native_get_gpu_data (renderer_native,
                                            secondary_gpu_state->gpu_kms);
-retry:
       switch (renderer_gpu_data->secondary.copy_mode)
         {
         case META_SHARED_FRAMEBUFFER_COPY_MODE_ZERO:
-          if (!import_shared_framebuffer (onscreen,
-                                          secondary_gpu_state))
-            goto retry;
+          if (import_shared_framebuffer (onscreen, secondary_gpu_state))
+            break;
+
+          /* The fallback was prepared in pre_swap_buffers */
+          renderer_gpu_data->secondary.copy_mode =
+            META_SHARED_FRAMEBUFFER_COPY_MODE_PRIMARY;
+          G_GNUC_FALLTHROUGH;
+        case META_SHARED_FRAMEBUFFER_COPY_MODE_PRIMARY:
+          /* Done before eglSwapBuffers. */
           break;
         case META_SHARED_FRAMEBUFFER_COPY_MODE_SECONDARY_GPU:
           copy_shared_framebuffer_gpu (onscreen,
@@ -995,9 +1000,6 @@ retry:
                                        renderer_gpu_data,
                                        egl_context_changed);
           break;
-        case META_SHARED_FRAMEBUFFER_COPY_MODE_PRIMARY:
-          /* Done before eglSwapBuffers. */
-          break;
         }
     }
 }
@@ -1044,9 +1046,9 @@ meta_onscreen_native_swap_buffers_with_damage (CoglOnscreen  *onscreen,
   ClutterFrame *frame = user_data;
   CoglOnscreenClass *parent_class;
   gboolean egl_context_changed = FALSE;
-  gboolean use_modifiers;
   MetaPowerSave power_save_mode;
   g_autoptr (GError) error = NULL;
+  MetaDrmBufferFlags buffer_flags;
   MetaDrmBufferGbm *buffer_gbm;
   MetaKmsCrtc *kms_crtc;
   MetaKmsDevice *kms_device;
@@ -1070,18 +1072,22 @@ meta_onscreen_native_swap_buffers_with_damage (CoglOnscreen  *onscreen,
 
   renderer_gpu_data = meta_renderer_native_get_gpu_data (renderer_native,
                                                          render_gpu);
-  render_device_file = renderer_gpu_data->device_file;
+  render_device_file =
+    meta_render_device_get_device_file (renderer_gpu_data->render_device);
   switch (renderer_gpu_data->mode)
     {
     case META_RENDERER_NATIVE_MODE_GBM:
       g_warn_if_fail (onscreen_native->gbm.next_fb == NULL);
       g_clear_object (&onscreen_native->gbm.next_fb);
 
-      use_modifiers = meta_renderer_native_use_modifiers (renderer_native);
+      buffer_flags = META_DRM_BUFFER_FLAG_NONE;
+      if (!meta_renderer_native_use_modifiers (renderer_native))
+        buffer_flags |= META_DRM_BUFFER_FLAG_DISABLE_MODIFIERS;
+
       buffer_gbm =
         meta_drm_buffer_gbm_new_lock_front (render_device_file,
                                             onscreen_native->gbm.surface,
-                                            use_modifiers,
+                                            buffer_flags,
                                             &error);
       if (!buffer_gbm)
         {
@@ -1210,46 +1216,30 @@ meta_onscreen_native_swap_buffers_with_damage (CoglOnscreen  *onscreen,
 }
 
 gboolean
-meta_onscreen_native_is_buffer_scanout_compatible (CoglOnscreen *onscreen,
-                                                   uint32_t      drm_format,
-                                                   uint64_t      drm_modifier,
-                                                   uint32_t      stride)
+meta_onscreen_native_is_buffer_scanout_compatible (CoglOnscreen  *onscreen,
+                                                   MetaDrmBuffer *fb)
 {
   MetaOnscreenNative *onscreen_native = META_ONSCREEN_NATIVE (onscreen);
-  const MetaCrtcConfig *crtc_config;
-  MetaDrmBuffer *fb;
-  struct gbm_bo *gbm_bo;
+  MetaCrtc *crtc = onscreen_native->crtc;
+  MetaCrtcKms *crtc_kms = META_CRTC_KMS (crtc);
+  MetaGpuKms *gpu_kms;
+  MetaKmsDevice *kms_device;
+  MetaKms *kms;
+  MetaKmsUpdate *test_update;
+  g_autoptr (MetaKmsFeedback) kms_feedback = NULL;
+  MetaKmsFeedbackResult result;
 
-  crtc_config = meta_crtc_get_config (onscreen_native->crtc);
-  if (crtc_config->transform != META_MONITOR_TRANSFORM_NORMAL)
-    return FALSE;
+  gpu_kms = META_GPU_KMS (meta_crtc_get_gpu (crtc));
+  kms_device = meta_gpu_kms_get_kms_device (gpu_kms);
+  kms = meta_kms_device_get_kms (kms_device);
+  test_update = meta_kms_update_new (kms_device);
 
-  if (onscreen_native->secondary_gpu_state)
-    return FALSE;
+  meta_crtc_kms_assign_primary_plane (crtc_kms, fb, test_update);
+  kms_feedback = meta_kms_post_test_update_sync (kms, test_update);
+  meta_kms_update_free (test_update);
 
-  if (!onscreen_native->gbm.surface)
-    return FALSE;
-
-  fb = onscreen_native->gbm.current_fb ? onscreen_native->gbm.current_fb
-                                       : onscreen_native->gbm.next_fb;
-  if (!fb)
-    return FALSE;
-
-  if (!META_IS_DRM_BUFFER_GBM (fb))
-    return FALSE;
-
-  gbm_bo = meta_drm_buffer_gbm_get_bo (META_DRM_BUFFER_GBM (fb));
-
-  if (gbm_bo_get_format (gbm_bo) != drm_format)
-    return FALSE;
-
-  if (gbm_bo_get_modifier (gbm_bo) != drm_modifier)
-    return FALSE;
-
-  if (gbm_bo_get_stride (gbm_bo) != stride)
-    return FALSE;
-
-  return TRUE;
+  result = meta_kms_feedback_get_result (kms_feedback);
+  return result == META_KMS_FEEDBACK_PASSED;
 }
 
 static gboolean
@@ -1520,6 +1510,8 @@ get_supported_egl_modifiers (CoglOnscreen *onscreen,
   MetaEgl *egl = meta_onscreen_native_get_egl (onscreen_native);
   MetaGpu *gpu;
   MetaRendererNativeGpuData *renderer_gpu_data;
+  MetaRenderDevice *render_device;
+  EGLDisplay egl_display;
   EGLint num_modifiers;
   GArray *modifiers;
   GError *error = NULL;
@@ -1528,13 +1520,15 @@ get_supported_egl_modifiers (CoglOnscreen *onscreen,
   gpu = meta_crtc_get_gpu (META_CRTC (crtc_kms));
   renderer_gpu_data = meta_renderer_native_get_gpu_data (renderer_native,
                                                          META_GPU_KMS (gpu));
+  render_device = renderer_gpu_data->render_device;
+  egl_display = meta_render_device_get_egl_display (render_device);
 
-  if (!meta_egl_has_extensions (egl, renderer_gpu_data->egl_display, NULL,
+  if (!meta_egl_has_extensions (egl, egl_display, NULL,
                                 "EGL_EXT_image_dma_buf_import_modifiers",
                                 NULL))
     return NULL;
 
-  ret = meta_egl_query_dma_buf_modifiers (egl, renderer_gpu_data->egl_display,
+  ret = meta_egl_query_dma_buf_modifiers (egl, egl_display,
                                           format, 0, NULL, NULL,
                                           &num_modifiers, NULL);
   if (!ret || num_modifiers == 0)
@@ -1542,7 +1536,7 @@ get_supported_egl_modifiers (CoglOnscreen *onscreen,
 
   modifiers = g_array_sized_new (FALSE, FALSE, sizeof (uint64_t),
                                  num_modifiers);
-  ret = meta_egl_query_dma_buf_modifiers (egl, renderer_gpu_data->egl_display,
+  ret = meta_egl_query_dma_buf_modifiers (egl, egl_display,
                                           format, num_modifiers,
                                           (EGLuint64KHR *) modifiers->data, NULL,
                                           &num_modifiers, &error);
@@ -1603,6 +1597,8 @@ create_surfaces_gbm (CoglOnscreen        *onscreen,
   CoglRenderer *cogl_renderer = cogl_display->renderer;
   CoglRendererEGL *cogl_renderer_egl = cogl_renderer->winsys;
   MetaRendererNativeGpuData *renderer_gpu_data = cogl_renderer_egl->platform;
+  MetaRenderDeviceGbm *render_device_gbm;
+  struct gbm_device *gbm_device;
   struct gbm_surface *new_gbm_surface = NULL;
   EGLNativeWindowType egl_native_window;
   EGLSurface new_egl_surface;
@@ -1612,6 +1608,8 @@ create_surfaces_gbm (CoglOnscreen        *onscreen,
   renderer_gpu_data =
     meta_renderer_native_get_gpu_data (renderer_native,
                                        onscreen_native->render_gpu);
+  render_device_gbm = META_RENDER_DEVICE_GBM (renderer_gpu_data->render_device);
+  gbm_device = meta_render_device_gbm_get_gbm_device (render_device_gbm);
 
   format = get_gbm_format_from_egl (egl,
                                     cogl_renderer_egl->edpy,
@@ -1625,7 +1623,7 @@ create_surfaces_gbm (CoglOnscreen        *onscreen,
   if (modifiers)
     {
       new_gbm_surface =
-        gbm_surface_create_with_modifiers (renderer_gpu_data->gbm.device,
+        gbm_surface_create_with_modifiers (gbm_device,
                                            width, height, format,
                                            (uint64_t *) modifiers->data,
                                            modifiers->len);
@@ -1639,7 +1637,7 @@ create_surfaces_gbm (CoglOnscreen        *onscreen,
       if (should_surface_be_sharable (onscreen))
         flags |= GBM_BO_USE_LINEAR;
 
-      new_gbm_surface = gbm_surface_create (renderer_gpu_data->gbm.device,
+      new_gbm_surface = gbm_surface_create (gbm_device,
                                             width, height,
                                             format,
                                             flags);
@@ -1690,9 +1688,10 @@ create_surfaces_egl_device (CoglOnscreen  *onscreen,
   CoglRenderer *cogl_renderer = cogl_display->renderer;
   CoglRendererEGL *cogl_renderer_egl = cogl_renderer->winsys;
   MetaRendererNativeGpuData *renderer_gpu_data = cogl_renderer_egl->platform;
+  MetaRenderDevice *render_device;
   MetaEgl *egl =
     meta_renderer_native_get_egl (renderer_gpu_data->renderer_native);
-  EGLDisplay egl_display = renderer_gpu_data->egl_display;
+  EGLDisplay egl_display;
   EGLConfig egl_config;
   EGLStreamKHR egl_stream;
   EGLSurface egl_surface;
@@ -1710,6 +1709,8 @@ create_surfaces_egl_device (CoglOnscreen  *onscreen,
     EGL_NONE
   };
 
+  render_device = renderer_gpu_data->render_device;
+  egl_display = meta_render_device_get_egl_display (render_device);
   egl_stream = meta_egl_create_stream (egl, egl_display, stream_attribs, error);
   if (egl_stream == EGL_NO_STREAM_KHR)
     return FALSE;
@@ -1785,7 +1786,8 @@ meta_onscreen_native_allocate (CoglFramebuffer  *framebuffer,
   int width;
   int height;
 #ifdef HAVE_EGL_DEVICE
-  MetaDeviceFile *render_device_file;
+  MetaRenderDevice *render_device;
+  MetaDrmBuffer *dumb_buffer;
   EGLStreamKHR egl_stream;
 #endif
   CoglFramebufferClass *parent_class;
@@ -1822,15 +1824,16 @@ meta_onscreen_native_allocate (CoglFramebuffer  *framebuffer,
       break;
 #ifdef HAVE_EGL_DEVICE
     case META_RENDERER_NATIVE_MODE_EGL_DEVICE:
-      render_device_file = renderer_gpu_data->device_file;
-      onscreen_native->egl.dumb_fb =
-        meta_drm_buffer_dumb_new (render_device_file,
-                                  width, height,
-                                  DRM_FORMAT_XRGB8888,
-                                  error);
-      if (!onscreen_native->egl.dumb_fb)
+      render_device = renderer_gpu_data->render_device;
+      dumb_buffer = meta_render_device_allocate_dumb_buf (render_device,
+                                                          width, height,
+                                                          DRM_FORMAT_XRGB8888,
+                                                          error);
+      if (!dumb_buffer)
         return FALSE;
 
+      onscreen_native->egl.dumb_fb = META_DRM_BUFFER_DUMB (dumb_buffer);
+
       if (!create_surfaces_egl_device (onscreen,
                                        width, height,
                                        &egl_stream,
@@ -1857,6 +1860,10 @@ init_secondary_gpu_state_gpu_copy_mode (MetaRendererNative         *renderer_nat
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
   MetaOnscreenNative *onscreen_native = META_ONSCREEN_NATIVE (onscreen);
   MetaEgl *egl = meta_onscreen_native_get_egl (onscreen_native);
+  MetaRenderDevice *render_device;
+  MetaRenderDeviceGbm *render_device_gbm;
+  struct gbm_device *gbm_device;
+  EGLDisplay egl_display;
   int width, height;
   EGLNativeWindowType egl_native_window;
   struct gbm_surface *gbm_surface;
@@ -1865,13 +1872,17 @@ init_secondary_gpu_state_gpu_copy_mode (MetaRendererNative         *renderer_nat
   MetaGpuKms *gpu_kms;
   uint32_t format;
 
+  render_device = renderer_gpu_data->render_device;
+  egl_display = meta_render_device_get_egl_display (render_device);
   width = cogl_framebuffer_get_width (framebuffer);
   height = cogl_framebuffer_get_height (framebuffer);
   format = get_gbm_format_from_egl (egl,
-                                    renderer_gpu_data->egl_display,
+                                    egl_display,
                                     renderer_gpu_data->secondary.egl_config);
 
-  gbm_surface = gbm_surface_create (renderer_gpu_data->gbm.device,
+  render_device_gbm = META_RENDER_DEVICE_GBM (render_device);
+  gbm_device = meta_render_device_gbm_get_gbm_device (render_device_gbm);
+  gbm_surface = gbm_surface_create (gbm_device,
                                     width, height,
                                     format,
                                     GBM_BO_USE_SCANOUT | GBM_BO_USE_RENDERING);
@@ -1885,7 +1896,7 @@ init_secondary_gpu_state_gpu_copy_mode (MetaRendererNative         *renderer_nat
   egl_native_window = (EGLNativeWindowType) gbm_surface;
   egl_surface =
     meta_egl_create_window_surface (egl,
-                                    renderer_gpu_data->egl_display,
+                                    egl_display,
                                     renderer_gpu_data->secondary.egl_config,
                                     egl_native_window,
                                     NULL,
@@ -1979,8 +1990,8 @@ init_secondary_gpu_state_cpu_copy_mode (MetaRendererNative         *renderer_nat
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
   MetaOnscreenNative *onscreen_native = META_ONSCREEN_NATIVE (onscreen);
   MetaOnscreenNativeSecondaryGpuState *secondary_gpu_state;
+  MetaRenderDevice *render_device;
   MetaGpuKms *gpu_kms;
-  MetaDeviceFile *device_file;
   int width, height;
   unsigned int i;
   uint32_t drm_format;
@@ -1998,10 +2009,10 @@ init_secondary_gpu_state_cpu_copy_mode (MetaRendererNative         *renderer_nat
   height = cogl_framebuffer_get_height (framebuffer);
 
   gpu_kms = META_GPU_KMS (meta_crtc_get_gpu (onscreen_native->crtc));
-  device_file = renderer_gpu_data->device_file;
+  render_device = renderer_gpu_data->render_device;
   meta_topic (META_DEBUG_KMS,
               "Secondary GPU %s using DRM format '%s' (0x%x) for a %dx%d output.",
-              meta_device_file_get_path (device_file),
+              meta_render_device_get_name (render_device),
               meta_drm_format_to_string (&tmp, drm_format),
               drm_format,
               width, height);
@@ -2013,16 +2024,19 @@ init_secondary_gpu_state_cpu_copy_mode (MetaRendererNative         *renderer_nat
 
   for (i = 0; i < G_N_ELEMENTS (secondary_gpu_state->cpu.dumb_fbs); i++)
     {
-      secondary_gpu_state->cpu.dumb_fbs[i] =
-        meta_drm_buffer_dumb_new (device_file,
-                                  width, height,
-                                  drm_format,
-                                  error);
-      if (!secondary_gpu_state->cpu.dumb_fbs[i])
+      MetaDrmBuffer *dumb_buffer;
+
+      dumb_buffer = meta_render_device_allocate_dumb_buf (render_device,
+                                                          width, height,
+                                                          drm_format,
+                                                          error);
+      if (!dumb_buffer)
         {
           secondary_gpu_state_free (secondary_gpu_state);
           return FALSE;
         }
+
+      secondary_gpu_state->cpu.dumb_fbs[i] = META_DRM_BUFFER_DUMB (dumb_buffer);
     }
 
   /*
