@@ -32,7 +32,11 @@ gboolean meta_is_udev_device_platform_device (GUdevDevice *device);
 
 gboolean meta_is_udev_device_boot_vga (GUdevDevice *device);
 
-gboolean meta_is_udev_device_requires_modifiers (GUdevDevice *device);
+gboolean meta_is_udev_device_disable_modifiers (GUdevDevice *device);
+
+gboolean meta_is_udev_device_ignore (GUdevDevice *device);
+
+gboolean meta_is_udev_test_device (GUdevDevice *device);
 
 gboolean meta_is_udev_device_preferred_primary (GUdevDevice *device);
 
@@ -42,6 +46,10 @@ gboolean meta_udev_is_drm_device (MetaUdev    *udev,
 GList * meta_udev_list_drm_devices (MetaUdev  *udev,
                                     GError   **error);
 
+void meta_udev_pause (MetaUdev *udev);
+
+void meta_udev_resume (MetaUdev *udev);
+
 MetaUdev * meta_udev_new (MetaBackendNative *backend_native);
 
 #endif /* META_UDEV_H */
