@@ -30,16 +30,15 @@
 #include "backends/x11/cm/meta-backend-x11-cm.h"
 #include "backends/x11/cm/meta-renderer-x11-cm.h"
 #include "backends/x11/meta-backend-x11.h"
+#include "backends/x11/meta-clutter-backend-x11.h"
 #include "backends/x11/meta-seat-x11.h"
 #include "backends/x11/meta-stage-x11.h"
 #include "backends/x11/nested/meta-stage-x11-nested.h"
 #include "clutter/clutter-mutter.h"
-#include "clutter/x11/clutter-x11.h"
-#include "clutter/x11/clutter-backend-x11.h"
 #include "cogl/cogl-mutter.h"
 #include "cogl/cogl.h"
 #include "core/display-private.h"
-#include "meta/main.h"
+#include "meta/meta-context.h"
 #include "meta/meta-x11-errors.h"
 
 #define STAGE_X11_IS_MAPPED(s)  ((((MetaStageX11 *) (s))->wm_state & STAGE_X11_WITHDRAWN) == 0)
@@ -49,13 +48,13 @@ static ClutterStageWindowInterface *clutter_stage_window_parent_iface = NULL;
 static void
 clutter_stage_window_iface_init (ClutterStageWindowInterface *iface);
 
-static ClutterStageCogl *meta_x11_get_stage_window_from_window (Window win);
+static MetaStageImpl *meta_x11_get_stage_window_from_window (Window win);
 
 static GHashTable *clutter_stages_by_xid = NULL;
 
 G_DEFINE_TYPE_WITH_CODE (MetaStageX11,
                          meta_stage_x11,
-                         CLUTTER_TYPE_STAGE_COGL,
+                         META_TYPE_STAGE_IMPL,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_STAGE_WINDOW,
                                                 clutter_stage_window_iface_init));
 
@@ -76,31 +75,39 @@ G_DEFINE_TYPE_WITH_CODE (MetaStageX11,
   ButtonReleaseMask | \
   PointerMotionMask
 
+static MetaClutterBackendX11 *
+clutter_backend_x11_from_stage (MetaStageX11 *stage_x11)
+{
+  MetaBackend *backend =
+    meta_stage_impl_get_backend (META_STAGE_IMPL (stage_x11));
+
+  return META_CLUTTER_BACKEND_X11 (meta_backend_get_clutter_backend (backend));
+}
+
+static Display *
+xdisplay_from_stage (MetaStageX11 *stage_x11)
+{
+  MetaBackend *backend =
+    meta_stage_impl_get_backend (META_STAGE_IMPL (stage_x11));
+
+  return meta_backend_x11_get_xdisplay (META_BACKEND_X11 (backend));
+}
+
 static void
 meta_stage_x11_fix_window_size (MetaStageX11 *stage_x11,
                                 int           new_width,
                                 int           new_height)
 {
-  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
+  g_return_if_fail (new_width > 0);
+  g_return_if_fail (new_height > 0);
 
   if (stage_x11->xwin != None)
     {
-      Display *xdisplay = clutter_x11_get_default_display ();
-      uint32_t min_width, min_height;
+      Display *xdisplay = xdisplay_from_stage (stage_x11);
       XSizeHints *size_hints;
 
       size_hints = XAllocSizeHints();
 
-      clutter_stage_get_minimum_size (stage_cogl->wrapper,
-                                      &min_width,
-                                      &min_height);
-
-      if (new_width <= 0)
-        new_width = min_width;
-
-      if (new_height <= 0)
-        new_height = min_height;
-
       size_hints->min_width = new_width;
       size_hints->min_height = new_height;
       size_hints->max_width = new_width;
@@ -116,14 +123,14 @@ meta_stage_x11_fix_window_size (MetaStageX11 *stage_x11,
 static void
 meta_stage_x11_set_wm_protocols (MetaStageX11 *stage_x11)
 {
-  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (stage_cogl->backend);
-  Display *xdisplay = clutter_x11_get_default_display ();
+  MetaClutterBackendX11 *clutter_backend_x11 =
+    clutter_backend_x11_from_stage (stage_x11);
+  Display *xdisplay = xdisplay_from_stage (stage_x11);
   Atom protocols[2];
   int n = 0;
   
-  protocols[n++] = backend_x11->atom_WM_DELETE_WINDOW;
-  protocols[n++] = backend_x11->atom_NET_WM_PING;
+  protocols[n++] = clutter_backend_x11->atom_WM_DELETE_WINDOW;
+  protocols[n++] = clutter_backend_x11->atom_NET_WM_PING;
 
   XSetWMProtocols (xdisplay, stage_x11->xwin, protocols, n);
 }
@@ -163,7 +170,7 @@ meta_stage_x11_resize (ClutterStageWindow *stage_window,
       if (width != stage_x11->xwin_width ||
           height != stage_x11->xwin_height)
         {
-          Display *xdisplay = clutter_x11_get_default_display ();
+          Display *xdisplay = xdisplay_from_stage (stage_x11);
 
           /* XXX: in this case we can rely on a subsequent
            * ConfigureNotify that will result in the stage
@@ -188,9 +195,12 @@ meta_stage_x11_resize (ClutterStageWindow *stage_window,
 static inline void
 set_wm_pid (MetaStageX11 *stage_x11)
 {
-  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (stage_cogl->backend);
-  Display *xdisplay = clutter_x11_get_default_display ();
+  MetaStageImpl *stage_impl = META_STAGE_IMPL (stage_x11);
+  MetaBackend *backend = meta_stage_impl_get_backend (stage_impl);
+  MetaBackendX11 *backend_x11 = META_BACKEND_X11 (backend);
+  MetaClutterBackendX11 *clutter_backend_x11 =
+    clutter_backend_x11_from_stage (stage_x11);
+  Display *xdisplay = meta_backend_x11_get_xdisplay (backend_x11);
   long pid;
 
   if (stage_x11->xwin == None)
@@ -206,7 +216,7 @@ set_wm_pid (MetaStageX11 *stage_x11)
   pid = getpid ();
   XChangeProperty (xdisplay,
                    stage_x11->xwin,
-                   backend_x11->atom_NET_WM_PID, XA_CARDINAL, 32,
+                   clutter_backend_x11->atom_NET_WM_PID, XA_CARDINAL, 32,
                    PropModeReplace,
                    (guchar *) &pid, 1);
 }
@@ -214,9 +224,9 @@ set_wm_pid (MetaStageX11 *stage_x11)
 static inline void
 set_wm_title (MetaStageX11 *stage_x11)
 {
-  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (stage_cogl->backend);
-  Display *xdisplay = clutter_x11_get_default_display ();
+  MetaClutterBackendX11 *clutter_backend_x11 =
+    clutter_backend_x11_from_stage (stage_x11);
+  Display *xdisplay = xdisplay_from_stage (stage_x11);
 
   if (stage_x11->xwin == None)
     return;
@@ -225,14 +235,14 @@ set_wm_title (MetaStageX11 *stage_x11)
     {
       XDeleteProperty (xdisplay,
                        stage_x11->xwin, 
-                       backend_x11->atom_NET_WM_NAME);
+                       clutter_backend_x11->atom_NET_WM_NAME);
     }
   else
     {
       XChangeProperty (xdisplay,
                        stage_x11->xwin, 
-                       backend_x11->atom_NET_WM_NAME,
-                       backend_x11->atom_UTF8_STRING,
+                       clutter_backend_x11->atom_NET_WM_NAME,
+                       clutter_backend_x11->atom_UTF8_STRING,
                        8, 
                        PropModeReplace, 
                        (unsigned char *) stage_x11->title,
@@ -292,20 +302,24 @@ static gboolean
 meta_stage_x11_realize (ClutterStageWindow *stage_window)
 {
   MetaStageX11 *stage_x11 = META_STAGE_X11 (stage_window);
-  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
-  ClutterBackend *backend = CLUTTER_BACKEND (stage_cogl->backend);
-  MetaSeatX11 *seat_x11 = META_SEAT_X11 (clutter_backend_get_default_seat (backend));
-  Display *xdisplay = clutter_x11_get_default_display ();
+  MetaStageImpl *stage_impl = META_STAGE_IMPL (stage_window);
+  MetaBackend *backend = meta_stage_impl_get_backend (stage_impl);
+  MetaBackendX11 *backend_x11 = META_BACKEND_X11 (backend);
+  ClutterBackend *clutter_backend = meta_backend_get_clutter_backend (backend);
+  MetaSeatX11 *seat_x11 =
+    META_SEAT_X11 (meta_backend_get_default_seat (backend));
+  Display *xdisplay = meta_backend_x11_get_xdisplay (backend_x11);
   float width, height;
   GError *error = NULL;
 
-  clutter_actor_get_size (CLUTTER_ACTOR (stage_cogl->wrapper), &width, &height);
+  clutter_actor_get_size (CLUTTER_ACTOR (stage_impl->wrapper), &width, &height);
 
-  stage_x11->onscreen = create_onscreen (backend->cogl_context, width, height);
+  stage_x11->onscreen = create_onscreen (clutter_backend->cogl_context,
+                                         width, height);
 
-  if (META_IS_BACKEND_X11_CM (stage_x11->backend))
+  if (META_IS_BACKEND_X11_CM (backend))
     {
-      MetaRenderer *renderer = meta_backend_get_renderer (stage_x11->backend);
+      MetaRenderer *renderer = meta_backend_get_renderer (backend);
       MetaRendererX11Cm *renderer_x11_cm = META_RENDERER_X11_CM (renderer);
 
       meta_renderer_x11_cm_init_screen_view (renderer_x11_cm,
@@ -345,7 +359,7 @@ meta_stage_x11_realize (ClutterStageWindow *stage_window)
 
   /* we unconditionally select input events even with event retrieval
    * disabled because we need to guarantee that the Clutter internal
-   * state is maintained when calling clutter_x11_handle_event() without
+   * state is maintained when calling meta_clutter_x11_handle_event() without
    * requiring applications or embedding toolkits to select events
    * themselves. if we did that, we'd have to document the events to be
    * selected, and also update applications and embedding toolkits each
@@ -362,7 +376,7 @@ meta_stage_x11_realize (ClutterStageWindow *stage_window)
    */
   XSelectInput (xdisplay, stage_x11->xwin, META_STAGE_X11_EVENT_MASK);
 
-  meta_seat_x11_select_stage_events (seat_x11, stage_cogl->wrapper);
+  meta_seat_x11_select_stage_events (seat_x11, stage_impl->wrapper);
 
   meta_stage_x11_fix_window_size (stage_x11,
                                   stage_x11->xwin_width,
@@ -386,7 +400,7 @@ meta_stage_x11_set_title (ClutterStageWindow *stage_window,
 static inline void
 update_wm_hints (MetaStageX11 *stage_x11)
 {
-  Display *xdisplay = clutter_x11_get_default_display ();
+  Display *xdisplay = xdisplay_from_stage (stage_x11);
   XWMHints wm_hints;
 
   if (stage_x11->wm_state & STAGE_X11_WITHDRAWN)
@@ -423,11 +437,11 @@ meta_stage_x11_show (ClutterStageWindow *stage_window,
                      gboolean            do_raise)
 {
   MetaStageX11 *stage_x11 = META_STAGE_X11 (stage_window);
-  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
+  MetaStageImpl *stage_impl = META_STAGE_IMPL (stage_x11);
 
   if (stage_x11->xwin != None)
     {
-      Display *xdisplay = clutter_x11_get_default_display ();
+      Display *xdisplay = xdisplay_from_stage (stage_x11);
 
       if (do_raise)
         {
@@ -443,7 +457,7 @@ meta_stage_x11_show (ClutterStageWindow *stage_window,
 
       g_assert (STAGE_X11_IS_MAPPED (stage_x11));
 
-      clutter_actor_map (CLUTTER_ACTOR (stage_cogl->wrapper));
+      clutter_actor_map (CLUTTER_ACTOR (stage_impl->wrapper));
 
       XMapWindow (xdisplay, stage_x11->xwin);
     }
@@ -453,18 +467,18 @@ static void
 meta_stage_x11_hide (ClutterStageWindow *stage_window)
 {
   MetaStageX11 *stage_x11 = META_STAGE_X11 (stage_window);
-  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
+  MetaStageImpl *stage_impl = META_STAGE_IMPL (stage_x11);
 
   if (stage_x11->xwin != None)
     {
-      Display *xdisplay = clutter_x11_get_default_display ();
+      Display *xdisplay = xdisplay_from_stage (stage_x11);
 
       if (STAGE_X11_IS_MAPPED (stage_x11))
         set_stage_x11_state (stage_x11, 0, STAGE_X11_WITHDRAWN);
 
       g_assert (!STAGE_X11_IS_MAPPED (stage_x11));
 
-      clutter_actor_unmap (CLUTTER_ACTOR (stage_cogl->wrapper));
+      clutter_actor_unmap (CLUTTER_ACTOR (stage_impl->wrapper));
 
       XWithdrawWindow (xdisplay, stage_x11->xwin, 0);
     }
@@ -550,17 +564,19 @@ clutter_stage_window_iface_init (ClutterStageWindowInterface *iface)
 }
 
 static inline void
-set_user_time (ClutterBackendX11 *backend_x11,
-               MetaStageX11      *stage_x11,
-               long               timestamp)
+set_user_time (MetaStageX11 *stage_x11,
+               long          timestamp)
 {
+  MetaClutterBackendX11 *clutter_backend_x11 =
+    clutter_backend_x11_from_stage (stage_x11);
+
   if (timestamp != CLUTTER_CURRENT_TIME)
     {
-      Display *xdisplay = clutter_x11_get_default_display ();
+      Display *xdisplay = xdisplay_from_stage (stage_x11);
 
       XChangeProperty (xdisplay,
                        stage_x11->xwin,
-                       backend_x11->atom_NET_WM_USER_TIME,
+                       clutter_backend_x11->atom_NET_WM_USER_TIME,
                        XA_CARDINAL, 32,
                        PropModeReplace,
                        (unsigned char *) &timestamp, 1);
@@ -568,26 +584,31 @@ set_user_time (ClutterBackendX11 *backend_x11,
 }
 
 static gboolean
-handle_wm_protocols_event (ClutterBackendX11 *backend_x11,
-                           MetaStageX11      *stage_x11,
-                           XEvent            *xevent)
+handle_wm_protocols_event (MetaStageX11 *stage_x11,
+                           XEvent       *xevent)
 {
+  MetaClutterBackendX11 *clutter_backend_x11 =
+    clutter_backend_x11_from_stage (stage_x11);
   Atom atom = (Atom) xevent->xclient.data.l[0];
 
-  if (atom == backend_x11->atom_WM_DELETE_WINDOW &&
+  if (atom == clutter_backend_x11->atom_WM_DELETE_WINDOW &&
       xevent->xany.window == stage_x11->xwin)
     {
-      set_user_time (backend_x11, stage_x11, xevent->xclient.data.l[1]);
+      set_user_time (stage_x11, xevent->xclient.data.l[1]);
 
       return TRUE;
     }
-  else if (atom == backend_x11->atom_NET_WM_PING &&
+  else if (atom == clutter_backend_x11->atom_NET_WM_PING &&
            xevent->xany.window == stage_x11->xwin)
     {
+      MetaBackend *backend =
+        meta_stage_impl_get_backend (META_STAGE_IMPL (stage_x11));
+      MetaBackendX11 *backend_x11 = META_BACKEND_X11 (backend);
+      Display *xdisplay = meta_backend_x11_get_xdisplay (backend_x11);
+      Window root_xwindow = meta_backend_x11_get_root_xwindow (backend_x11);
       XClientMessageEvent xclient = xevent->xclient;
-      Display *xdisplay = clutter_x11_get_default_display ();
 
-      xclient.window = backend_x11->xwin_root;
+      xclient.window = root_xwindow;
       XSendEvent (xdisplay, xclient.window,
                   False,
                   SubstructureRedirectMask | SubstructureNotifyMask,
@@ -614,17 +635,19 @@ meta_stage_x11_translate_event (MetaStageX11 *stage_x11,
                                 XEvent       *xevent,
                                 ClutterEvent *event)
 {
-  ClutterStageCogl *stage_cogl;
+  MetaClutterBackendX11 *clutter_backend_x11 =
+    clutter_backend_x11_from_stage (stage_x11);
+  MetaBackend *backend;
+  MetaStageImpl *stage_impl;
   gboolean res = FALSE;
-  ClutterBackendX11 *backend_x11;
   ClutterStage *stage;
 
-  stage_cogl = meta_x11_get_stage_window_from_window (xevent->xany.window);
-  if (stage_cogl == NULL)
+  stage_impl = meta_x11_get_stage_window_from_window (xevent->xany.window);
+  if (stage_impl == NULL)
     return FALSE;
 
-  stage = stage_cogl->wrapper;
-  backend_x11 = CLUTTER_BACKEND_X11 (stage_cogl->backend);
+  backend = meta_stage_impl_get_backend (stage_impl);
+  stage = stage_impl->wrapper;
 
   switch (xevent->type)
     {
@@ -650,7 +673,7 @@ meta_stage_x11_translate_event (MetaStageX11 *stage_x11,
           stage_width = xevent->xconfigure.width;
           stage_height = xevent->xconfigure.height;
 
-          if (META_IS_BACKEND_X11_CM (stage_x11->backend))
+          if (META_IS_BACKEND_X11_CM (backend))
             {
               clutter_actor_set_size (CLUTTER_ACTOR (stage),
                                       stage_width,
@@ -721,9 +744,8 @@ meta_stage_x11_translate_event (MetaStageX11 *stage_x11,
                * X11 compositing manager, we need to reset the legacy
                * stage view, now that it has a new size.
                */
-              if (META_IS_BACKEND_X11_CM (stage_x11->backend))
+              if (META_IS_BACKEND_X11_CM (backend))
                 {
-                  MetaBackend *backend = stage_x11->backend;
                   MetaRenderer *renderer = meta_backend_get_renderer (backend);
                   MetaRendererX11Cm *renderer_x11_cm =
                     META_RENDERER_X11_CM (renderer);
@@ -737,11 +759,11 @@ meta_stage_x11_translate_event (MetaStageX11 *stage_x11,
       break;
 
     case FocusIn:
-      meta_stage_set_active ((MetaStage *) stage_cogl->wrapper, TRUE);
+      meta_stage_set_active ((MetaStage *) stage_impl->wrapper, TRUE);
       break;
 
     case FocusOut:
-      meta_stage_set_active ((MetaStage *) stage_cogl->wrapper, FALSE);
+      meta_stage_set_active ((MetaStage *) stage_impl->wrapper, FALSE);
       break;
 
     case Expose:
@@ -771,7 +793,7 @@ meta_stage_x11_translate_event (MetaStageX11 *stage_x11,
 
       g_return_val_if_fail (META_IS_STAGE_X11_NESTED (stage_x11),
                             FALSE);
-      meta_quit (META_EXIT_SUCCESS);
+      meta_context_terminate (meta_backend_get_context (backend));
       res = FALSE;
       break;
 
@@ -779,13 +801,13 @@ meta_stage_x11_translate_event (MetaStageX11 *stage_x11,
       g_debug ("Client message for stage, win:0x%x",
                (unsigned int) xevent->xany.window);
 
-      if (xevent->xclient.message_type == backend_x11->atom_WM_PROTOCOLS)
+      if (xevent->xclient.message_type == clutter_backend_x11->atom_WM_PROTOCOLS)
         {
-          if (handle_wm_protocols_event (backend_x11, stage_x11, xevent))
+          if (handle_wm_protocols_event (stage_x11, xevent))
             {
               g_return_val_if_fail (META_IS_STAGE_X11_NESTED (stage_x11),
                                     FALSE);
-              meta_quit (META_EXIT_SUCCESS);
+              meta_context_terminate (meta_backend_get_context (backend));
               res = FALSE;
             }
         }
@@ -813,7 +835,7 @@ meta_x11_get_stage_window (ClutterStage *stage)
   return META_STAGE_X11 (impl)->xwin;
 }
 
-static ClutterStageCogl *
+static MetaStageImpl *
 meta_x11_get_stage_window_from_window (Window win)
 {
   if (clutter_stages_by_xid == NULL)
@@ -826,12 +848,12 @@ meta_x11_get_stage_window_from_window (Window win)
 ClutterStage *
 meta_x11_get_stage_from_window (Window win)
 {
-  ClutterStageCogl *stage_cogl;
+  MetaStageImpl *stage_impl;
 
-  stage_cogl = meta_x11_get_stage_window_from_window (win);
+  stage_impl = meta_x11_get_stage_window_from_window (win);
 
-  if (stage_cogl != NULL)
-    return stage_cogl->wrapper;
+  if (stage_impl != NULL)
+    return stage_impl->wrapper;
 
   return NULL;
 }
@@ -840,8 +862,5 @@ void
 meta_stage_x11_set_user_time (MetaStageX11 *stage_x11,
                               uint32_t      user_time)
 {
-  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_x11);
-  ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (stage_cogl->backend);
-
-  set_user_time (backend_x11, stage_x11, user_time);
+  set_user_time (stage_x11, user_time);
 }
