@@ -30,7 +30,6 @@
 
 #include "backends/meta-backend-private.h"
 #include "backends/meta-logical-monitor.h"
-#include "cogl/cogl-wayland-server.h"
 #include "compositor/meta-shaped-texture-private.h"
 #include "compositor/region-utils.h"
 #include "wayland/meta-wayland-buffer.h"
@@ -73,7 +72,15 @@ meta_surface_actor_wayland_try_acquire_scanout (MetaSurfaceActorWayland *self,
   MetaWaylandSurface *surface;
   CoglScanout *scanout;
 
+  /* If the actor appears to be obscured, need to go through the normal paint
+   * machinery to ensure the unobscured region is up to date
+   */
+  if (meta_surface_actor_is_obscured (META_SURFACE_ACTOR (self)))
+    return NULL;
+
   surface = meta_surface_actor_wayland_get_surface (self);
+  g_return_val_if_fail (surface, NULL);
+
   scanout = meta_wayland_surface_try_acquire_scanout (surface, onscreen);
   if (!scanout)
     return NULL;
