@@ -429,12 +429,12 @@ merge_spanning_rects_in_region (GList *region)
    * considerations that also apply to this function.
    */
 
-  GList* compare;
+  GList *compare;
   compare = region;
 
   if (region == NULL)
     {
-      g_warning ("Region to merge was empty!  Either you have a some "
+      g_warning ("Region to merge was empty! Either you have some "
                  "pathological STRUT list or there's a bug somewhere!");
       return NULL;
     }
@@ -899,6 +899,25 @@ meta_rectangle_overlaps_with_region (const GList         *spanning_rects,
   return overlaps;
 }
 
+gboolean
+meta_rectangle_is_adjacent_to_any_in_region (const GList   *spanning_rects,
+                                             MetaRectangle *rect)
+{
+  const GList *l;
+
+  for (l = spanning_rects; l; l = l->next)
+    {
+      MetaRectangle *other = (MetaRectangle *) l->data;
+
+      if (rect == other || meta_rectangle_equal (rect, other))
+        continue;
+
+      if (meta_rectangle_is_adjacent_to (rect, other))
+        return TRUE;
+    }
+
+  return FALSE;
+}
 
 void
 meta_rectangle_clamp_to_fit_into_region (const GList         *spanning_rects,
