@@ -22,6 +22,8 @@
 
 #define _XOPEN_SOURCE /* for kill() */
 
+#define MAX_QUEUED_EVENTS 400
+
 #include "config.h"
 
 #include <errno.h>
@@ -39,6 +41,8 @@ close_dialog_response_cb (MetaCloseDialog         *dialog,
 {
   if (response == META_CLOSE_DIALOG_RESPONSE_FORCE_CLOSE)
     meta_window_kill (window);
+  else
+    meta_window_ensure_close_dialog_timeout (window);
 }
 
 static void
@@ -57,23 +61,22 @@ meta_window_ensure_close_dialog (MetaWindow *window)
 }
 
 void
-meta_window_set_alive (MetaWindow *window,
-                       gboolean    is_alive)
+meta_window_show_close_dialog (MetaWindow *window)
 {
-  if (is_alive && window->close_dialog)
-    {
-      meta_close_dialog_hide (window->close_dialog);
-    }
-  else if (!is_alive)
-    {
-      meta_window_ensure_close_dialog (window);
-      meta_close_dialog_show (window->close_dialog);
+  meta_window_ensure_close_dialog (window);
+  meta_close_dialog_show (window->close_dialog);
 
-      if (window->display &&
-          window->display->event_route == META_EVENT_ROUTE_NORMAL &&
-          window == window->display->focus_window)
-        meta_close_dialog_focus (window->close_dialog);
-    }
+  if (window->display &&
+      window->display->event_route == META_EVENT_ROUTE_NORMAL &&
+      window == window->display->focus_window)
+    meta_close_dialog_focus (window->close_dialog);
+}
+
+void
+meta_window_hide_close_dialog (MetaWindow *window)
+{
+  if (window->close_dialog)
+    meta_close_dialog_hide (window->close_dialog);
 }
 
 void
@@ -83,6 +86,21 @@ meta_window_check_alive (MetaWindow *window,
   meta_display_ping_window (window, timestamp);
 }
 
+void
+meta_window_check_alive_on_event (MetaWindow *window,
+                                  uint32_t    timestamp)
+{
+  if (!meta_window_can_ping (window))
+    return;
+
+  meta_display_ping_window (window, timestamp);
+
+  window->events_during_ping++;
+
+  if (window->events_during_ping > MAX_QUEUED_EVENTS)
+    meta_window_set_alive (window, FALSE);
+}
+
 void
 meta_window_delete (MetaWindow  *window,
                     guint32      timestamp)
@@ -100,14 +118,14 @@ meta_window_kill (MetaWindow *window)
   if (pid > 0)
     {
       meta_topic (META_DEBUG_WINDOW_OPS,
-                  "Killing %s with kill()\n",
+                  "Killing %s with kill()",
                   window->desc);
 
       if (kill (pid, 9) == 0)
         return;
 
       meta_topic (META_DEBUG_WINDOW_OPS,
-                  "Failed to signal %s: %s\n",
+                  "Failed to signal %s: %s",
                   window->desc, strerror (errno));
     }
 
