@@ -31,13 +31,10 @@
 #include <X11/extensions/sync.h>
 #include <X11/Xlib.h>
 
-#ifdef HAVE_STARTUP_NOTIFICATION
-#include <libsn/sn.h>
-#endif
-
 #include "clutter/clutter.h"
 #include "core/keybindings-private.h"
 #include "core/meta-gesture-tracker-private.h"
+#include "core/meta-pad-action-mapper.h"
 #include "core/stack-tracker.h"
 #include "core/startup-notification-private.h"
 #include "meta/barrier.h"
@@ -192,7 +189,7 @@ struct _MetaDisplay
   gboolean    grab_threshold_movement_reached; /* raise_on_click == FALSE.    */
   int64_t     grab_last_moveresize_time;
   MetaEdgeResistanceData *grab_edge_resistance_data;
-  unsigned int grab_last_user_action_was_snap;
+  unsigned int grab_last_edge_resistance_flags;
 
   int	      grab_resize_timeout_id;
 
@@ -200,6 +197,7 @@ struct _MetaDisplay
 
   /* Opening the display */
   unsigned int display_opening : 1;
+  unsigned int grabbed_in_clutter : 1;
 
   /* Closing down the display */
   int closing;
@@ -211,6 +209,7 @@ struct _MetaDisplay
   ClutterEventSequence *pointer_emulating_sequence;
 
   ClutterActor *current_pad_osd;
+  MetaPadActionMapper *pad_action_mapper;
 
   MetaStartupNotification *startup_notification;
 
@@ -258,7 +257,8 @@ struct _MetaDisplayClass
      (time2) != 0)                                                      \
   )
 
-gboolean      meta_display_open                (void);
+MetaDisplay * meta_display_new (MetaContext  *context,
+                                GError      **error);
 
 void meta_display_manage_all_xwindows (MetaDisplay *display);
 void meta_display_unmanage_windows   (MetaDisplay *display,
@@ -432,4 +432,12 @@ gboolean meta_display_init_x11_finish (MetaDisplay   *display,
 
 void     meta_display_shutdown_x11 (MetaDisplay  *display);
 
+void meta_display_queue_window (MetaDisplay   *display,
+                                MetaWindow    *window,
+                                MetaQueueType  queue_types);
+
+void meta_display_unqueue_window (MetaDisplay   *display,
+                                  MetaWindow    *window,
+                                  MetaQueueType  queue_types);
+
 #endif
