@@ -21,13 +21,17 @@
 #define META_COMPOSITOR_H
 
 #include <glib.h>
-#include <X11/Xlib.h>
 
 #include <meta/types.h>
 #include <meta/boxes.h>
 #include <meta/window.h>
 #include <meta/workspace.h>
 
+#define META_TYPE_COMPOSITOR (meta_compositor_get_type ())
+META_EXPORT
+G_DECLARE_DERIVABLE_TYPE (MetaCompositor, meta_compositor,
+                          META, COMPOSITOR, GObject)
+
 /**
  * MetaCompEffect:
  * @META_COMP_EFFECT_CREATE: The window is newly created
@@ -62,9 +66,6 @@ typedef enum
   META_SIZE_CHANGE_UNFULLSCREEN,
 } MetaSizeChange;
 
-META_EXPORT
-MetaCompositor *meta_compositor_new     (MetaDisplay    *display);
-
 META_EXPORT
 void            meta_compositor_destroy (MetaCompositor *compositor);
 
@@ -82,11 +83,6 @@ META_EXPORT
 void meta_compositor_window_opacity_changed (MetaCompositor *compositor,
                                              MetaWindow     *window);
 
-META_EXPORT
-gboolean meta_compositor_process_event (MetaCompositor *compositor,
-                                        XEvent         *event,
-                                        MetaWindow     *window);
-
 META_EXPORT
 gboolean meta_compositor_filter_keybinding (MetaCompositor *compositor,
                                             MetaKeyBinding *binding);
@@ -166,4 +162,7 @@ void meta_compositor_show_window_menu_for_rect (MetaCompositor     *compositor,
 				                MetaWindowMenuType  menu,
                                                 MetaRectangle      *rect);
 
+META_EXPORT
+MetaLaters * meta_compositor_get_laters (MetaCompositor *compositor);
+
 #endif /* META_COMPOSITOR_H */
