@@ -32,23 +32,12 @@
 META_EXPORT
 gboolean meta_is_verbose  (void);
 
-META_EXPORT
-gboolean meta_is_debugging (void);
-
 META_EXPORT
 gboolean meta_is_syncing (void);
 
 META_EXPORT
 gboolean meta_is_wayland_compositor (void);
 
-META_EXPORT
-void meta_debug_spew_real (const char *format,
-                           ...) G_GNUC_PRINTF (1, 2);
-
-META_EXPORT
-void meta_verbose_real    (const char *format,
-                           ...) G_GNUC_PRINTF (1, 2);
-
 META_EXPORT
 void meta_bug        (const char *format,
                       ...) G_GNUC_PRINTF (1, 2);
@@ -67,7 +56,6 @@ void meta_fatal      (const char *format,
  * @META_DEBUG_FOCUS: focus
  * @META_DEBUG_WORKAREA: workarea
  * @META_DEBUG_STACK: stack
- * @META_DEBUG_THEMES: themes
  * @META_DEBUG_SM: session management
  * @META_DEBUG_EVENTS: events
  * @META_DEBUG_WINDOW_STATE: window state
@@ -75,17 +63,20 @@ void meta_fatal      (const char *format,
  * @META_DEBUG_GEOMETRY: geometry
  * @META_DEBUG_PLACEMENT: window placement
  * @META_DEBUG_PING: ping
- * @META_DEBUG_XINERAMA: Xinerama
  * @META_DEBUG_KEYBINDINGS: keybindings
  * @META_DEBUG_SYNC: sync
- * @META_DEBUG_ERRORS: errors
  * @META_DEBUG_STARTUP: startup
  * @META_DEBUG_PREFS: preferences
  * @META_DEBUG_GROUPS: groups
  * @META_DEBUG_RESIZING: resizing
  * @META_DEBUG_SHAPES: shapes
- * @META_DEBUG_COMPOSITOR: compositor
  * @META_DEBUG_EDGE_RESISTANCE: edge resistance
+ * @META_DEBUG_WAYLAND: Wayland
+ * @META_DEBUG_KMS: kernel mode setting
+ * @META_DEBUG_SCREEN_CAST: screencasting
+ * @META_DEBUG_REMOTE_DESKTOP: remote desktop
+ * @META_DEBUG_BACKEND: backend
+ * @META_DEBUG_RENDER: native backend rendering
  */
 typedef enum
 {
@@ -93,27 +84,29 @@ typedef enum
   META_DEBUG_FOCUS           = 1 << 0,
   META_DEBUG_WORKAREA        = 1 << 1,
   META_DEBUG_STACK           = 1 << 2,
-  META_DEBUG_THEMES          = 1 << 3,
-  META_DEBUG_SM              = 1 << 4,
-  META_DEBUG_EVENTS          = 1 << 5,
-  META_DEBUG_WINDOW_STATE    = 1 << 6,
-  META_DEBUG_WINDOW_OPS      = 1 << 7,
-  META_DEBUG_GEOMETRY        = 1 << 8,
-  META_DEBUG_PLACEMENT       = 1 << 9,
-  META_DEBUG_PING            = 1 << 10,
-  META_DEBUG_XINERAMA        = 1 << 11,
-  META_DEBUG_KEYBINDINGS     = 1 << 12,
-  META_DEBUG_SYNC            = 1 << 13,
-  META_DEBUG_ERRORS          = 1 << 14,
-  META_DEBUG_STARTUP         = 1 << 15,
-  META_DEBUG_PREFS           = 1 << 16,
-  META_DEBUG_GROUPS          = 1 << 17,
-  META_DEBUG_RESIZING        = 1 << 18,
-  META_DEBUG_SHAPES          = 1 << 19,
-  META_DEBUG_COMPOSITOR      = 1 << 20,
-  META_DEBUG_EDGE_RESISTANCE = 1 << 21,
-  META_DEBUG_DBUS            = 1 << 22,
-  META_DEBUG_INPUT           = 1 << 23
+  META_DEBUG_SM              = 1 << 3,
+  META_DEBUG_EVENTS          = 1 << 4,
+  META_DEBUG_WINDOW_STATE    = 1 << 5,
+  META_DEBUG_WINDOW_OPS      = 1 << 6,
+  META_DEBUG_GEOMETRY        = 1 << 7,
+  META_DEBUG_PLACEMENT       = 1 << 8,
+  META_DEBUG_PING            = 1 << 9,
+  META_DEBUG_KEYBINDINGS     = 1 << 10,
+  META_DEBUG_SYNC            = 1 << 11,
+  META_DEBUG_STARTUP         = 1 << 12,
+  META_DEBUG_PREFS           = 1 << 13,
+  META_DEBUG_GROUPS          = 1 << 14,
+  META_DEBUG_RESIZING        = 1 << 15,
+  META_DEBUG_SHAPES          = 1 << 16,
+  META_DEBUG_EDGE_RESISTANCE = 1 << 17,
+  META_DEBUG_DBUS            = 1 << 18,
+  META_DEBUG_INPUT           = 1 << 19,
+  META_DEBUG_WAYLAND         = 1 << 20,
+  META_DEBUG_KMS             = 1 << 21,
+  META_DEBUG_SCREEN_CAST     = 1 << 22,
+  META_DEBUG_REMOTE_DESKTOP  = 1 << 23,
+  META_DEBUG_BACKEND         = 1 << 24,
+  META_DEBUG_RENDER          = 1 << 25,
 } MetaDebugTopic;
 
 /**
@@ -128,9 +121,7 @@ typedef enum
 } MetaDebugPaintFlag;
 
 META_EXPORT
-void meta_topic_real      (MetaDebugTopic topic,
-                           const char    *format,
-                           ...) G_GNUC_PRINTF (2, 3);
+gboolean meta_is_topic_enabled (MetaDebugTopic topic);
 
 META_EXPORT
 void meta_add_verbose_topic    (MetaDebugTopic topic);
@@ -177,18 +168,29 @@ GPid meta_show_dialog (const char *type,
 /* To disable verbose mode, we make these functions into no-ops */
 #ifdef WITH_VERBOSE_MODE
 
-#define meta_debug_spew meta_debug_spew_real
-#define meta_verbose    meta_verbose_real
-#define meta_topic      meta_topic_real
+const char * meta_topic_to_string (MetaDebugTopic topic);
+
+#define meta_topic(debug_topic, ...) \
+  G_STMT_START \
+    { \
+      if (meta_is_topic_enabled (debug_topic)) \
+        { \
+          g_autofree char *message = NULL; \
+\
+          message = g_strdup_printf (__VA_ARGS__); \
+          g_message ("%s: %s", meta_topic_to_string (debug_topic), message); \
+        } \
+    } \
+  G_STMT_END
+
+#define meta_verbose(...) meta_topic (META_DEBUG_VERBOSE, __VA_ARGS__)
 
 #else
 
 #  ifdef G_HAVE_ISO_VARARGS
-#    define meta_debug_spew(...)
 #    define meta_verbose(...)
 #    define meta_topic(...)
 #  elif defined(G_HAVE_GNUC_VARARGS)
-#    define meta_debug_spew(format...)
 #    define meta_verbose(format...)
 #    define meta_topic(format...)
 #  else
@@ -216,6 +218,11 @@ void meta_remove_clutter_debug_flags (ClutterDebugFlag     debug_flags,
                                       ClutterDrawDebugFlag draw_flags,
                                       ClutterPickDebugFlag pick_flags);
 
+META_EXPORT
+void meta_get_clutter_debug_flags (ClutterDebugFlag     *debug_flags,
+                                   ClutterDrawDebugFlag *draw_flags,
+                                   ClutterPickDebugFlag *pick_flags);
+
 META_EXPORT
 void meta_add_debug_paint_flag (MetaDebugPaintFlag flag);
 
