@@ -52,7 +52,7 @@ get_random_rect (MetaRectangle *rect)
 static MetaRectangle*
 new_meta_rect (int x, int y, int width, int height)
 {
-  MetaRectangle* temporary;
+  MetaRectangle *temporary;
   temporary = g_new (MetaRectangle, 1);
   temporary->x = x;
   temporary->y = y;
@@ -65,7 +65,7 @@ new_meta_rect (int x, int y, int width, int height)
 static MetaStrut*
 new_meta_strut (int x, int y, int width, int height, int side)
 {
-  MetaStrut* temporary;
+  MetaStrut *temporary;
   temporary = g_new (MetaStrut, 1);
   temporary->rect = meta_rect(x, y, width, height);
   temporary->side = side;
@@ -76,7 +76,7 @@ new_meta_strut (int x, int y, int width, int height, int side)
 static MetaEdge*
 new_screen_edge (int x, int y, int width, int height, int side_type)
 {
-  MetaEdge* temporary;
+  MetaEdge *temporary;
   temporary = g_new (MetaEdge, 1);
   temporary->rect.x = x;
   temporary->rect.y = y;
@@ -91,7 +91,7 @@ new_screen_edge (int x, int y, int width, int height, int side_type)
 static MetaEdge*
 new_monitor_edge (int x, int y, int width, int height, int side_type)
 {
-  MetaEdge* temporary;
+  MetaEdge *temporary;
   temporary = g_new (MetaEdge, 1);
   temporary->rect.x = x;
   temporary->rect.y = y;
@@ -103,6 +103,18 @@ new_monitor_edge (int x, int y, int width, int height, int side_type)
   return temporary;
 }
 
+static void
+test_init_rect (void)
+{
+  MetaRectangle rect;
+
+  rect = META_RECTANGLE_INIT (1, 2, 3, 4);
+  g_assert_cmpint (rect.x, ==, 1);
+  g_assert_cmpint (rect.y, ==, 2);
+  g_assert_cmpint (rect.width, ==, 3);
+  g_assert_cmpint (rect.height, ==, 4);
+}
+
 static void
 test_area (void)
 {
@@ -364,8 +376,8 @@ test_merge_regions (void)
    *   number of those merges that were of the form A is adjacent to B
    */
 
-  GList* region;
-  GList* compare;
+  GList *region;
+  GList *compare;
   int num_contains, num_merged, num_part_contains, num_adjacent;
 
   num_contains = num_merged = num_part_contains = num_adjacent = 0;
@@ -570,8 +582,8 @@ verify_lists_are_equal (GList *code, GList *answer)
 static void
 test_regions_okay (void)
 {
-  GList* region;
-  GList* tmp;
+  GList *region;
+  GList *tmp;
 
   /*************************************************************/
   /* Make sure test region 0 has the right spanning rectangles */
@@ -656,7 +668,7 @@ test_regions_okay (void)
 static void
 test_region_fitting (void)
 {
-  GList* region;
+  GList *region;
   MetaRectangle rect;
 
   /* See test_basic_fitting() for how/why these automated random tests work */
@@ -698,7 +710,7 @@ test_region_fitting (void)
 static void
 test_clamping_to_region (void)
 {
-  GList* region;
+  GList *region;
   MetaRectangle rect;
   MetaRectangle min_size;
   FixedDirections fixed_directions;
@@ -819,7 +831,7 @@ gboolean time_to_print = FALSE;
 static void
 test_clipping_to_region (void)
 {
-  GList* region;
+  GList *region;
   MetaRectangle rect, temp;
   FixedDirections fixed_directions = 0;
   int i;
@@ -879,7 +891,7 @@ test_clipping_to_region (void)
 static void
 test_shoving_into_region (void)
 {
-  GList* region;
+  GList *region;
   MetaRectangle rect, temp;
   FixedDirections fixed_directions = 0;
   int i;
@@ -994,8 +1006,8 @@ verify_edge_lists_are_equal (GList *code, GList *answer)
 static void
 test_find_onscreen_edges (void)
 {
-  GList* edges;
-  GList* tmp;
+  GList *edges;
+  GList *tmp;
 
   int left   = META_DIRECTION_LEFT;
   int right  = META_DIRECTION_RIGHT;
@@ -1125,8 +1137,8 @@ test_find_onscreen_edges (void)
 static void
 test_find_nonintersected_monitor_edges (void)
 {
-  GList* edges;
-  GList* tmp;
+  GList *edges;
+  GList *tmp;
 
   int left   = META_DIRECTION_LEFT;
   int right  = META_DIRECTION_RIGHT;
@@ -1221,7 +1233,7 @@ test_gravity_resize (void)
   temp    = meta_rect ( 50,  300,  20,   5);
   meta_rectangle_resize_with_gravity (&oldrect,
                                       &rect,
-                                      NorthWestGravity,
+                                      META_GRAVITY_NORTH_WEST,
                                       20,
                                       5);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1230,7 +1242,7 @@ test_gravity_resize (void)
   temp = meta_rect (165,  300,  20,   5);
   meta_rectangle_resize_with_gravity (&rect,
                                       &rect,
-                                      NorthGravity,
+                                      META_GRAVITY_NORTH,
                                       20,
                                       5);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1239,7 +1251,7 @@ test_gravity_resize (void)
   temp = meta_rect (280,  300,  20,   5);
   meta_rectangle_resize_with_gravity (&rect,
                                       &rect,
-                                      NorthEastGravity,
+                                      META_GRAVITY_NORTH_EAST,
                                       20,
                                       5);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1248,7 +1260,7 @@ test_gravity_resize (void)
   temp = meta_rect ( 50,  695,  50,   5);
   meta_rectangle_resize_with_gravity (&rect,
                                       &rect,
-                                      SouthWestGravity,
+                                      META_GRAVITY_SOUTH_WEST,
                                       50,
                                       5);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1257,7 +1269,7 @@ test_gravity_resize (void)
   temp = meta_rect (150,  695,  50,   5);
   meta_rectangle_resize_with_gravity (&rect,
                                       &rect,
-                                      SouthGravity,
+                                      META_GRAVITY_SOUTH,
                                       50,
                                       5);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1266,7 +1278,7 @@ test_gravity_resize (void)
   temp = meta_rect (250,  695,  50,   5);
   meta_rectangle_resize_with_gravity (&rect,
                                       &rect,
-                                      SouthEastGravity,
+                                      META_GRAVITY_SOUTH_EAST,
                                       50,
                                       5);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1275,7 +1287,7 @@ test_gravity_resize (void)
   temp = meta_rect (167, 1113, 832,  93);
   meta_rectangle_resize_with_gravity (&rect,
                                       &rect,
-                                      WestGravity,
+                                      META_GRAVITY_WEST,
                                       832,
                                       93);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1284,7 +1296,7 @@ test_gravity_resize (void)
   temp = meta_rect (-131, 1113, 833,  93);
   meta_rectangle_resize_with_gravity (&rect,
                                       &rect,
-                                      CenterGravity,
+                                      META_GRAVITY_CENTER,
                                       832,
                                       93);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1293,7 +1305,7 @@ test_gravity_resize (void)
   temp = meta_rect (270,  994, 430, 212);
   meta_rectangle_resize_with_gravity (&rect,
                                       &rect,
-                                      EastGravity,
+                                      META_GRAVITY_EAST,
                                       430,
                                       211);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1302,7 +1314,7 @@ test_gravity_resize (void)
   temp = meta_rect (300, 1000, 430, 211);
   meta_rectangle_resize_with_gravity (&rect,
                                       &rect,
-                                      StaticGravity,
+                                      META_GRAVITY_STATIC,
                                       430,
                                       211);
   g_assert (meta_rectangle_equal (&rect, &temp));
@@ -1364,6 +1376,7 @@ init_boxes_tests (void)
 {
   init_random_ness ();
 
+  g_test_add_func ("/util/boxes/init", test_init_rect);
   g_test_add_func ("/util/boxes/area", test_area);
   g_test_add_func ("/util/boxes/intersect", test_intersect);
   g_test_add_func ("/util/boxes/equal", test_equal);
