@@ -6,6 +6,7 @@
 
 #include <clutter/clutter.h>
 #include "test-utils.h"
+#include "tests/clutter-test-utils.h"
 
 /* layout actor, by Lucas Rocha */
 
@@ -497,7 +498,7 @@ keypress_cb (ClutterActor *actor,
   switch (clutter_event_get_key_symbol (event))
     {
     case CLUTTER_KEY_q:
-      clutter_main_quit ();
+      clutter_test_quit ();
       break;
 
     case CLUTTER_KEY_a:
@@ -599,13 +600,12 @@ test_layout_main (int argc, char *argv[])
   gint i, size;
   GError *error = NULL;
 
-  if (clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS)
-    return 1;
+  clutter_test_init (&argc, &argv);
 
-  stage = clutter_stage_new ();
+  stage = clutter_test_get_stage ();
   clutter_actor_set_size (stage, 800, 600);
   clutter_stage_set_title (CLUTTER_STAGE (stage), "Layout");
-  g_signal_connect (stage, "destroy", G_CALLBACK (clutter_main_quit), NULL);
+  g_signal_connect (stage, "destroy", G_CALLBACK (clutter_test_quit), NULL);
 
   main_timeline = clutter_timeline_new_for_actor (stage, 2000);
   clutter_timeline_set_repeat_count (main_timeline, -1);
@@ -670,7 +670,7 @@ test_layout_main (int argc, char *argv[])
 
   clutter_actor_show (stage);
 
-  clutter_main ();
+  clutter_test_main ();
 
   g_object_unref (main_timeline);
 
