@@ -25,43 +25,45 @@
 #include "backends/meta-monitor-config-store.h"
 #include "backends/meta-monitor-config-manager.h"
 #include "backends/meta-monitor-manager-private.h"
-#include "tests/monitor-test-utils.h"
+#include "tests/meta-monitor-test-utils.h"
+#include "tests/unit-tests.h"
 
 #define MAX_N_MONITORS 10
 #define MAX_N_LOGICAL_MONITORS 10
 #define MAX_N_CONFIGURATIONS 10
 
-typedef struct _MonitorTestCaseMonitorMode
+typedef struct _MonitorStoreTestCaseMonitorMode
 {
   int width;
   int height;
   float refresh_rate;
-} MonitorTestCaseMonitorMode;
+  MetaCrtcModeFlag flags;
+} MonitorStoreTestCaseMonitorMode;
 
-typedef struct _MonitorTestCaseMonitor
+typedef struct _MonitorStoreTestCaseMonitor
 {
   const char *connector;
   const char *vendor;
   const char *product;
   const char *serial;
-  MonitorTestCaseMonitorMode mode;
+  MonitorStoreTestCaseMonitorMode mode;
   gboolean is_underscanning;
-} MonitorTestCaseMonitor;
+} MonitorStoreTestCaseMonitor;
 
-typedef struct _MonitorTestCaseLogicalMonitor
+typedef struct _MonitorStoreTestCaseLogicalMonitor
 {
   MetaRectangle layout;
-  int scale;
+  float scale;
   MetaMonitorTransform transform;
   gboolean is_primary;
   gboolean is_presentation;
-  MonitorTestCaseMonitor monitors[MAX_N_MONITORS];
+  MonitorStoreTestCaseMonitor monitors[MAX_N_MONITORS];
   int n_monitors;
-} MonitorTestCaseLogicalMonitor;
+} MonitorStoreTestCaseLogicalMonitor;
 
 typedef struct _MonitorStoreTestConfiguration
 {
-  MonitorTestCaseLogicalMonitor logical_monitors[MAX_N_LOGICAL_MONITORS];
+  MonitorStoreTestCaseLogicalMonitor logical_monitors[MAX_N_LOGICAL_MONITORS];
   int n_logical_monitors;
 } MonitorStoreTestConfiguration;
 
@@ -86,7 +88,7 @@ create_config_key_from_expect (MonitorStoreTestConfiguration *expect_config)
       for (j = 0; j < expect_config->logical_monitors[i].n_monitors; j++)
         {
           MetaMonitorSpec *monitor_spec;
-          MonitorTestCaseMonitor *test_monitor =
+          MonitorStoreTestCaseMonitor *test_monitor =
             &expect_config->logical_monitors[i].monitors[j];
 
           monitor_spec = g_new0 (MetaMonitorSpec, 1);
@@ -114,8 +116,8 @@ create_config_key_from_expect (MonitorStoreTestConfiguration *expect_config)
 }
 
 static void
-check_monitor_configuration (MetaMonitorConfigStore        *config_store,
-                             MonitorStoreTestConfiguration *config_expect)
+check_monitor_store_configuration (MetaMonitorConfigStore        *config_store,
+                                   MonitorStoreTestConfiguration *config_expect)
 {
   MetaMonitorsConfigKey *config_key;
   MetaMonitorsConfig *config;
@@ -141,9 +143,9 @@ check_monitor_configuration (MetaMonitorConfigStore        *config_store,
 
       g_assert (meta_rectangle_equal (&logical_monitor_config->layout,
                                       &config_expect->logical_monitors[i].layout));
-      g_assert_cmpint (logical_monitor_config->scale,
-                       ==,
-                       config_expect->logical_monitors[i].scale);
+      g_assert_cmpfloat (logical_monitor_config->scale,
+                         ==,
+                         config_expect->logical_monitors[i].scale);
       g_assert_cmpint (logical_monitor_config->transform,
                        ==,
                        config_expect->logical_monitors[i].transform);
@@ -163,7 +165,7 @@ check_monitor_configuration (MetaMonitorConfigStore        *config_store,
            k = k->next, j++)
         {
           MetaMonitorConfig *monitor_config = k->data;
-          MonitorTestCaseMonitor *test_monitor =
+          MonitorStoreTestCaseMonitor *test_monitor =
             &config_expect->logical_monitors[i].monitors[j];
 
           g_assert_cmpstr (monitor_config->monitor_spec->connector,
@@ -188,6 +190,9 @@ check_monitor_configuration (MetaMonitorConfigStore        *config_store,
           g_assert_cmpfloat (monitor_config->mode_spec->refresh_rate,
                              ==,
                              test_monitor->mode.refresh_rate);
+          g_assert_cmpint (monitor_config->mode_spec->flags,
+                           ==,
+                           test_monitor->mode.flags);
           g_assert_cmpint (monitor_config->enable_underscanning,
                            ==,
                            test_monitor->is_underscanning);
@@ -196,7 +201,7 @@ check_monitor_configuration (MetaMonitorConfigStore        *config_store,
 }
 
 static void
-check_monitor_configurations (MonitorStoreTestExpect *expect)
+check_monitor_store_configurations (MonitorStoreTestExpect *expect)
 {
   MetaBackend *backend = meta_get_backend ();
   MetaMonitorManager *monitor_manager =
@@ -211,7 +216,7 @@ check_monitor_configurations (MonitorStoreTestExpect *expect)
                    expect->n_configurations);
 
   for (i = 0; i < expect->n_configurations; i++)
-    check_monitor_configuration (config_store, &expect->configurations[i]);
+    check_monitor_store_configuration (config_store, &expect->configurations[i]);
 }
 
 static void
@@ -253,15 +258,9 @@ meta_test_monitor_store_single (void)
     .n_configurations = 1
   };
 
-  if (!is_using_monitor_config_manager ())
-    {
-      g_test_skip ("Not using MetaMonitorConfigManager");
-      return;
-    }
+  meta_set_custom_monitor_config (test_context, "single.xml");
 
-  set_custom_monitor_config ("single.xml");
-
-  check_monitor_configurations (&expect);
+  check_monitor_store_configurations (&expect);
 }
 
 static void
@@ -328,15 +327,9 @@ meta_test_monitor_store_vertical (void)
     .n_configurations = 1
   };
 
-  if (!is_using_monitor_config_manager ())
-    {
-      g_test_skip ("Not using MetaMonitorConfigManager");
-      return;
-    }
+  meta_set_custom_monitor_config (test_context, "vertical.xml");
 
-  set_custom_monitor_config ("vertical.xml");
-
-  check_monitor_configurations (&expect);
+  check_monitor_store_configurations (&expect);
 }
 
 static void
@@ -403,15 +396,9 @@ meta_test_monitor_store_primary (void)
     .n_configurations = 1
   };
 
-  if (!is_using_monitor_config_manager ())
-    {
-      g_test_skip ("Not using MetaMonitorConfigManager");
-      return;
-    }
+  meta_set_custom_monitor_config (test_context, "primary.xml");
 
-  set_custom_monitor_config ("primary.xml");
-
-  check_monitor_configurations (&expect);
+  check_monitor_store_configurations (&expect);
 }
 
 static void
@@ -454,15 +441,9 @@ meta_test_monitor_store_underscanning (void)
     .n_configurations = 1
   };
 
-  if (!is_using_monitor_config_manager ())
-    {
-      g_test_skip ("Not using MetaMonitorConfigManager");
-      return;
-    }
+  meta_set_custom_monitor_config (test_context, "underscanning.xml");
 
-  set_custom_monitor_config ("underscanning.xml");
-
-  check_monitor_configurations (&expect);
+  check_monitor_store_configurations (&expect);
 }
 
 static void
@@ -504,21 +485,115 @@ meta_test_monitor_store_scale (void)
     .n_configurations = 1
   };
 
-  if (!is_using_monitor_config_manager ())
+  if (!meta_is_stage_views_enabled ())
     {
-      g_test_skip ("Not using MetaMonitorConfigManager");
+      g_test_skip ("Not using stage views");
       return;
     }
 
+  meta_set_custom_monitor_config (test_context, "scale.xml");
+
+  check_monitor_store_configurations (&expect);
+}
+
+static void
+meta_test_monitor_store_fractional_scale (void)
+{
+  MonitorStoreTestExpect expect = {
+    .configurations = {
+      {
+        .logical_monitors = {
+          {
+            .layout = {
+              .x = 0,
+              .y = 0,
+              .width = 800,
+              .height = 600
+            },
+            .scale = 1.5,
+            .is_primary = TRUE,
+            .is_presentation = FALSE,
+            .monitors = {
+              {
+                .connector = "DP-1",
+                .vendor = "MetaProduct's Inc.",
+                .product = "MetaMonitor",
+                .serial = "0x123456",
+                .mode = {
+                  .width = 1200,
+                  .height = 900,
+                  .refresh_rate = 60.000495910644531
+                }
+              }
+            },
+            .n_monitors = 1,
+          }
+        },
+        .n_logical_monitors = 1
+      }
+    },
+    .n_configurations = 1
+  };
+
   if (!meta_is_stage_views_enabled ())
     {
       g_test_skip ("Not using stage views");
       return;
     }
 
-  set_custom_monitor_config ("scale.xml");
+  meta_set_custom_monitor_config (test_context, "fractional-scale.xml");
 
-  check_monitor_configurations (&expect);
+  check_monitor_store_configurations (&expect);
+}
+
+static void
+meta_test_monitor_store_high_precision_fractional_scale (void)
+{
+  MonitorStoreTestExpect expect = {
+    .configurations = {
+      {
+        .logical_monitors = {
+          {
+            .layout = {
+              .x = 0,
+              .y = 0,
+              .width = 744,
+              .height = 558
+            },
+            .scale = 1.3763440847396851,
+            .is_primary = TRUE,
+            .is_presentation = FALSE,
+            .monitors = {
+              {
+                .connector = "DP-1",
+                .vendor = "MetaProduct's Inc.",
+                .product = "MetaMonitor",
+                .serial = "0x123456",
+                .mode = {
+                  .width = 1024,
+                  .height = 768,
+                  .refresh_rate = 60.000495910644531
+                }
+              }
+            },
+            .n_monitors = 1,
+          }
+        },
+        .n_logical_monitors = 1
+      }
+    },
+    .n_configurations = 1
+  };
+
+  if (!meta_is_stage_views_enabled ())
+    {
+      g_test_skip ("Not using stage views");
+      return;
+    }
+
+  meta_set_custom_monitor_config (test_context, "high-precision-fractional-scale.xml");
+
+  check_monitor_store_configurations (&expect);
 }
 
 static void
@@ -570,15 +645,9 @@ meta_test_monitor_store_mirrored (void)
     .n_configurations = 1
   };
 
-  if (!is_using_monitor_config_manager ())
-    {
-      g_test_skip ("Not using MetaMonitorConfigManager");
-      return;
-    }
+  meta_set_custom_monitor_config (test_context, "mirrored.xml");
 
-  set_custom_monitor_config ("mirrored.xml");
-
-  check_monitor_configurations (&expect);
+  check_monitor_store_configurations (&expect);
 }
 
 static void
@@ -647,15 +716,9 @@ meta_test_monitor_store_first_rotated (void)
     .n_configurations = 1
   };
 
-  if (!is_using_monitor_config_manager ())
-    {
-      g_test_skip ("Not using MetaMonitorConfigManager");
-      return;
-    }
+  meta_set_custom_monitor_config (test_context, "first-rotated.xml");
 
-  set_custom_monitor_config ("first-rotated.xml");
-
-  check_monitor_configurations (&expect);
+  check_monitor_store_configurations (&expect);
 }
 
 static void
@@ -724,15 +787,219 @@ meta_test_monitor_store_second_rotated (void)
     .n_configurations = 1
   };
 
-  if (!is_using_monitor_config_manager ())
-    {
-      g_test_skip ("Not using MetaMonitorConfigManager");
-      return;
-    }
+  meta_set_custom_monitor_config (test_context, "second-rotated.xml");
 
-  set_custom_monitor_config ("second-rotated.xml");
+  check_monitor_store_configurations (&expect);
+}
 
-  check_monitor_configurations (&expect);
+static void
+meta_test_monitor_store_interlaced (void)
+{
+  MonitorStoreTestExpect expect = {
+    .configurations = {
+      {
+        .logical_monitors = {
+          {
+            .layout = {
+              .x = 0,
+              .y = 0,
+              .width = 1024,
+              .height = 768
+            },
+            .scale = 1,
+            .is_primary = TRUE,
+            .is_presentation = FALSE,
+            .monitors = {
+              {
+                .connector = "DP-1",
+                .vendor = "MetaProduct's Inc.",
+                .product = "MetaMonitor",
+                .serial = "0x123456",
+                .mode = {
+                  .width = 1024,
+                  .height = 768,
+                  .refresh_rate = 60.000495910644531,
+                  .flags = META_CRTC_MODE_FLAG_INTERLACE,
+                }
+              }
+            },
+            .n_monitors = 1,
+          },
+        },
+        .n_logical_monitors = 1
+      }
+    },
+    .n_configurations = 1
+  };
+
+  meta_set_custom_monitor_config (test_context, "interlaced.xml");
+
+  check_monitor_store_configurations (&expect);
+}
+
+static void
+meta_test_monitor_store_unknown_elements (void)
+{
+  MonitorStoreTestExpect expect = {
+    .configurations = {
+      {
+        .logical_monitors = {
+          {
+            .layout = {
+              .x = 0,
+              .y = 0,
+              .width = 1920,
+              .height = 1080
+            },
+            .scale = 1,
+            .is_primary = TRUE,
+            .is_presentation = FALSE,
+            .monitors = {
+              {
+                .connector = "DP-1",
+                .vendor = "MetaProduct's Inc.",
+                .product = "MetaMonitor",
+                .serial = "0x123456",
+                .mode = {
+                  .width = 1920,
+                  .height = 1080,
+                  .refresh_rate = 60.000495910644531
+                }
+              }
+            },
+            .n_monitors = 1,
+          }
+        },
+        .n_logical_monitors = 1
+      }
+    },
+    .n_configurations = 1
+  };
+
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+                         "Unknown element <unknownundermonitors> "
+                         "under <monitors>, ignoring");
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+                         "Unknown element <unknownunderconfiguration> "
+                         "under <configuration>, ignoring");
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+                         "Unknown element <unknownunderlogicalmonitor> "
+                         "under <logicalmonitor>, ignoring");
+  meta_set_custom_monitor_config (test_context, "unknown-elements.xml");
+  g_test_assert_expected_messages ();
+
+  check_monitor_store_configurations (&expect);
+}
+
+static void
+meta_test_monitor_store_policy_not_allowed (void)
+{
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+                         "*Policy can only be defined in system level "
+                         "configurations*");
+  meta_set_custom_monitor_config (test_context, "policy.xml");
+  g_test_assert_expected_messages ();
+}
+
+static void
+meta_test_monitor_store_policy (void)
+{
+  MetaBackend *backend = meta_get_backend ();
+  MetaMonitorManager *monitor_manager =
+    meta_backend_get_monitor_manager (backend);
+  MetaMonitorConfigManager *config_manager = monitor_manager->config_manager;
+  MetaMonitorConfigStore *config_store =
+    meta_monitor_config_manager_get_store (config_manager);
+  GList *stores_policy;
+
+  meta_set_custom_monitor_system_config (test_context, "policy.xml");
+  stores_policy = meta_monitor_config_store_get_stores_policy (config_store);
+  g_assert_cmpuint (g_list_length (stores_policy), ==, 1);
+  g_assert_cmpint (GPOINTER_TO_INT (stores_policy->data),
+                   ==,
+                   META_CONFIG_STORE_SYSTEM);
+}
+
+static void
+meta_test_monitor_store_policy_empty (void)
+{
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+                         "*Invalid store*");
+  meta_set_custom_monitor_system_config (test_context, "policy-empty.xml");
+  g_test_assert_expected_messages ();
+}
+
+static void
+meta_test_monitor_store_policy_duplicate (void)
+{
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+                         "*Multiple identical stores*");
+  meta_set_custom_monitor_system_config (test_context, "policy-duplicate.xml");
+  g_test_assert_expected_messages ();
+}
+
+static void
+meta_test_monitor_store_policy_invalid (void)
+{
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+                         "*Invalid store*");
+  meta_set_custom_monitor_system_config (test_context, "policy-invalid.xml");
+  g_test_assert_expected_messages ();
+}
+
+static void
+meta_test_monitor_store_policy_multiple (void)
+{
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+                         "*Multiple stores elements under policy*");
+  meta_set_custom_monitor_system_config (test_context, "policy-multiple.xml");
+  g_test_assert_expected_messages ();
+}
+
+static void
+meta_test_monitor_store_policy_dbus (void)
+{
+  MetaBackend *backend = meta_context_get_backend (test_context);
+  MetaMonitorManager *monitor_manager =
+    meta_backend_get_monitor_manager (backend);
+  MetaMonitorConfigManager *config_manager =
+    meta_monitor_manager_get_config_manager (monitor_manager);
+  MetaMonitorConfigStore *config_store =
+    meta_monitor_config_manager_get_store (config_manager);
+  const MetaMonitorConfigPolicy *policy;
+
+  policy = meta_monitor_config_store_get_policy (config_store);
+  g_assert_nonnull (policy);
+  g_assert_cmpint (policy->enable_dbus, ==, TRUE);
+
+  meta_set_custom_monitor_system_config (test_context, "policy-dbus.xml");
+
+  policy = meta_monitor_config_store_get_policy (config_store);
+  g_assert_nonnull (policy);
+  g_assert_cmpint (policy->enable_dbus, ==, FALSE);
+}
+
+static void
+meta_test_monitor_store_policy_dbus_invalid (void)
+{
+  MetaBackend *backend = meta_context_get_backend (test_context);
+  MetaMonitorManager *monitor_manager =
+    meta_backend_get_monitor_manager (backend);
+  MetaMonitorConfigManager *config_manager =
+    meta_monitor_manager_get_config_manager (monitor_manager);
+  MetaMonitorConfigStore *config_store =
+    meta_monitor_config_manager_get_store (config_manager);
+  const MetaMonitorConfigPolicy *policy;
+
+  g_test_expect_message (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+                         "*Multiple dbus elements under policy*");
+  meta_set_custom_monitor_system_config (test_context,
+                                         "policy-dbus-invalid.xml");
+  g_test_assert_expected_messages ();
+
+  policy = meta_monitor_config_store_get_policy (config_store);
+  g_assert_nonnull (policy);
+  g_assert_cmpint (policy->enable_dbus, ==, FALSE);
 }
 
 void
@@ -748,10 +1015,34 @@ init_monitor_store_tests (void)
                    meta_test_monitor_store_underscanning);
   g_test_add_func ("/backends/monitor-store/scale",
                    meta_test_monitor_store_scale);
+  g_test_add_func ("/backends/monitor-store/fractional-scale",
+                   meta_test_monitor_store_fractional_scale);
+  g_test_add_func ("/backends/monitor-store/high-precision-fractional-scale",
+                   meta_test_monitor_store_high_precision_fractional_scale);
   g_test_add_func ("/backends/monitor-store/mirrored",
                    meta_test_monitor_store_mirrored);
   g_test_add_func ("/backends/monitor-store/first-rotated",
                    meta_test_monitor_store_first_rotated);
   g_test_add_func ("/backends/monitor-store/second-rotated",
                    meta_test_monitor_store_second_rotated);
+  g_test_add_func ("/backends/monitor-store/interlaced",
+                   meta_test_monitor_store_interlaced);
+  g_test_add_func ("/backends/monitor-store/unknown-elements",
+                   meta_test_monitor_store_unknown_elements);
+  g_test_add_func ("/backends/monitor-store/policy-not-allowed",
+                   meta_test_monitor_store_policy_not_allowed);
+  g_test_add_func ("/backends/monitor-store/policy",
+                   meta_test_monitor_store_policy);
+  g_test_add_func ("/backends/monitor-store/policy-empty",
+                   meta_test_monitor_store_policy_empty);
+  g_test_add_func ("/backends/monitor-store/policy-duplicate",
+                   meta_test_monitor_store_policy_duplicate);
+  g_test_add_func ("/backends/monitor-store/policy-invalid",
+                   meta_test_monitor_store_policy_invalid);
+  g_test_add_func ("/backends/monitor-store/policy-multiple",
+                   meta_test_monitor_store_policy_multiple);
+  g_test_add_func ("/backends/monitor-store/dbus",
+                   meta_test_monitor_store_policy_dbus);
+  g_test_add_func ("/backends/monitor-store/dbus-invalid",
+                   meta_test_monitor_store_policy_dbus_invalid);
 }
