@@ -43,5 +43,6 @@ main (int    argc,
 
   init_tests ();
 
-  return meta_context_test_run_tests (META_CONTEXT_TEST (context));
+  return meta_context_test_run_tests (META_CONTEXT_TEST (context),
+                                      META_TEST_RUN_FLAG_NONE);
 }
