@@ -108,7 +108,7 @@ meta_wayland_tablet_manager_new (MetaWaylandCompositor *compositor)
 {
   MetaWaylandTabletManager *tablet_manager;
 
-  tablet_manager = g_slice_new0 (MetaWaylandTabletManager);
+  tablet_manager = g_new0 (MetaWaylandTabletManager, 1);
   tablet_manager->compositor = compositor;
   tablet_manager->wl_display = compositor->wayland_display;
   tablet_manager->seats = g_hash_table_new_full (NULL, NULL, NULL,
@@ -132,7 +132,7 @@ void
 meta_wayland_tablet_manager_free (MetaWaylandTabletManager *tablet_manager)
 {
   g_hash_table_destroy (tablet_manager->seats);
-  g_slice_free (MetaWaylandTabletManager, tablet_manager);
+  g_free (tablet_manager);
 }
 
 static MetaWaylandTabletSeat *
@@ -242,30 +242,3 @@ meta_wayland_tablet_manager_ensure_seat (MetaWaylandTabletManager *manager,
 
   return tablet_seat;
 }
-
-void
-meta_wayland_tablet_manager_update_cursor_position (MetaWaylandTabletManager *manager,
-                                                    const ClutterEvent       *event)
-{
-  MetaWaylandTabletSeat *tablet_seat = NULL;
-  MetaWaylandTabletTool *tool = NULL;
-  ClutterInputDeviceTool *device_tool;
-  ClutterInputDevice *device;
-
-  device = clutter_event_get_source_device (event);
-  device_tool = clutter_event_get_device_tool (event);
-
-  if (device)
-    tablet_seat = meta_wayland_tablet_manager_lookup_seat (manager, device);
-
-  if (tablet_seat && device_tool)
-    tool = meta_wayland_tablet_seat_lookup_tool (tablet_seat, device_tool);
-
-  if (tool)
-    {
-      gfloat new_x, new_y;
-
-      clutter_event_get_coords (event, &new_x, &new_y);
-      meta_wayland_tablet_tool_set_cursor_position (tool, new_x, new_y);
-    }
-}
